#!/usr/bin/perl
#$Id: CheckDaemon.pl,v 1.5 2005/01/10 12:07:04 tortone Exp $
#Copyright (c) 2003 Istituto Nazionale di Fisica Nucleare
#For license conditions see http://www.cnaf.infn.it/license.html
#(c) by Gennaro Tortone <gennaro.tortone@na.infn.it>, INFN-NAPOLI

use strict;
use POSIX;
use Getopt::Long;

#
# FUNCTIONS
#

sub trim {
        my(@out)=@_;
        for (@out) {
		chomp;
                s/^\s+//;
                s/\s+$//;
        }
        return wantarray ? @out : $out[0];
}

sub round {
       my($num)=@_;
       my $mod=$num-floor($num);
       return  (($mod < 0.5) ? floor($num) : ceil($num));
}

#
# MAIN
#

my $debug    =  0;
my $interval = 60;

# check arguments
my ($config_file,$help);
GetOptions( "cfg=s" => \$config_file,
            "help" => \$help);

if (!defined($config_file)) {
	$config_file = "/opt/gridice/monitoring/etc/gridice-role.cfg";
}

if(defined($help)) {
	print "usage: CheckDaemon --cfg <gridice-role.cfg>\n";
	exit 0;
}

open(CONFIG, "< $config_file")
	or die "Couldn't open $config_file for reading: $!\n";

my %daemons;
while (<CONFIG>) {

	# discard comments and blank lines
        next if (/^#/ or /^\s+/);

        # parse single-instance sections
        if ( $_ =~ /^\[(.+)\]/ ) {
		my $role = $1;
		$debug && print "start section: $role\n";
		while(<CONFIG>) {
			next if (/^#/ or /^\s+/);

			last if /^\[$role\s+end\]/;

			$_ =~ /([^ \t]+)\s+([^ \t]+)\s+(\w+)?/;
			my $procname = $1;
			my $procinstance = $2;
			my $isapp = $3;

			$procname = trim($procname);
			$procinstance = trim($procinstance);
			$isapp = trim($isapp);

			$daemons{$role}{$procname}{"matchingstring"} = $procinstance;
			$daemons{$role}{$procname}{"found"} = 0;			

			if (lc($isapp) eq "app") {
				$daemons{$role}{$procname}{"isapp"} = 1;
			} else {
				$daemons{$role}{$procname}{"isapp"} = 0;
			} 

			$debug && print "procname = $procname - procinstance = $procinstance \n";	
		}
        }
}

#open pipe to ps
open PS, "/bin/ps -H --cols 1000 -eo user,pid,state,pcpu,pmem,etime,bsdtime,command |";

#loop for each file
while(<PS>) {
	#take off newline character
	chomp();
	
	my($user,$pid,$state,$pcpu,$pmem,$etime,$bsdtime,$command) = split /\s+/, $_, 8;
	#($command) = split /\s+/, $command;
	
	$debug && print $pid . " " . $command . " " . $bsdtime . "\n";

	my ($key_role, $key_daemon);
	foreach $key_role ( keys %daemons ) {
		foreach $key_daemon ( keys %{ $daemons{$key_role} } ) {
					
			if ($command =~ /$daemons{$key_role}{$key_daemon}{"matchingstring"}/) {
				# ps line matches regexp in gridice-role.cfg

				if (!defined($daemons{$key_role}{$key_daemon}{"command"})) {
					my $tmpComm = trim($command);
					$tmpComm = substr($tmpComm,0,125);
					$daemons{$key_role}{$key_daemon}{"command"} = $tmpComm;
				}

				# mark entry as valid
				$daemons{$key_role}{$key_daemon}{"found"} = 1;

				# Status
				if (!defined($daemons{$key_role}{$key_daemon}{"status"})) {
					$daemons{$key_role}{$key_daemon}{"status"} = $state; 
				} else {
					if ($daemons{$key_role}{$key_daemon}{"status"} !~ /$state/) {
						$daemons{$key_role}{$key_daemon}{"status"} = 
							$daemons{$key_role}{$key_daemon}{"status"} . "," . $state;
					}
				}

				# NumberOfInstances
				if (!defined($daemons{$key_role}{$key_daemon}{"numofinstances"})) {
					$daemons{$key_role}{$key_daemon}{"numofinstances"} = 1;
				} else {
					$daemons{$key_role}{$key_daemon}{"numofinstances"} += 1;
				}

				# FirstStarted
					# convert etime in seconds
                                my $etimeLen=length($etime);
			        my $etimeS=0; my $d=0; my $h=0; my $m=0; my $s=0;
				if ($etimeLen==5) {
					 my($m,$s) = ($etime =~ /(\d+):(\d+)/);
					 $etimeS= $m*60+$s;
					 #$debug && print "etime:         m$m s$s etime$etime-$etimeLen\n";
				} elsif ($etimeLen==8) {
					 my($h,$m,$s) = ($etime =~ /(\d+):(\d+):(\d+)/);
                                         $etimeS= $h*3600+$m*60+$s;	
					 #$debug && print "etime:     h$h m$m s$s etime$etime-$etimeLen\n";
                                } elsif ( ($etimeLen==11) or ($etimeLen==10) ) {
					 my ($d,$h,$m,$s) = ($etime =~ /(\d+)-(\d+):(\d+):(\d+)/);
                                         $etimeS= $d*86400+$h*3600+$m*60+$s;
					 #$debug && print "etime: d$d h$h m$m s$s etime$etime-$etimeLen\n";
				} else {
					 #$debug && print "wrong etime";
				}

				if (!defined($daemons{$key_role}{$key_daemon}{"firststarted"})) {
					$daemons{$key_role}{$key_daemon}{"firststarted"} = $etimeS
				}
				
				# LastStarted
				$daemons{$key_role}{$key_daemon}{"laststarted"} = $etimeS;

				# CPUUsageAll
				if (!defined($daemons{$key_role}{$key_daemon}{"cpuusageall"})) {
					$daemons{$key_role}{$key_daemon}{"cpuusageall"} = round($pcpu); 
				} else {
					$daemons{$key_role}{$key_daemon}{"cpuusageall"} += round($pcpu);
				}
			
				# CPUUsageOneMax
				if (!defined($daemons{$key_role}{$key_daemon}{"cpuusageonemax"}) or
					$pcpu > $daemons{$key_role}{$key_daemon}{"cpuusageonemax"}) {
						$daemons{$key_role}{$key_daemon}{"cpuusageonemax"} = round($pcpu);
				}

				# MemUsageOneMax
				if (!defined($daemons{$key_role}{$key_daemon}{"memusageonemax"}) or
					$pmem > $daemons{$key_role}{$key_daemon}{"memusageonemax"}) {
						$daemons{$key_role}{$key_daemon}{"memusageonemax"} = round($pmem);
				}

				# MemUsageAll, to be used to compute Avg; not to be published
				if (!defined($daemons{$key_role}{$key_daemon}{"memusageall"})) {
					$daemons{$key_role}{$key_daemon}{"memusageall"} = round($pmem);
		                } else {
				 	$daemons{$key_role}{$key_daemon}{"memusageall"} += round($pmem);		
				}

				# TimeUsageOneMax
                                	# convert into seconds
				my($m,$s) = ($bsdtime =~ /(\d+):(\d+)/);
                                my $timeUsage=$m*60+$s;
					# if first instance
				if (!defined($daemons{$key_role}{$key_daemon}{"timeusageonemax"})) {
					$daemons{$key_role}{$key_daemon}{"timeusageonemax"} = $timeUsage;
				} else {
                                        # if second or later instance, check which Max
					if ($timeUsage > $daemons{$key_role}{$key_daemon}{"timeusageonemax"}) {
						$daemons{$key_role}{$key_daemon}{"timeusageonemax"}=$timeUsage;
					}  
				}	
				
				# TimeUsageAll
				if (!defined($daemons{$key_role}{$key_daemon}{"timeusageall"})) {
					$daemons{$key_role}{$key_daemon}{"timeusageall"} = $timeUsage;
				} else {
					$daemons{$key_role}{$key_daemon}{"timeusageall"} += $timeUsage;
				}

			} # foreach $key_daemon

			if ($daemons{$key_role}{$key_daemon}{"numofinstances"} > 0) {
				$daemons{$key_role}{$key_daemon}{"memusageavg"} = 
					round($daemons{$key_role}{$key_daemon}{"memusageall"} / 
						$daemons{$key_role}{$key_daemon}{"numofinstances"});
			}
		} # foreach $key_role
	}
}
close PS;



foreach my $key_role ( sort keys %daemons ) {
	foreach my $key_daemon ( sort keys %{ $daemons{$key_role} } ) {
		if (!$daemons{$key_role}{$key_daemon}{"found"}) {
			$daemons{$key_role}{$key_daemon}{"command"} = "null";
			$daemons{$key_role}{$key_daemon}{"cpuusageall"} = 0;
			$daemons{$key_role}{$key_daemon}{"cpuusageonemax"} = 0;
			$daemons{$key_role}{$key_daemon}{"firststarted"} = "0";
			$daemons{$key_role}{$key_daemon}{"laststarted"} = "0";
			$daemons{$key_role}{$key_daemon}{"memusageavg"} = 0;
			$daemons{$key_role}{$key_daemon}{"memusageonemax"} = 0;
			$daemons{$key_role}{$key_daemon}{"numofinstances"} = 0;
			$daemons{$key_role}{$key_daemon}{"status"} = "OFF";
			$daemons{$key_role}{$key_daemon}{"timeusageall"} = 0;
			$daemons{$key_role}{$key_daemon}{"timeusageonemax"} = 0;
		}
	}
}

if ($debug) {
	foreach my $key_role ( sort keys %daemons ) {
		foreach my $key_daemon ( sort keys %{ $daemons{$key_role} } ) {
			print "$key_role - $key_daemon" . "\n";
				foreach my $key_attr ( sort keys %{ $daemons{$key_role}{$key_daemon}} ) {
					#next if ( $key_attr eq "matchingstring" or $key_attr eq "found" ); # hidden fields
					print "ATTRIBUTE => NAME=$key_attr VALUE=$daemons{$key_role}{$key_daemon}{$key_attr}\n";
				}
			print "\n";
		}
		print "\n";
	}
}

my $fmonstring = "";
foreach my $key_role ( sort keys %daemons ) {
	foreach my $key_daemon ( sort keys %{ $daemons{$key_role} } ) {
		next if ( ($daemons{$key_role}{$key_daemon}{"found"} eq 0) and ($daemons{$key_role}{$key_daemon}{"isapp"} eq 1) );
		$fmonstring = $fmonstring . "$key_role|$key_daemon|";
		foreach my $key_attr ( sort keys %{ $daemons{$key_role}{$key_daemon}} ) {
			next if ( $key_attr eq "matchingstring" or $key_attr eq "found" or $key_attr eq "memusageall" or $key_attr eq "isapp");
			$debug && print $key_daemon." ".$key_attr."\n";
			$fmonstring = $fmonstring . $daemons{$key_role}{$key_daemon}{$key_attr} . "*";
		}
		$fmonstring =~ s/\*$//;
		$fmonstring = $fmonstring . "%";
	}
}
$fmonstring =~ s/%$//;

print $fmonstring . "\n";

exit 0;
