#!/usr/bin/perl

#Copyright (c) 2004 Istituto Nazionale di Fisica Nucleare
#For license conditions see http://www.cnaf.infn.it/license.html
#(c) by Guido Cuscela <g.cuscela@ba.infn.it> INFN-BARI and
#Giacinto Donvito <giacinto.donvito@ba.infn.it>, INFN-BARI

#use Data::Dumper;
use Getopt::Long;

my ($lrms,$log_path,$interval,$help);
GetOptions( "lrms:s" => \$lrms,
		"lrms-path:s" => \$log_path,
		"interval:s" => \$interval,
		"hide-subject:s" => \$sub_par,
		"jm-dir:s" => \$jm_dir,
		"no-update" => \$update,
		"d" => \$debug,
		"help" => \$help
	  );

if(defined($help)) {
	print "usage: perl $0 --lrms=pbs/lsf --lrms-path=<installation-path-of-batch-system-log> --interval=<monitoring interval(hours)> --hide-subject=<yes|no> --jm-dir=<job monitoring temporary dir> --no-update -d --help\n";
	print "unknown\$BAD_CONFIGURATION";
	exit 0;
}
elsif (!$lrms) {
	print "unknown\$BAD_CONFIGURATION";
	exit 0;
}
elsif (!$log_path) {
	if($lrms =~ /pbs/i and (-e "/var/spool/pbs/server_priv/accounting")){
		$log_path="/var/spool/pbs/server_priv/accounting";
	}
	elsif($lrms =~ /pbs/i){
		print "$lrms\$BAD_CONFIGURATION";
		exit 0;
	}
	else{
		print "$lrms\$BAD_CONFIGURATION";
		exit 0;
	}
}
else{
	unless($lrms =~ /\bpbs\b|\blsf\b/i){
		print "$lrms\$BAD_CONFIGURATION";
		exit 0;
	}
	elsif($lrms =~ /pbs/i){
		$lrms = "pbs";
	}
	else{$lrms = "lsf";}
}
if (!$jm_dir) {
        print "$lrms\$BAD_CONFIGURATION";
        exit 0;
}
else{
        my @check_jmdir=`ls $jm_dir 2>/dev/null`;
        $check_jmdir=1;
        $check_jmdir=0 if(!grep /new/,@check_jmdir);
        $check_jmdir=0 if(!grep /ended/,@check_jmdir);
        $check_jmdir=0 if(!grep /processed/,@check_jmdir);
        $check_jmdir=0 if(!grep /subject/,@check_jmdir);
        if($check_jmdir == 0){
                print "$lrms\$BAD_CONFIGURATION";
                exit 0;
        }
}
if (!($interval=~/\b\d+\b/)) {
	$interval=2;
}
if (!($sub_par=~/\byes\b/i)) {
        $sub_par="no";
}
if($jm_dir=~/\/$/){
	chop($jm_dir);
}

sub find{
        my $list="";
        my $week=`date +%W 2>/dev/null`;
        chop($week);
        unless(-e "$jm_dir/processed/.jmgridice_grep_mess_lines"){
                @grid_sub=`grep -P 'Submitted.+to batch system.+with ID' /var/log/messages 2>/dev/null`;
                @grid_sub_old=`grep -P 'Submitted.+to batch system.+with ID' /var/log/messages.1 2>/dev/null`;
                push(@grid_sub,@grid_sub_old);
                open(COUNT,">$jm_dir/processed/.jmgridice_grep_mess_lines");
                print COUNT $week;
                close(COUNT);
        }
        while(my $job_id=shift(@_)){
                my $found=grep /with ID $job_id/,@grid_sub;
                if($found){
                        $list.=" $job_id";
                }
        }
        return $list;
}

sub retrieve{
	my $file=$_[0];
	my @values=("id","grid_id","user","group","qtime","queue","exit_status","start","end","status","used_cput","used_walltime","used_mem","used_vmem","exec_host","subject","name","r_proc");
	open(RD,"<$file");
	my @temp_detail=<RD>;
	close(RD);
	my %detail=();
	foreach my $job_det (@temp_detail){
		my @fields=split(/\$/,$job_det);
		my $i=0;
		while($i<17){
			$detail{$fields[3]}{$fields[5]}{$fields[0]}{$values[$i]}=$fields[$i];
			$i++;
		}
		if(defined($fields[$i])){
			$detail{$fields[3]}{$fields[5]}{$fields[0]}{"r_proc"}=$fields[$i];
			chop($detail{$fields[3]}{$fields[5]}{$fields[0]}{"r_proc"});
		}
		else{chop($detail{$fields[3]}{$fields[5]}{$fields[0]}{"name"});}
	}
	return \%detail;
}

sub store{
	my %hash=%{$_[0]};
	my $file=$_[1];
	open(JOB,">$file");
	foreach my $k (keys(%hash)){
		foreach my $k1 (keys(%{$hash{$k}})){
			foreach my $k2 (sort keys(%{$hash{$k}{$k1}})){
				if($hash{$k}{$k1}{$k2}{"id"}){
					my $out=$hash{$k}{$k1}{$k2}{"id"};
					$out=$out."\$".$hash{$k}{$k1}{$k2}{"grid_id"};
					$out=$out."\$".$hash{$k}{$k1}{$k2}{"user"};
					$out=$out."\$".$hash{$k}{$k1}{$k2}{"group"};
					$out=$out."\$".$hash{$k}{$k1}{$k2}{"qtime"};
					$out=$out."\$".$hash{$k}{$k1}{$k2}{"queue"};
					$out=$out."\$".$hash{$k}{$k1}{$k2}{"exit_status"};
					$out=$out."\$".$hash{$k}{$k1}{$k2}{"start"};
					$out=$out."\$".$hash{$k}{$k1}{$k2}{"end"};
					$out=$out."\$".$hash{$k}{$k1}{$k2}{"status"};
					$out=$out."\$".$hash{$k}{$k1}{$k2}{"used_cput"};
					$out=$out."\$".$hash{$k}{$k1}{$k2}{"used_walltime"};
					$out=$out."\$".$hash{$k}{$k1}{$k2}{"used_mem"};
					$out=$out."\$".$hash{$k}{$k1}{$k2}{"used_vmem"};
					$out=$out."\$".$hash{$k}{$k1}{$k2}{"exec_host"};
					$out=$out."\$".$hash{$k}{$k1}{$k2}{"subject"};
					$out=$out."\$".$hash{$k}{$k1}{$k2}{"name"};
					$out=$out."\$".$hash{$k}{$k1}{$k2}{"r_proc"} if($hash{$k}{$k1}{$k2}{"r_proc"});
					$out=$out."\n";
					print JOB $out;
				}
			}
		}
	}
	close(JOB);
}
my $act=`date +%s 2>/dev/null`;
chop($act);
my $year=`date +%Y 2>/dev/null`;
chop($year);
$year--;

#############################################
###                                       ###
###                PBS                    ###
###                                       ###
#############################################

if($lrms =~ /pbs/){
	use threads;
	use threads::shared;
	my $t_var : shared =0;
	my $t_loop=40;
	my $globus_loc_buf=`grep GLOBUS_LOCATION= /etc/globus.conf 2> /dev/null`;
	my ($globus_loc)=($globus_loc_buf=~ /GLOBUS_LOCATION=(.+)/);
	my $command_buff_search=`grep QSTAT $globus_loc/libexec/globus-sh-tools-vars.sh 2> /dev/null | sed "s/GLOBUS_SH_QSTAT=//g" 2> /dev/null`;
	my ($command_search)=($command_buff_search=~ /"(.+)\/qstat"/);
	unless($command_search=~/\w+/){
		$command_search=`whereis qstat 2>/dev/null`;
		if($command_search=~/qstat:\s+(\S+)/){
			my $tmp_command=$1;
			my $ver=`file $1 2>/dev/null`;
			if($ver=~/executable/){
				$command=$tmp_command;
			}
			else{
				$command_search=`which qstat 2>/dev/null`;
				if($command_search=~/^(\S+qstat)/){
					$command=$1;
				}
				else{$command="qstat";}
			}
		}
		else{
			$command_search=`which qstat 2>/dev/null`;
			if($command_search=~/^(\S+qstat)/){
				$command=$1;
			}
			else{$command="qstat";}
		}
	}
	else{$command=$command_search."/qstat";}

	my $key=0;
	my %proc=();
	my $z=0;
	if(-e "$jm_dir/processed/.jmgridice_job_detail"){
		%job_detail=%{retrieve("$jm_dir/processed/.jmgridice_job_detail")};
	}
	my $date = `date +%Y%m%d 2>/dev/null`;
	my $yesterday=`date -d '1 day ago' +%Y%m%d 2>/dev/null`;
	chop($date);
	chop($yesterday);
	my $since_mid=`date +%S 2>/dev/null`+`date +%M 2>/dev/null`*60+`date +%k 2>/dev/null`*3600;
	my $pbs_log=$log_path."/".$date;
	my $pbs_log_y=$log_path."/".$yesterday;
	my $check_log=`grep \`hostname -s 2>/dev/null\` $log_path/* 2>/dev/null | head -n1 2>/dev/null`;
	unless($check_log){
		print "pbs\$NO_LOG";
		exit 0;
	}
	$thread = threads->create(sub { `$command 2>/dev/null`;
			$t_var=1; });
	my @job_ended=();
	my %sub=();
	my %rem=();
	$interval=$interval*3600;
	my @grid_sr=();
## GRID JOBs: SUBMITTED - DELETED
	my $gcount=0;
	my $el=0;
	unless(-e "$jm_dir/processed/.jmgridice_grep_mess_lines"){
                my $tmp_date=`date +%u 2>/dev/null`;
                chop($tmp_date);
                my @grid=`tail -n 20000 /var/log/messages 2>/dev/null`;
                if($tmp_date==1){
                        my @grid_tmp=`tail -n 20000 /var/log/messages.1 2>/dev/null`;
                        push(@grid,@grid_tmp);
                }
                open(COUNT,">$jm_dir/processed/.jmgridice_grep_mess_lines");
                print COUNT $#grid;
                close(COUNT);
                @grid_sub=grep /Submitted|(ID.+REMOVED)/,@grid;
        }
	while($item = $grid_sub[$el]){
		if($item=~/^(\w+\s+\d+\s+\d+:\d+:\d+).+with ID\s+(\d+).+/){
			$sub{$2}=`date -d "$1" +\%s 2>/dev/null`;
			chop($sub{$2});
			if($sub{$2} > $act){
				$sub{$2}=`date -d "$1 $year" +%s 2>/dev/null`;
				chop($sub{$2});
			}
		}
		elsif($item=~/^(\w+\s+\d+\s+\d+:\d+:\d+).+batch ID\s+(\d+)\.(.+)\)\s+REMOVED/){
			$rem{$2}{"end"}=`date -d "$1" +\%s 2>/dev/null`;
			chop($rem{$2}{"end"});
			if($rem{$2} > $act){
                                $rem{$2}=`date -d "$1 $year" +%s 2>/dev/null`;
                                chop($rem{$2});
                        }
			$rem{$2}{"dom"}=$3;
		}
		$el++;
	}

## ENDED JOBS

	unless(-e "$jm_dir/processed/.jmgridice_acc_lines"){
		my @acct=`cat $pbs_log 2>/dev/null`;
		my $count=$#acct+1;
		open(COUNT,">$jm_dir/processed/.jmgridice_acc_lines");
		print COUNT $count;
		close(COUNT);
		@ended=grep /;E;/,@acct;
	}
	else{
		open(ACC,"<$jm_dir/processed/.jmgridice_acc_lines");
		$acc_lines=(<ACC>)[0];
		close(ACC);
		my $acount = `wc -l $pbs_log 2>/dev/null`;
		$acount =~/(\d+)/;
		$acount = $1;
		if($acount > $acc_lines){
			my $tail_c=$acc_lines+1;
			my @end_tmp=`tail -n +$tail_c $pbs_log 2>/dev/null`;
			@ended = grep /;E;/,@end_tmp;
			open(COUNT,">$jm_dir/processed/.jmgridice_acc_lines");
			print COUNT $acount;
			close(COUNT);
		}
		elsif($acount < $acc_lines){
			my $tail_c=$acc_lines+1;
			my @ended_tmp = `tail -n +$tail_c $pbs_log_y 2>/dev/null`;
			@ended = `cat $pbs_log 2>/dev/null`;
			open(COUNT,">$jm_dir/processed/.jmgridice_acc_lines");
			print COUNT $#ended+1;
			close(COUNT);
			push(@ended,@ended_tmp);
			@ended = grep /;E;/,@ended;
		}
	}

	foreach $job_log (@ended){
		if($job_log=~/(.+);E;(\d+).+;user/){
			my $eid=$2;
			my $end=`date -d "$1" +%s 2>/dev/null`;
			chop($end);
			my $diff=$act-$end;
			if(($diff < $interval) or (-e "$jm_dir/ended/$eid")){
				$job_ended{$eid}{"log"}=$job_log;
			}
			delete($rem{$eid}) if($rem{$eid}{"end"});
		}
	}
	if($since_mid<$interval){
		my @ended=`grep \\;E\\; $pbs_log_y 2>/dev/null`;
		foreach $job_log (@ended){
			if($job_log=~/(.+);E;(\d+).+;user/){
				my $eid=$2;
				my $end=`date -d "$1" +%s 2>/dev/null`;
				chop($end);
				my $diff=$act-$end;
				if(($diff < $interval) or (-e "$jm_dir/ended/$eid")){
					$job_ended{$eid}{"log"}=$job_log;
				}
				delete($rem{$eid}) if($rem{$eid}{"end"});
			}
		}
	}

##############################
##       BACKUP             ##
##############################

	if(-e "$jm_dir/processed/$date"){
		%backup=%{retrieve("$jm_dir/processed/$date")};
	}
	else{%backup=();}
	if(-e "$jm_dir/processed/$yesterday"){
		%backup_tmp=%{retrieve("$jm_dir/processed/$yesterday")};
	}
	else{%backup_tmp=();}
	my $midnight=(`date +%s 2>/dev/null`)-(`date +%S 2>/dev/null`+`date +%M 2>/dev/null`*60+`date +%k 2>/dev/null`*3600);
	my $flag_y=0;
	my $flag_yb=0;
	my %job_proc=();
	foreach my $del (keys(%job_detail)){
		foreach my $del1 (keys(%{$job_detail{$del}})){
			foreach my $del2 (keys(%{$job_detail{$del}{$del1}})){
				$job_proc{$del2}{"status"}=$job_detail{$del}{$del1}{$del2}{"status"};
				$job_proc{$del2}{"vo"}=$del;
				$job_proc{$del2}{"queue"}=$del1;
				if($job_detail{$del}{$del1}{$del2}{"status"} eq "E" and $job_detail{$del}{$del1}{$del2}{"end"}=~/^\d+$/){
					delete($rem{$del2}) if($rem{$del2}{"end"});
					my $end=$job_detail{$del}{$del1}{$del2}{"end"};
					my $diff=$act-$end;
					if($diff >= $interval){
						unless($backup{$del}{$del1}{$del2}{"id"}){
							if($job_detail{$del}{$del1}{$del2}{"end"}>=$midnight){
								$backup{$del}{$del1}{$del2}=$job_detail{$del}{$del1}{$del2};
								$flag_y=1;
							}
							else{
								unless($backup_tmp{$del}{$del1}{$del2}{"id"}){
									$backup_tmp{$del}{$del1}{$del2}=$job_detail{$del}{$del1}{$del2};
									$flag_yb=1;
								}
							}
						}
						delete($job_detail{$del}{$del1}{$del2});
					}
				}
			}
		}
	}
	if($debug){
		if($flag_y==1){
			store(\%backup,"$jm_dir/processed/$date");
		}
		else{%backup=();}
		if($flag_yb==1){
			store(\%backup_tmp,"$jm_dir/processed/$yesterday");
		}
		else{%backup_tmp=();}
	}

##############################
##       ENDED JOBS         ##
##############################

	foreach my $job (keys(%job_ended)){
		if(-e "$jm_dir/ended/$job"){
			delete($rem{$job}) if($rem{$job}{"end"});
			$proc{$z}="$jm_dir/ended/$job";
			$z++;
			my %ref_new=%{retrieve("$jm_dir/ended/$job")};
			foreach my $vo (keys(%ref_new)){
				foreach my $queue (keys(%{$ref_new{$vo}})){
					unless($job_detail{$vo}{$queue}{$job}{"status"} eq "E" and $job_detail{$vo}{$queue}{$job}{"end"}){
						if($ref_new{$vo}{$queue}{$job}{"id"}){
							if($job_proc{$job}{"status"} and $job_proc{$job}{"vo"} ne $vo){
								$vo=$job_proc{$job}{"vo"};
								$queue=$job_proc{$job}{"queue"};
								if($job_ended{$job}{"log"}=~/;E;.+start=(\d+).+exec_host=([\w-]+)\..+end=(\d+)\s+Exit_status=(.+)\s+resources_used.cput=(\S*\d+):(\S*\d+):(\S*\d+)\s+resources_used.mem=(\d+)kb\s+resources_used.vmem=(\d+)kb\s+resources_used.walltime=(\S*\d+):(\S*\d+):(\S*\d+)/){
									if(($act-$3)<$interval or $job_proc{$job}){
										$job_detail{$vo}{$queue}{$job}{"status"}="E";
										$job_detail{$vo}{$queue}{$job}{"id"}=$ref_new{$vo}{$queue}{$job}{"id"};
										$job_detail{$vo}{$queue}{$job}{"user"}=$ref_new{$vo}{$queue}{$job}{"user"};
										$job_detail{$vo}{$queue}{$job}{"group"}=$ref_new{$vo}{$queue}{$job}{"group"};
										$job_detail{$vo}{$queue}{$job}{"qtime"}=$ref_new{$vo}{$queue}{$job}{"qtime"};
										$job_detail{$vo}{$queue}{$job}{"queue"}=$ref_new{$vo}{$queue}{$job}{"queue"};
										$job_detail{$vo}{$queue}{$job}{"name"}=$ref_new{$vo}{$queue}{$job}{"name"};
										$job_detail{$vo}{$queue}{$job}{"subject"}=$ref_new{$vo}{$queue}{$job}{"subject"};
										$job_detail{$vo}{$queue}{$job}{"start"}=$1;
										$job_detail{$vo}{$queue}{$job}{"exec_host"}=$2;
										$job_detail{$vo}{$queue}{$job}{"end"}=$3;
										$job_detail{$vo}{$queue}{$job}{"exit_status"}=$4;
										my $cput_h=$5;
										my $cput_m=$6;
										my $cput_s=$7;
										$job_detail{$vo}{$queue}{$job}{"used_cput"}=$cput_s+$cput_m*60+$cput_h*3600;
										my $wallt_h=$10;
										my $wallt_m=$11;
										my $wallt_s=$12;
										$job_detail{$vo}{$queue}{$job}{"used_walltime"}=$wallt_s+$wallt_m*60+$wallt_h*3600;
										if($job_detail{$vo}{$queue}{$job}{"used_walltime"}<0){$job_detail{$vo}{$queue}{$job}{"used_walltime"}=$job_detail{$vo}{$queue}{$job}{"end"}-$job_detail{$vo}{$queue}{$job}{"start"};}
										$job_detail{$vo}{$queue}{$job}{"used_mem"}=$8;
										$job_detail{$vo}{$queue}{$job}{"used_vmem"}=$9;
										$job_detail{$vo}{$queue}{$job}{"grid_id"}=$ref_new{$vo}{$queue}{$job}{"grid_id"} unless($job_detail{$vo}{$queue}{$job}{"grid_id"}=~/https/);
									}
								}
								elsif($job_ended{$job}{"log"}=~/;E;.+start=(\d+).+exec_host=([\w-]+)\..+end=(\d+)\s+Exit_status=(\d+)/){
									if(($act-$3)<$interval or $job_proc{$job}{"status"}){
										$job_detail{$vo}{$queue}{$job}{"status"}="E";
										$job_detail{$vo}{$queue}{$job}{"id"}=$ref_new{$vo}{$queue}{$job}{"id"};
										$job_detail{$vo}{$queue}{$job}{"user"}=$ref_new{$vo}{$queue}{$job}{"user"};
										$job_detail{$vo}{$queue}{$job}{"group"}=$ref_new{$vo}{$queue}{$job}{"group"};
										$job_detail{$vo}{$queue}{$job}{"qtime"}=$ref_new{$vo}{$queue}{$job}{"qtime"};
										$job_detail{$vo}{$queue}{$job}{"queue"}=$ref_new{$vo}{$queue}{$job}{"queue"};
										$job_detail{$vo}{$queue}{$job}{"name"}=$ref_new{$vo}{$queue}{$job}{"name"};
										$job_detail{$vo}{$queue}{$job}{"subject"}=$ref_new{$vo}{$queue}{$job}{"subject"};
										$job_detail{$vo}{$queue}{$job}{"start"}=$1;
										$job_detail{$vo}{$queue}{$job}{"exec_host"}=$2;
										$job_detail{$vo}{$queue}{$job}{"end"}=$3;
										$job_detail{$vo}{$queue}{$job}{"exit_status"}=$4;
										$job_detail{$vo}{$queue}{$job}{"used_cput"}="0";
										$job_detail{$vo}{$queue}{$job}{"used_walltime"}="0";
										$job_detail{$vo}{$queue}{$job}{"used_mem"}="0";
										$job_detail{$vo}{$queue}{$job}{"used_vmem"}="0";
										$job_detail{$vo}{$queue}{$job}{"grid_id"}=$ref_new{$vo}{$queue}{$job}{"grid_id"} unless($job_detail{$vo}{$queue}{$job}{"grid_id"}=~/https/);
									}
								}
							}
							else{
								$job_detail{$vo}{$queue}{$job}{"status"}="E";
								$job_detail{$vo}{$queue}{$job}{"id"}=$ref_new{$vo}{$queue}{$job}{"id"};
								$job_detail{$vo}{$queue}{$job}{"grid_id"}=$ref_new{$vo}{$queue}{$job}{"grid_id"} unless($job_detail{$vo}{$queue}{$job}{"grid_id"}=~/https/);
								$job_detail{$vo}{$queue}{$job}{"user"}=$ref_new{$vo}{$queue}{$job}{"user"};
								$job_detail{$vo}{$queue}{$job}{"group"}=$ref_new{$vo}{$queue}{$job}{"group"};
								$job_detail{$vo}{$queue}{$job}{"qtime"}=$ref_new{$vo}{$queue}{$job}{"qtime"};
								$job_detail{$vo}{$queue}{$job}{"queue"}=$ref_new{$vo}{$queue}{$job}{"queue"};
								$job_detail{$vo}{$queue}{$job}{"exit_status"}=$ref_new{$vo}{$queue}{$job}{"exit_status"};
								$job_detail{$vo}{$queue}{$job}{"exec_host"}=$ref_new{$vo}{$queue}{$job}{"exec_host"};
								$job_detail{$vo}{$queue}{$job}{"name"}=$ref_new{$vo}{$queue}{$job}{"name"};
								$job_detail{$vo}{$queue}{$job}{"subject"}=$ref_new{$vo}{$queue}{$job}{"subject"};
								$job_detail{$vo}{$queue}{$job}{"used_cput"}=$ref_new{$vo}{$queue}{$job}{"used_cput"};
								$job_detail{$vo}{$queue}{$job}{"used_walltime"}=$ref_new{$vo}{$queue}{$job}{"used_walltime"};
								$job_detail{$vo}{$queue}{$job}{"used_mem"}=$ref_new{$vo}{$queue}{$job}{"used_mem"};
								$job_detail{$vo}{$queue}{$job}{"used_vmem"}=$ref_new{$vo}{$queue}{$job}{"used_vmem"};
								$job_detail{$vo}{$queue}{$job}{"start"}=$ref_new{$vo}{$queue}{$job}{"start"};
								$job_detail{$vo}{$queue}{$job}{"end"}=$ref_new{$vo}{$queue}{$job}{"end"};
							}
						}
					}
				}
			}
		}
		elsif(-e "$jm_dir/new/$job"){
			delete($rem{$job}) if($rem{$job}{"end"});
			$proc{$z}="$jm_dir/new/$job";
			$z++;
			my %ref_new=%{retrieve("$jm_dir/new/$job")};
			foreach my $vo (keys(%ref_new)){
				foreach my $queue (keys(%{$ref_new{$vo}})){
					if($ref_new{$vo}{$queue}{$job}{"id"}){
						unless($job_detail{$vo}{$queue}{$job}{"status"} eq "E" and $job_detail{$vo}{$queue}{$job}{"end"}){
							if($job_ended{$job}{"log"}=~/;E;.+start=(\d+).+exec_host=([\w-]+)\..+end=(\d+)\s+Exit_status=(.+)\s+resources_used.cput=(\S*\d+):(\S*\d+):(\S*\d+)\s+resources_used.mem=(\d+)kb\s+resources_used.vmem=(\d+)kb\s+resources_used.walltime=(\S*\d+):(\S*\d+):(\S*\d+)/){
								if(($act-$3)<$interval or $job_proc{$job}{"status"}){
									$job_detail{$vo}{$queue}{$job}{"status"}="E";
									$job_detail{$vo}{$queue}{$job}{"id"}=$ref_new{$vo}{$queue}{$job}{"id"};
									$job_detail{$vo}{$queue}{$job}{"user"}=$ref_new{$vo}{$queue}{$job}{"user"};
									$job_detail{$vo}{$queue}{$job}{"group"}=$ref_new{$vo}{$queue}{$job}{"group"};
									$job_detail{$vo}{$queue}{$job}{"qtime"}=$ref_new{$vo}{$queue}{$job}{"qtime"};
									$job_detail{$vo}{$queue}{$job}{"queue"}=$ref_new{$vo}{$queue}{$job}{"queue"};
									$job_detail{$vo}{$queue}{$job}{"name"}=$ref_new{$vo}{$queue}{$job}{"name"};
									$job_detail{$vo}{$queue}{$job}{"subject"}=$ref_new{$vo}{$queue}{$job}{"subject"};
									$job_detail{$vo}{$queue}{$job}{"start"}=$1;
									$job_detail{$vo}{$queue}{$job}{"exec_host"}=$2;
									$job_detail{$vo}{$queue}{$job}{"end"}=$3;
									$job_detail{$vo}{$queue}{$job}{"exit_status"}=$4;
									my $cput_h=$5;
									my $cput_m=$6;
									my $cput_s=$7;
									$job_detail{$vo}{$queue}{$job}{"used_cput"}=$cput_s+$cput_m*60+$cput_h*3600;
									my $wallt_h=$10;
									my $wallt_m=$11;
									my $wallt_s=$12;
									$job_detail{$vo}{$queue}{$job}{"used_walltime"}=$wallt_s+$wallt_m*60+$wallt_h*3600;
									if($job_detail{$vo}{$queue}{$job}{"used_walltime"}<0){$job_detail{$vo}{$queue}{$job}{"used_walltime"}=$job_detail{$vo}{$queue}{$job}{"end"}-$job_detail{$vo}{$queue}{$job}{"start"};}
									$job_detail{$vo}{$queue}{$job}{"used_mem"}=$8;
									$job_detail{$vo}{$queue}{$job}{"used_vmem"}=$9;
									$job_detail{$vo}{$queue}{$job}{"grid_id"}=$ref_new{$vo}{$queue}{$job}{"grid_id"} unless($job_detail{$vo}{$queue}{$job}{"grid_id"}=~/https/);
								}
							}
							elsif($job_ended{$job}{"log"}=~/;E;.+start=(\d+).+exec_host=([\w-]+)\..+end=(\d+)\s+Exit_status=(\d+)/){
								if(($act-$3)<$interval or $job_proc{$job}{"status"}){
									$job_detail{$vo}{$queue}{$job}{"status"}="E";
									$job_detail{$vo}{$queue}{$job}{"id"}=$ref_new{$vo}{$queue}{$job}{"id"};
									$job_detail{$vo}{$queue}{$job}{"user"}=$ref_new{$vo}{$queue}{$job}{"user"};
									$job_detail{$vo}{$queue}{$job}{"group"}=$ref_new{$vo}{$queue}{$job}{"group"};
									$job_detail{$vo}{$queue}{$job}{"qtime"}=$ref_new{$vo}{$queue}{$job}{"qtime"};
									$job_detail{$vo}{$queue}{$job}{"queue"}=$ref_new{$vo}{$queue}{$job}{"queue"};
									$job_detail{$vo}{$queue}{$job}{"name"}=$ref_new{$vo}{$queue}{$job}{"name"};
									$job_detail{$vo}{$queue}{$job}{"subject"}=$ref_new{$vo}{$queue}{$job}{"subject"};
									$job_detail{$vo}{$queue}{$job}{"start"}=$1;
									$job_detail{$vo}{$queue}{$job}{"exec_host"}=$2;
									$job_detail{$vo}{$queue}{$job}{"end"}=$3;
									$job_detail{$vo}{$queue}{$job}{"exit_status"}=$4;
									$job_detail{$vo}{$queue}{$job}{"grid_id"}=$ref_new{$vo}{$queue}{$job}{"grid_id"} unless($job_detail{$vo}{$queue}{$job}{"grid_id"}=~/https/);
									$job_detail{$vo}{$queue}{$job}{"used_cput"}="0";
									$job_detail{$vo}{$queue}{$job}{"used_walltime"}="0";
									$job_detail{$vo}{$queue}{$job}{"used_mem"}="0";
									$job_detail{$vo}{$queue}{$job}{"used_vmem"}="0";
								}
							}
						}
					}
				}
			}
		}
		elsif($sub{$job} or $rem{$job}{"end"} or $job_proc{$job}{"status"}){
			if($job_ended{$job}{"log"}=~/;E;(\d+).+;user=(.+)\s+group=(.+)\s+jobname=(.+)\s+queue=(.+)\s+ctime=(\d+).+start=(\d+)\s+exec_host=([\w-]+)\..+end=(\d+)\s+Exit_status=(.+)\s+resources_used.cput=(\S*\d+):(\S*\d+):(\S*\d+)\s+resources_used.mem=(\d+)kb\s+resources_used.vmem=(\d+)kb\s+resources_used.walltime=(\S*\d+):(\S*\d+):(\S*\d+)/){
				my $vo=$3;
				my $queue=$5;
				delete($rem{$job}) if($rem{$job}{"end"});
				if(!($job_detail{$vo}{$queue}{$job}{"status"} eq "E" and $job_detail{$vo}{$queue}{$job}{"end"})){
					$job_detail{$vo}{$queue}{$job}{"status"}="E";
					$job_detail{$vo}{$queue}{$job}{"id"}=$1;
					$job_detail{$vo}{$queue}{$job}{"user"}=$2;
					$job_detail{$vo}{$queue}{$job}{"group"}=$3;
					$job_detail{$vo}{$queue}{$job}{"name"}=$4;
					$job_detail{$vo}{$queue}{$job}{"queue"}=$5;
					$job_detail{$vo}{$queue}{$job}{"qtime"}=$6;
					$job_detail{$vo}{$queue}{$job}{"start"}=$7;
					$job_detail{$vo}{$queue}{$job}{"exec_host"}=$8;
					$job_detail{$vo}{$queue}{$job}{"end"}=$9;
					$job_detail{$vo}{$queue}{$job}{"exit_status"}=$10;
					my $cput_h=$11;
					my $cput_m=$12;
					my $cput_s=$13;
					$job_detail{$vo}{$queue}{$job}{"used_cput"}=$cput_s+$cput_m*60+$cput_h*3600;
					$job_detail{$vo}{$queue}{$job}{"used_mem"}=$14;
					$job_detail{$vo}{$queue}{$job}{"used_vmem"}=$15;
					my $wallt_h=$16;
					my $wallt_m=$17;
					my $wallt_s=$18;
					$job_detail{$vo}{$queue}{$job}{"used_walltime"}=$wallt_s+$wallt_m*60+$wallt_h*3600;
					if($job_detail{$vo}{$queue}{$job}{"used_walltime"}<0){$job_detail{$vo}{$queue}{$job}{"used_walltime"}=$job_detail{$vo}{$queue}{$job}{"end"}-$job_detail{$vo}{$queue}{$job}{"start"};}
					unless($job_detail{$vo}{$queue}{$job}{"subject"}){
						my $user=$job_detail{$vo}{$queue}{$job}{"user"};
						if(-e "$jm_dir/subject/$user"){
							$job_detail{$vo}{$queue}{$job}{"subject"}=`cat $jm_dir/subject/$user 2>/dev/null`;
						}
						else{
							my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
							$sbj=~/"(.+)"\s+mapped/;
							$job_detail{$vo}{$queue}{$job}{"subject"}=$1;
						}
					}
					$job_detail{$vo}{$queue}{$job}{"grid_id"}="n/a" unless($job_detail{$vo}{$queue}{$job}{"grid_id"}=~/https/);
				}
			}
			elsif($job_ended{$job}{"log"}=~/;E;(\d+).+;user=(.+)\s+group=(.+)\s+jobname=(.+)\s+queue=(.+)\s+ctime=(\d+).+start=(\d+)\s+exec_host=([\w-]+)\..+end=(\d+)\s+Exit_status=(\d+)/){
				delete($rem{$job}) if($rem{$job}{"end"});
				my $vo=$3;
				my $queue=$5;
				if(!($job_detail{$vo}{$queue}{$job}{"status"} eq "E" and $job_detail{$vo}{$queue}{$job}{"end"})){
					$job_detail{$vo}{$queue}{$job}{"status"}="E";
					$job_detail{$vo}{$queue}{$job}{"id"}=$1;
					$job_detail{$vo}{$queue}{$job}{"user"}=$2;
					$job_detail{$vo}{$queue}{$job}{"group"}=$3;
					$job_detail{$vo}{$queue}{$job}{"name"}=$4;
					$job_detail{$vo}{$queue}{$job}{"queue"}=$5;
					$job_detail{$vo}{$queue}{$job}{"qtime"}=$6;
					$job_detail{$vo}{$queue}{$job}{"start"}=$7;
					$job_detail{$vo}{$queue}{$job}{"exec_host"}=$8;
					$job_detail{$vo}{$queue}{$job}{"end"}=$9;
					$job_detail{$vo}{$queue}{$job}{"exit_status"}=$10;
					$job_detail{$vo}{$queue}{$job}{"used_cput"}="0";
					$job_detail{$vo}{$queue}{$job}{"used_mem"}="0";
					$job_detail{$vo}{$queue}{$job}{"used_vmem"}="0";
					$job_detail{$vo}{$queue}{$job}{"used_walltime"}="0";
					unless($job_detail{$vo}{$queue}{$job}{"subject"}){
						my $user=$job_detail{$vo}{$queue}{$job}{"user"};
						if(-e "$jm_dir/subject/$user"){
							$job_detail{$vo}{$queue}{$job}{"subject"}=`cat $jm_dir/subject/$user 2>/dev/null`;
						}
						else{
							my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
							$sbj=~/"(.+)"\s+mapped/;
							$job_detail{$vo}{$queue}{$job}{"subject"}=$1;
						}
					}
					$job_detail{$vo}{$queue}{$job}{"grid_id"}="n/a" unless($job_detail{$vo}{$queue}{$job}{"grid_id"}=~/https/);
				}
			}
		}
		delete($sub{$job}) if($sub{$job});
	}

##############################
##      REMOVED JOBS        ##
##############################

	my $flag=0;
	foreach my $rem (keys(%rem)){
		foreach my $vo (keys(%job_detail)){
			foreach my $queue (keys(%{$job_detail{$vo}})){
				if($job_detail{$vo}{$queue}{$rem}{"id"}){
					$flag=1;
					if($job_detail{$vo}{$queue}{$rem}{"status"} ne "E"){
						$job_detail{$vo}{$queue}{$rem}{"status"}="E";
						$job_detail{$vo}{$queue}{$rem}{"start"}="00";
						$job_detail{$vo}{$queue}{$rem}{"end"} = $rem{$rem}{"end"};
						my $dom=$rem{$rem}{"dom"};
						my $line=`grep \\;D\\; $pbs_log 2>/dev/null | grep $rem.$dom 2>/dev/null`;
						$line=`grep \\;D\\; $pbs_log_y 2>/dev/null | grep $rem.$dom 2>/dev/null` unless($line);
						$line=~/requestor=(.+)@/;
						if($1 =~ /root/){
							$job_detail{$vo}{$queue}{$rem}{"exit_status"}="-201";
						}
						else{
							$job_detail{$vo}{$queue}{$rem}{"exit_status"}="-200";
						}
						if(-e "$jm_dir/ended/$rem"){
							$proc{$z}="$jm_dir/ended/$rem";
							$z++;
						}
						elsif(-e "$jm_dir/new/$rem"){
							$proc{$z}="$jm_dir/new/$rem";
							$z++;
						}
					}
				}
			}
		}
		if($flag==0){
			if(-e "$jm_dir/ended/$rem"){
				$proc{$z}="$jm_dir/ended/$rem";
				$z++;
				my %ref=%{retrieve("$jm_dir/ended/$rem")};
				foreach my $vo (keys(%ref)){
					foreach my $queue (keys(%{$ref{$vo}})){
						if($ref{$vo}{$queue}{$rem}{"id"} and !($ref{$vo}{$queue}{$rem}{"status"}="E")){
							$job_detail{$vo}{$queue}{$rem}{"id"}=$rem;
							$job_detail{$vo}{$queue}{$rem}{"status"}="E";
							$job_detail{$vo}{$queue}{$rem}{"start"} = "00";
							$job_detail{$vo}{$queue}{$rem}{"exec_host"} = "NO HOST";
							$job_detail{$vo}{$queue}{$rem}{"end"} = $rem{$rem}{"end"};
							$job_detail{$vo}{$queue}{$rem}{"exit_status"}=$ref{$vo}{$queue}{$rem}{"exit_status"};
							$job_detail{$vo}{$queue}{$rem}{"used_cput"} = $ref{$vo}{$queue}{$rem}{"used_cput"};
							$job_detail{$vo}{$queue}{$rem}{"used_mem"}=$ref{$vo}{$queue}{$rem}{"used_mem"};
							$job_detail{$vo}{$queue}{$rem}{"used_vmem"}=$ref{$vo}{$queue}{$rem}{"used_vmem"};
							$job_detail{$vo}{$queue}{$rem}{"used_walltime"} = $ref{$vo}{$queue}{$rem}{"used_walltime"};
							$job_detail{$vo}{$queue}{$rem}{"name"}=$ref{$vo}{$queue}{$rem}{"name"};
							$job_detail{$vo}{$queue}{$rem}{"grid_id"}=$ref{$vo}{$queue}{$rem}{"grid_id"} unless($job_detail{$vo}{$queue}{$rem}{"grid_id"}=~/https/);
							$job_detail{$vo}{$queue}{$rem}{"group"}=$ref{$vo}{$queue}{$rem}{"group"};
							$job_detail{$vo}{$queue}{$rem}{"subject"}=$ref{$vo}{$queue}{$rem}{"subject"};
							$job_detail{$vo}{$queue}{$rem}{"user"}=$ref{$vo}{$queue}{$rem}{"user"};
							$job_detail{$vo}{$queue}{$rem}{"qtime"}=$ref{$vo}{$queue}{$rem}{"qtime"};
							$job_detail{$vo}{$queue}{$rem}{"queue"}=$ref{$vo}{$queue}{$rem}{"queue"};
						}
					}
				}
			}
			elsif(-e "$jm_dir/new/$rem"){
				$proc{$z}="$jm_dir/new/$rem";
				$z++;
				%ref=%{retrieve("$jm_dir/new/$rem")};
				foreach my $vo (keys(%ref)){
					foreach my $queue (keys(%{$ref{$vo}})){
						if($ref{$vo}{$queue}{$rem}{"id"} and !($ref{$vo}{$queue}{$rem}{"status"}="E")){
							$job_detail{$vo}{$queue}{$rem}{"id"}=$rem;
							$job_detail{$vo}{$queue}{$rem}{"status"}="E";
							$job_detail{$vo}{$queue}{$rem}{"start"} = "00";
							$job_detail{$vo}{$queue}{$rem}{"exec_host"} = "NO HOST";
							$job_detail{$vo}{$queue}{$rem}{"end"} = $rem{$rem}{"end"};
							my $dom=$rem{$rem}{"dom"};
							my $line=`grep \\;D\\; $pbs_log 2>/dev/null | grep $rem.$dom 2>/dev/null`;
							$line=~/requestor=(.+)@/;
							if($1 =~ /root/){
								$job_detail{$vo}{$queue}{$rem}{"exit_status"}="-201";
							}
							else{
								$job_detail{$vo}{$queue}{$rem}{"exit_status"}="-200";
							}
							$job_detail{$vo}{$queue}{$rem}{"used_cput"} = $ref{$vo}{$queue}{$rem}{"used_cput"};
							$job_detail{$vo}{$queue}{$rem}{"used_mem"}=$ref{$vo}{$queue}{$rem}{"used_mem"};
							$job_detail{$vo}{$queue}{$rem}{"used_vmem"}=$ref{$vo}{$queue}{$rem}{"used_vmem"};
							$job_detail{$vo}{$queue}{$rem}{"used_walltime"} = $ref{$vo}{$queue}{$rem}{"used_walltime"};
							$job_detail{$vo}{$queue}{$rem}{"name"}=$ref{$vo}{$queue}{$rem}{"name"};
							$job_detail{$vo}{$queue}{$rem}{"grid_id"}=$ref{$vo}{$queue}{$rem}{"grid_id"} unless($job_detail{$vo}{$queue}{$rem}{"grid_id"}=~/https/);
							$job_detail{$vo}{$queue}{$rem}{"group"}=$ref{$vo}{$queue}{$rem}{"group"};
							$job_detail{$vo}{$queue}{$rem}{"subject"}=$ref{$vo}{$queue}{$rem}{"subject"};
							$job_detail{$vo}{$queue}{$rem}{"user"}=$ref{$vo}{$queue}{$rem}{"user"};
							$job_detail{$vo}{$queue}{$rem}{"qtime"}=$ref{$vo}{$queue}{$rem}{"qtime"};
							$job_detail{$vo}{$queue}{$rem}{"queue"}=$ref{$vo}{$queue}{$rem}{"queue"};
						}
					}
				}
			}
		}
	}

##############################
##       qstat -f           ##
##############################

	until($t_loop==0){
		if($t_var==1){
			$thread->join();
			$t_loop=0;
		}
		else{
			$t_loop--;
			if($t_loop==0){
				$t_update=1;
				$thread->join();
				next;
			}
			sleep 1;
		}
	}
	
	unless($t_update){

		%unvalid_id=();
                if(-e "$jm_dir/processed/.jmgridice_unvalid_id"){
                        open(RD,"<$jm_dir/processed/.jmgridice_unvalid_id");
                        $valid=<RD>;
                        close(RD);
                        if($valid=~/unvalid\$(.+)\$$/){
                                my @unvalid_list=split("\$",$1);
                                while(my $unv_id=shift(@unvalid_list)){
                                        $unvalid_id{$unv_id}=1;
                                }
                                $valid="unvalid\$";
                        }
                }
                else{$valid="unvalid\$";}
		my $valid_list="";
		my @job_list=`$command 2>/dev/null`;
		foreach my $qstat_line (@job_list){
                        if($qstat_line=~/^(\d+)/){
                                my $id=$1;
				$qstat{$id}=$id;
                                unless($unvalid_id{$id}){
                                        if($qstat_line=~/Q\s+\w+\s*$/){
                                                unless($job_proc{$id}{"status"}){
                                                        $valid_list.=" $id";
                                                }
                                        }
                                        else{
                                                if($job_proc{$id}{"status"} eq "Q"){
                                                        $valid_list.=" $id";
                                                }
                                                elsif(!$update){
                                                        $valid_list.=" $id";
                                                }
                                                else{
                                                        $valid_list.=" $id" unless($job_proc{$id}{"status"} eq "R");
                                                }
                                        }
                                }
                                else{$unvalid_id{$id}=0;}
                        }
                }
		@job_list=`$command -f $valid_list 2>/dev/null` if($valid_list=~/\d+/);
		$flag=0;
		my $flagl=0;
		my $flagg=0;
		my $flagd=0;
		my $grid_id1="";
		my $grid_id="";
		foreach $line (@job_list){
			if($line =~ /Job Id:\s+(\d+).+/){
				$jid=$1;
				if(-e "$jm_dir/new/$jid"){
					my %ref=%{retrieve("$jm_dir/new/$jid")};
					foreach my $vo (keys(%ref)){
						foreach my $queue (keys(%{$ref{$vo}})){
							if($ref{$vo}{$queue}{$jid}{"id"}){
								$job_detail{$vo}{$queue}{$jid}=$ref{$vo}{$queue}{$jid};
								$job_detail{$vo}{$queue}{$jid}{"r_proc"}=$act;
								$proc{$z}="$jm_dir/new/$jid";
								$z++;
								$flag=1;
							}
						}
					}
				}
			}
			if($line =~ /Job_Name =\s+(.+)\n/){
				$name=$1;
			}
			if($line =~ /resources_used.cput =\s+(\d+):(\d+):(\d+)/){
				$used_cput=$3+$2*60+$1*3600;
			}
			if($line =~ /resources_used.mem =\s+(.+)kb/){
				$used_mem=$1;
			}
			if($line =~ /resources_used.vmem =\s+(.+)kb/){
				$used_vmem=$1;
			}
			if($line =~ /resources_used.walltime =\s+(\d+):(\d+):(\d+)/){
				$used_walltime=$3+$2*60+$1*3600;
			}
			if($line =~ /job_state =\s+(\w)/){
				$status=$1;
			}
			if($line =~ /queue =\s+(.+)\n/){
				$queue=$1;
			}
			if($line =~ /ctime\s=\s\w+\s(.+)\n/){
				$qtime=`date -d "$1" +%s 2>/dev/null`;
				chop($qtime);
			}
			if($line =~ /exec_host =\s+(\w+)\..+/){
				$exec_host=$1;
			}
			if($line =~ /mtime\s=\s\w+\s(.+)\n/){
				$stime_tmp=$1;
				unless($status =~ /[QWHS]/){
					$stime=`date -d "$stime_tmp" +%s 2>/dev/null`;
					chop($stime);
				}
			}
			if($line =~ /X509_USER_PROXY/){
				$flagg=1;
			}
			if($line =~ /EDG_WL_JOBID=(.+),/){
				$grid_id=$1;
			}
			elsif($line =~ /EDG_WL_JOBID=(.+)/){
				$grid_id1=$1;
			}
			if($line =~ /\s+(.*),/ and $grid_id1){
				$grid_id=$grid_id1.$1;
				$grid_id1="";
			}
			if($line =~ /euser =\s+(.+)\n/){
				$user=$1;
			}
			if($line =~ /egroup =\s+(.+)\n/){
				$group=$1;
				$flagl=1;
			}
			if($flagl==1){
				$flagl=0;
				unless($status =~ /E/){
					if($flag==1){
						$job_detail{$group}{$queue}{$jid}{"used_cput"}=$used_cput if($used_cput);
						$job_detail{$group}{$queue}{$jid}{"used_walltime"}=$used_walltime if($used_walltime);
						$job_detail{$group}{$queue}{$jid}{"start"}=$stime if($stime);
						$job_detail{$group}{$queue}{$jid}{"used_mem"}=$used_mem if($used_mem);
						$job_detail{$group}{$queue}{$jid}{"used_vmem"}=$used_vmem if($used_vmem);
						$job_detail{$group}{$queue}{$jid}{"exec_host"}=$exec_host if($exec_host);
						unless($job_detail{$group}{$queue}{$jid}{"grid_id"}=~/https/){
							if($grid_id=~/https/){
								$job_detail{$group}{$queue}{$jid}{"grid_id"}=$grid_id;
							}
							else{
								$job_detail{$group}{$queue}{$jid}{"grid_id"}="n/a";
							}
						}
						$job_detail{$group}{$queue}{$jid}{"status"}=$status;
						$used_cput="";
						$used_walltime="";
						$used_mem="";
						$used_vmem="";
						$exec_host="";
						$status="";
						$stime="";
						$grid_id="";
						$flagg=0;
						$flag=0;
					}
					elsif($sub{$jid}){
						$job_detail{$group}{$queue}{$jid}{"id"}=$jid;
						$job_detail{$group}{$queue}{$jid}{"r_proc"}=$act;
						if($used_cput){
							$job_detail{$group}{$queue}{$jid}{"used_cput"}=$used_cput;
							$used_cput="";
						}
						else{
							$job_detail{$group}{$queue}{$jid}{"used_cput"}="00";
						}
						if($used_walltime){
							$job_detail{$group}{$queue}{$jid}{"used_walltime"}=$used_walltime;
							$used_walltime="";
						}
						else{
							$job_detail{$group}{$queue}{$jid}{"used_walltime"}="00";
						}
						if($stime){
							$job_detail{$group}{$queue}{$jid}{"start"}=$stime;
							$stime="";
						}
						else{
							$job_detail{$group}{$queue}{$jid}{"start"}=undef;
						}
						if($used_mem){
							$job_detail{$group}{$queue}{$jid}{"used_mem"}=$used_mem;
							$used_mem="";
						}
						else{
							$job_detail{$group}{$queue}{$jid}{"used_mem"}="00";
						}
						if($used_vmem){
							$job_detail{$group}{$queue}{$jid}{"used_vmem"}=$used_vmem;
							$used_vmem="";
						}
						else{
							$job_detail{$group}{$queue}{$jid}{"used_vmem"}="00";
						}
						if($exec_host){
							$job_detail{$group}{$queue}{$jid}{"exec_host"}=$exec_host;
							$exec_host="";
						}
						else{
							$job_detail{$group}{$queue}{$jid}{"exec_host"}="NO HOST";
						}
						$job_detail{$group}{$queue}{$jid}{"name"}=$name;
						$name="";
						$job_detail{$group}{$queue}{$jid}{"status"}=$status;
						$status="";
						$job_detail{$group}{$queue}{$jid}{"queue"}=$queue;
						$job_detail{$group}{$queue}{$jid}{"qtime"}=$qtime;
						$qtime="";
						unless($job_detail{$group}{$queue}{$jid}{"grid_id"}=~/https/){
							if($grid_id=~/https/){
								$job_detail{$group}{$queue}{$jid}{"grid_id"}=$grid_id;
							}
							else{
								$job_detail{$group}{$queue}{$jid}{"grid_id"}="n/a";
							}
						}
						$grid_id="";
						$job_detail{$group}{$queue}{$jid}{"user"}=$user;
						$job_detail{$group}{$queue}{$jid}{"group"}=$group;
						$job_detail{$group}{$queue}{$jid}{"end"}=undef;
						$job_detail{$group}{$queue}{$jid}{"exit_status"}=undef;
						if(-e "$jm_dir/subject/$user"){
							$job_detail{$group}{$queue}{$jid}{"subject"}=`cat $jm_dir/subject/$user 2> /dev/null`;
						}
						else{
							my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
							$sbj=~/"(.+)"\s+mapped/;
							$job_detail{$group}{$queue}{$jid}{"subject"}=$1;
						}
						$user="";
						$group="";
						$queue="";
						delete($sub{$jid});
						$flagg=0;
					}
					elsif($flagg==1){
						$job_detail{$group}{$queue}{$jid}{"id"}=$jid;
						$job_detail{$group}{$queue}{$jid}{"r_proc"}=$act;
						unless($job_detail{$group}{$queue}{$jid}{"grid_id"}=~/https/){
							if($grid_id=~/https/){
								$job_detail{$group}{$queue}{$jid}{"grid_id"}=$grid_id;
							}
							else{
								$job_detail{$group}{$queue}{$jid}{"grid_id"}="n/a";
							}
						}
						$grid_id="";
						$job_detail{$group}{$queue}{$jid}{"qtime"}=$qtime;
						$qtime="";
						$job_detail{$group}{$queue}{$jid}{"name"}=$name;
						$name="";
						$job_detail{$group}{$queue}{$jid}{"status"}=$status;
						$status="";
						$job_detail{$group}{$queue}{$jid}{"queue"}=$queue;
						if($used_cput){
							$job_detail{$group}{$queue}{$jid}{"used_cput"}=$used_cput;
							$used_cput="";
						}
						else{
							$job_detail{$group}{$queue}{$jid}{"used_cput"}="00";
						}
						if($used_walltime){
							$job_detail{$group}{$queue}{$jid}{"used_walltime"}=$used_walltime;
							$used_walltime="";
						}
						else{
							$job_detail{$group}{$queue}{$jid}{"used_walltime"}="00";
						}
						if($stime){
							$job_detail{$group}{$queue}{$jid}{"start"}=$stime;
							$stime="";
						}
						else{
							$job_detail{$group}{$queue}{$jid}{"start"}=undef;
						}
						if($used_mem){
							$job_detail{$group}{$queue}{$jid}{"used_mem"}=$used_mem;
							$used_mem="";
						}
						else{
							$job_detail{$group}{$queue}{$jid}{"used_mem"}="00";
						}
						if($used_vmem){
							$job_detail{$group}{$queue}{$jid}{"used_vmem"}=$used_vmem;
							$used_vmem="";
						}
						else{
							$job_detail{$group}{$queue}{$jid}{"used_vmem"}="00";
						}
						if($exec_host){
							$job_detail{$group}{$queue}{$jid}{"exec_host"}=$exec_host;
							$exec_host="";
						}
						else{
							$job_detail{$group}{$queue}{$jid}{"exec_host"}="NO HOST";
						}
						$job_detail{$group}{$queue}{$jid}{"user"}=$user;
						$job_detail{$group}{$queue}{$jid}{"group"}=$group;
						$job_detail{$group}{$queue}{$jid}{"end"}=undef;
						$job_detail{$group}{$queue}{$jid}{"exit_status"}=undef;
						if(-e "$jm_dir/subject/$user"){
							$job_detail{$group}{$queue}{$jid}{"subject"}=`cat $jm_dir/subject/$user 2> /dev/null`;
						}
						else{
							my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
							$sbj=~/"(.+)"\s+mapped/;
							$job_detail{$group}{$queue}{$jid}{"subject"}=$1;
						}
						$user="";
						$group="";
						$queue="";
						$flagg=0;
					}
					else{
						$unvalid_id{$jid}=0;
						$used_cput="";
						$used_walltime="";
						$stime="";
						$used_mem="";
						$used_vmem="";
						$exec_host="";
						$name="";
						$status="";
						$queue="";
						$qtime="";
						$grid_id="";
						$user="";
						$group="";
					}
				}
			}
		}
		foreach my $uv_id (sort keys(%unvalid_id)){
			$valid.="$uv_id\$" if($unvalid_id{$uv_id}==0);
		}
		open(UID,">$jm_dir/processed/.jmgridice_unvalid_id");
		print UID $valid;
		close(UID);

	}

############################
#   close job ended        #
############################

	foreach my $vo (keys(%job_detail)){
		foreach my $queue (keys(%{$job_detail{$vo}})){
			foreach my $job (keys(%{$job_detail{$vo}{$queue}})){
				if(($job_detail{$vo}{$queue}{$job}{"status"} =~ /R|Q|W|H/) and ($act!=$job_detail{$vo}{$queue}{$job}{"r_proc"}) and !($qstat{$job})){
					my $log=`grep ";E;$job." $pbs_log 2>/dev/null`;
					unless($log){
						$log=`grep ";E;$job." $pbs_log_y 2>/dev/null`;
					}
					unless($log){
						$log=`grep ";D;$job." $pbs_log 2>/dev/null`;
					}
					unless($log){
						$log=`grep ";D;$job." $pbs_log_y 2>/dev/null`;
					}
					unless($log){
						my $f_act=`grep -h -P ";(Q|S);$job." $pbs_log 2>/dev/null | tail -n1 2>/dev/null`;
						next if($f_act);
					}
					unless($log){
						$log=`grep -h -P ";(D|E);$job." $log_path/* 2>/dev/null | tail -n1 2>/dev/null` unless($t_update);
					}
					if($log=~/;E;.+start=(\d+).+exec_host=([\w-]+)\..+end=(\d+)\s+Exit_status=(.+)\s+resources_used.cput=(\S*\d+):(\S*\d+):(\S*\d+)\s+resources_used.mem=(\d+)kb\s+resources_used.vmem=(\d+)kb\s+resources_used.walltime=(\S*\d+):(\S*\d+):(\S*\d+)/){
						$job_detail{$vo}{$queue}{$job}{"status"}="E";
						$job_detail{$vo}{$queue}{$job}{"start"}=$1;
						$job_detail{$vo}{$queue}{$job}{"exec_host"}=$2;
						$job_detail{$vo}{$queue}{$job}{"end"}=$3;
						$job_detail{$vo}{$queue}{$job}{"exit_status"}=$4;
						my $cput_h=$5;
						my $cput_m=$6;
						my $cput_s=$7;
						$job_detail{$vo}{$queue}{$job}{"used_cput"}=$cput_s+$cput_m*60+$cput_h*3600;
						my $wallt_h=$10;
						my $wallt_m=$11;
						my $wallt_s=$12;
						$job_detail{$vo}{$queue}{$job}{"used_walltime"}=$wallt_s+$wallt_m*60+$wallt_h*3600;
						if($job_detail{$vo}{$queue}{$job}{"used_walltime"}<0){$job_detail{$vo}{$queue}{$job}{"used_walltime"}=$job_detail{$vo}{$queue}{$job}{"end"}-$job_detail{$vo}{$queue}{$job}{"start"};}
						$job_detail{$vo}{$queue}{$job}{"used_mem"}=$8;
						$job_detail{$vo}{$queue}{$job}{"used_vmem"}=$9;
					}
					elsif($log=~/;E;.+start=(\d+).+exec_host=([\w-]+)\..+end=(\d+)\s+Exit_status=(\d+)/){
						$job_detail{$vo}{$queue}{$job}{"status"}="E";
						$job_detail{$vo}{$queue}{$job}{"start"}=$1;
						$job_detail{$vo}{$queue}{$job}{"exec_host"}=$2;
						$job_detail{$vo}{$queue}{$job}{"end"}=$3;
						$job_detail{$vo}{$queue}{$job}{"exit_status"}=$4;
						$job_detail{$vo}{$queue}{$job}{"used_cput"}="0";
						$job_detail{$vo}{$queue}{$job}{"used_walltime"}="0";
						$job_detail{$vo}{$queue}{$job}{"used_mem"}="0";
						$job_detail{$vo}{$queue}{$job}{"used_vmem"}="0";
					}
					elsif($log=~/^(.+);D;.+requestor=(\w+)\@/){
						$job_detail{$vo}{$queue}{$job}{"status"}="E";
						$job_detail{$vo}{$queue}{$job}{"start"}="00";
						$job_detail{$vo}{$queue}{$job}{"end"}=`date -d "$1" +%s 2> /dev/null`;
						chop($job_detail{$vo}{$queue}{$job}{"end"});
						if($2=~/root/){
							$job_detail{$vo}{$queue}{$job}{"exit_status"}="-201";
						}
						else{$job_detail{$vo}{$queue}{$job}{"exit_status"}="-200";}
						$job_detail{$vo}{$queue}{$job}{"used_cput"}="0";
                                                $job_detail{$vo}{$queue}{$job}{"used_walltime"}="0";
                                                $job_detail{$vo}{$queue}{$job}{"used_mem"}="0";
                                                $job_detail{$vo}{$queue}{$job}{"used_vmem"}="0";
					}
				}
				elsif(($job_detail{$vo}{$queue}{$job}{"status"} eq "E") and (!($job_detail{$vo}{$queue}{$job}{"end"})) and ($act!=$job_detail{$vo}{$queue}{$job}{"r_proc"}) and !($qstat{$job})){
					my $log=`grep ";E;$job." $pbs_log 2>/dev/null`;
                                        unless($log){
                                                $log=`grep ";E;$job." $pbs_log_y 2>/dev/null`;
                                        }
                                        unless($log){
                                                $log=`grep ";D;$job." $pbs_log 2>/dev/null`;
                                        }
                                        unless($log){
                                                $log=`grep ";D;$job." $pbs_log_y 2>/dev/null`;
                                        }
					unless($log){
                                                my $f_act=`grep -h -P ";(Q|S);$job." $pbs_log 2>/dev/null | tail -n1 2>/dev/null`;
                                                next if($f_act);
                                        }
					unless($log){
                                                $log=`grep -h -P ";(D|E);$job." $log_path/* 2>/dev/null | tail -n1 2>/dev/null` unless($t_update);
                                        }
					if($log=~/;E;.+start=(\d+).+exec_host=([\w-]+)\..+end=(\d+)\s+Exit_status=(.+)\s+resources_used.cput=(\S*\d+):(\S*\d+):(\S*\d+)\s+resources_used.mem=(\d+)kb\s+resources_used.vmem=(\d+)kb\s+resources_used.walltime=(\S*\d+):(\S*\d+):(\S*\d+)/){
						$job_detail{$vo}{$queue}{$job}{"status"}="E";
						$job_detail{$vo}{$queue}{$job}{"start"}=$1;
						$job_detail{$vo}{$queue}{$job}{"exec_host"}=$2;
						$job_detail{$vo}{$queue}{$job}{"end"}=$3;
						$job_detail{$vo}{$queue}{$job}{"exit_status"}=$4;
						my $cput_h=$5;
						my $cput_m=$6;
						my $cput_s=$7;
						$job_detail{$vo}{$queue}{$job}{"used_cput"}=$cput_s+$cput_m*60+$cput_h*3600;
						my $wallt_h=$10;
						my $wallt_m=$11;
						my $wallt_s=$12;
						$job_detail{$vo}{$queue}{$job}{"used_walltime"}=$wallt_s+$wallt_m*60+$wallt_h*3600;
						if($job_detail{$vo}{$queue}{$job}{"used_walltime"}<0){$job_detail{$vo}{$queue}{$job}{"used_walltime"}=$job_detail{$vo}{$queue}{$job}{"end"}-$job_detail{$vo}{$queue}{$job}{"start"};}
						$job_detail{$vo}{$queue}{$job}{"used_mem"}=$8;
						$job_detail{$vo}{$queue}{$job}{"used_vmem"}=$9;
					}
					elsif($log=~/;E;.+start=(\d+).+exec_host=([\w-]+)\..+end=(\d+)\s+Exit_status=(\d+)/){
						$job_detail{$vo}{$queue}{$job}{"status"}="E";
						$job_detail{$vo}{$queue}{$job}{"start"}=$1;
						$job_detail{$vo}{$queue}{$job}{"exec_host"}=$2;
						$job_detail{$vo}{$queue}{$job}{"end"}=$3;
						$job_detail{$vo}{$queue}{$job}{"exit_status"}=$4;
						$job_detail{$vo}{$queue}{$job}{"used_cput"}="0";
						$job_detail{$vo}{$queue}{$job}{"used_walltime"}="0";
						$job_detail{$vo}{$queue}{$job}{"used_mem"}="0";
						$job_detail{$vo}{$queue}{$job}{"used_vmem"}="0";
					}
					elsif($log=~/^(.+);D;.+requestor=(\w+)\@/){
						$job_detail{$vo}{$queue}{$job}{"status"}="E";
						$job_detail{$vo}{$queue}{$job}{"start"}="00";
						$job_detail{$vo}{$queue}{$job}{"end"}=`date -d "$1" +%s 2> /dev/null`;
						chop($job_detail{$vo}{$queue}{$job}{"end"});
						if($2=~/root/){
							$job_detail{$vo}{$queue}{$job}{"exit_status"}="-201";
						}
						else{$job_detail{$vo}{$queue}{$job}{"exit_status"}="-200";}
						$job_detail{$vo}{$queue}{$job}{"used_cput"}="0";
                                                $job_detail{$vo}{$queue}{$job}{"used_walltime"}="0";
                                                $job_detail{$vo}{$queue}{$job}{"used_mem"}="0";
                                                $job_detail{$vo}{$queue}{$job}{"used_vmem"}="0";
					}
				}
			}
		}
	}
	foreach $file (keys(%proc)){
		if($debug){
			my $f=$proc{$file};
			`cp "$f" "$f.backup_p" 2>/dev/null`;
		}
		unlink($proc{$file});
	}
	opendir(DIR,"$jm_dir/new/");
	@list_new=readdir(DIR);
	closedir(DIR);
	@list_new=grep !/\./,@list_new;
	foreach my $njob (@list_new){
		my %ref=%{retrieve("$jm_dir/new/$njob")};
		foreach my $vo (keys(%ref)){
			foreach my $queue (keys(%{$ref{$vo}})){
				if($ref{$vo}{$queue}{$njob}{"id"}){
					$job_detail{$vo}{$queue}{$njob}=$ref{$vo}{$queue}{$njob} unless(defined($job_detail{$vo}{$queue}{$njob}{"id"}));
					unlink("$jm_dir/new/$njob");
				}
			}
		}
	}
	opendir(DIR,"$jm_dir/ended/");
	@list_end=readdir(DIR);
	closedir(DIR);
	@list_end=grep !/\./,@list_end;
	foreach my $ejob (@list_end){
		my %ref=%{retrieve("$jm_dir/ended/$ejob")};
		foreach my $vo (keys(%ref)){
			foreach my $queue (keys(%{$ref{$vo}})){
				if($ref{$vo}{$queue}{$ejob}{"id"}){
					unless($job_detail{$vo}{$queue}{$ejob}{"status"} eq "E" and $job_detail{$vo}{$queue}{$ejob}{"end"}=~/^\d+$/){
						$job_detail{$vo}{$queue}{$ejob}{"status"}="E";
						$job_detail{$vo}{$queue}{$ejob}{"id"}=$ref{$vo}{$queue}{$ejob}{"id"};
						$job_detail{$vo}{$queue}{$ejob}{"grid_id"}=$ref{$vo}{$queue}{$ejob}{"grid_id"} unless($job_detail{$vo}{$queue}{$ejob}{"grid_id"}=~/https/);
						$job_detail{$vo}{$queue}{$ejob}{"user"}=$ref{$vo}{$queue}{$ejob}{"user"};
						$job_detail{$vo}{$queue}{$ejob}{"group"}=$ref{$vo}{$queue}{$ejob}{"group"};
						$job_detail{$vo}{$queue}{$ejob}{"qtime"}=$ref{$vo}{$queue}{$ejob}{"qtime"};
						$job_detail{$vo}{$queue}{$ejob}{"queue"}=$ref{$vo}{$queue}{$ejob}{"queue"};
						$job_detail{$vo}{$queue}{$ejob}{"name"}=$ref{$vo}{$queue}{$ejob}{"name"};
						$job_detail{$vo}{$queue}{$ejob}{"subject"}=$ref{$vo}{$queue}{$ejob}{"subject"};
						$job_detail{$vo}{$queue}{$ejob}{"start"}=$ref{$vo}{$queue}{$ejob}{"start"};
						$job_detail{$vo}{$queue}{$ejob}{"exec_host"}=$ref{$vo}{$queue}{$ejob}{"exec_host"};
						$job_detail{$vo}{$queue}{$ejob}{"end"}=$ref{$vo}{$queue}{$ejob}{"end"};
						$job_detail{$vo}{$queue}{$ejob}{"exit_status"}=$ref{$vo}{$queue}{$ejob}{"exit_status"};
						$job_detail{$vo}{$queue}{$ejob}{"used_cput"}=$ref{$vo}{$queue}{$ejob}{"used_cput"};
						$job_detail{$vo}{$queue}{$ejob}{"used_walltime"}=$ref{$vo}{$queue}{$ejob}{"used_walltime"};
						$job_detail{$vo}{$queue}{$ejob}{"used_mem"}=$ref{$vo}{$queue}{$ejob}{"used_mem"};
						$job_detail{$vo}{$queue}{$ejob}{"used_vmem"}=$ref{$vo}{$queue}{$ejob}{"used_vmem"};
					}
					unlink("$jm_dir/ended/$ejob");
				}
			}
		}
	}
	store(\%job_detail,"$jm_dir/processed/.jmgridice_job_detail");
}


#############################################
###                                       ###
###      LSF                              ###
###                                       ###
#############################################

elsif($lrms =~ /lsf/i){
	my $check_log=`ls -l $log_path/lsb.acct 2>/dev/null`;
	unless($check_log){
		print "lsf\$NO_LOG";
		exit 0;
	}
	if(-e "$jm_dir/processed/.jmgridice_job_detail"){
		%job_detail=%{retrieve("$jm_dir/processed/.jmgridice_job_detail")};
	}
	else{%job_detail=();}
	my $date = `date +%Y%m%d 2>/dev/null`;
	my $yesterday=`date -d '1 day ago' +%Y%m%d 2>/dev/null`;
	chop($date);
	chop($yesterday);
	my $z=0;
	my $lsf_acct="$jm_dir/acct_output";
	my $lsf_acct_y="$jm_dir/acct_output.1";
	my $since_mid=`date +%S 2>/dev/null`+`date +%M 2>/dev/null`*60+`date +%k 2>/dev/null`*3600;
	$interval=$interval*3600;

	unless(-e "$jm_dir/processed/.jmgridice_acc_lines"){
		@ended=`cat $lsf_acct 2>/dev/null`;
		my $count=$#acct+1;
		open(COUNT,">$jm_dir/processed/.jmgridice_acc_lines");
		print COUNT $#acct+1;
		close(COUNT);
	}
	else{
		open(ACC,"<$jm_dir/processed/.jmgridice_acc_lines");
		$acc_lines=<ACC>;
		close(ACC);
		my $acount = `wc -l $lsf_acct 2>/dev/null`;
                $acount =~/(\d+)/;
                $acount = $1;
		chop($acount);
		if($acount > $acc_lines){
			my $tail_c=$acc_lines+1;
			@ended =`tail -n +$tail_c $lsf_acct 2>/dev/null`;
			open(COUNT,">$jm_dir/processed/.jmgridice_acc_lines");
			print COUNT $acount;
			close(COUNT);
		}
		elsif($acount < $acc_lines){
			my $tail_c=$acc_lines+1;
			my @ended_tmp = `tail -n +$tail_c $lsf_acct_y 2>/dev/null`;
			@ended = `cat $lsf_acct 2>/dev/null`;
			open(COUNT,">$jm_dir/processed/.jmgridice_acc_lines");
			print COUNT $#ended+1;
			close(COUNT);
			push(@ended,@ended_tmp);
		}
	}
	foreach $job_log (@ended){
		if($job_log=~/jobid:(\d+).+end:(\d+)/){
			my $eid=$1;
			my $end=$2;
			my $diff=$act-$end;
			if(($diff < $interval) or (-e "$jm_dir/ended/$eid")){
				$job_ended{$eid}{"log"}=$job_log;
			}
		}
	}
	if($since_mid<$interval){
		my @ended=`cat $lsf_acct_y 2>/dev/null`;
		foreach $job_log (@ended){
			if($job_log=~/jobid:(\d+).+end:(\d+)/){
				my $eid=$1;
				my $end=$2;
				my $diff=$act-$end;
				if(($diff < $interval) or (-e "$jm_dir/ended/$eid")){
					$job_ended{$eid}{"log"}=$job_log;
				}
			}
		}
	}

##############################
##       BACKUP             ##
##############################

	if($debug){
		if(-e "$jm_dir/processed/$date"){
			%backup=%{retrieve("$jm_dir/processed/$date")};
		}
		else{%backup=();}
		if(-e "$jm_dir/processed/$yesterday"){
			%backup_tmp=%{retrieve("$jm_dir/processed/$yesterday")};
		}
		else{%backup_tmp=();}
		my $midnight=(`date +%s 2>/dev/null`)-(`date +%S 2>/dev/null`+`date +%M 2>/dev/null`*60+`date +%k 2>/dev/null`*3600);
		my $flag_y=0;
		my $flag_yb=0;
		foreach my $del (keys(%job_detail)){
			foreach my $del1 (keys(%{$job_detail{$del}})){
				foreach my $del2 (keys(%{$job_detail{$del}{$del1}})){
					if($job_detail{$del}{$del1}{$del2}{"status"} eq "E" and $job_detail{$del}{$del1}{$del2}{"end"}){
						$fin_job{$del2}=$del2;
						my $end=$job_detail{$del}{$del1}{$del2}{"end"};
						my $diff=$act-$end;
						if($diff >= $interval){
							unless($backup{$del}{$del1}{$del2}{"id"}){
								if($job_detail{$del}{$del1}{$del2}{"end"}>=$midnight){
									$backup{$del}{$del1}{$del2}=$job_detail{$del}{$del1}{$del2};
									$flag_y=1;
								}
								else{
									unless($backup_tmp{$del}{$del1}{$del2}{"id"}){
										$backup_tmp{$del}{$del1}{$del2}=$job_detail{$del}{$del1}{$del2};
										$flag_yb=1;
									}
								}
							}
							delete($job_detail{$del}{$del1}{$del2});
						}
					}
				}
			}
		}
		if($flag_y==1){
			store(\%backup,"$jm_dir/processed/$date");
		}
		else{%backup=();}
		if($flag_yb==1){
			store(\%backup_tmp,"$jm_dir/processed/$yesterday");
		}
		else{%backup_tmp=();}
	}
	else{
		foreach my $del (keys(%job_detail)){
			foreach my $del1 (keys(%{$job_detail{$del}})){
				foreach my $del2 (keys(%{$job_detail{$del}{$del1}})){
					if($job_detail{$del}{$del1}{$del2}{"status"} eq "E" and $job_detail{$del}{$del1}{$del2}{"end"}){
						my $end=$job_detail{$del}{$del1}{$del2}{"end"};
						my $diff=$act-$end;
						if($diff >= $interval){
							delete($job_detail{$del}{$del1}{$del2});
						}
					}
				}
			}
		}
	}


##############################
##       ENDED JOBS         ##
##############################

	foreach my $job (keys(%job_ended)){
		unless($fin_job{$job}){
			if(-e "$jm_dir/ended/$job.store"){
				$proc{$z}="$jm_dir/ended/$job.store";
				$z++;
				my %ref_new=%{retrieve("$jm_dir/ended/$job.store")};
				foreach my $vo (keys(%ref_new)){
					foreach my $queue (keys(%{$ref_new{$vo}})){
						unless($job_detail{$vo}{$queue}{$job}{"status"} eq "E" and $job_detail{$vo}{$queue}{$job}{"end"}){
							if($ref_new{$vo}{$queue}{$job}{"id"} and ($act-($ref_new{$vo}{$queue}{$job}{"end"}))<$interval){
								if($ref_new{$vo}{$queue}{$job}{"exit_status"}==0){
									$job_detail{$vo}{$queue}{$job}{"status"}="E";
									$job_detail{$vo}{$queue}{$job}{"id"}=$ref_new{$vo}{$queue}{$job}{"id"};
									$job_detail{$vo}{$queue}{$job}{"grid_id"}=$ref_new{$vo}{$queue}{$job}{"grid_id"} unless($job_detail{$vo}{$queue}{$job}{"grid_id"}=~/https/);
									$job_detail{$vo}{$queue}{$job}{"user"}=$ref_new{$vo}{$queue}{$job}{"user"};
									$job_detail{$vo}{$queue}{$job}{"group"}=$ref_new{$vo}{$queue}{$job}{"group"};
									$job_detail{$vo}{$queue}{$job}{"qtime"}=$ref_new{$vo}{$queue}{$job}{"qtime"};
									$job_detail{$vo}{$queue}{$job}{"queue"}=$ref_new{$vo}{$queue}{$job}{"queue"};
									$job_detail{$vo}{$queue}{$job}{"exit_status"}=$ref_new{$vo}{$queue}{$job}{"exit_status"};
									$job_detail{$vo}{$queue}{$job}{"exec_host"}=$ref_new{$vo}{$queue}{$job}{"exec_host"};
									$job_detail{$vo}{$queue}{$job}{"name"}=$ref_new{$vo}{$queue}{$job}{"name"};
									$job_detail{$vo}{$queue}{$job}{"subject"}=$ref_new{$vo}{$queue}{$job}{"subject"};
									$job_detail{$vo}{$queue}{$job}{"used_cput"}=$ref_new{$vo}{$queue}{$job}{"used_cput"};
									$job_detail{$vo}{$queue}{$job}{"used_walltime"}=$ref_new{$vo}{$queue}{$job}{"used_walltime"};
									$job_detail{$vo}{$queue}{$job}{"used_mem"}=$ref_new{$vo}{$queue}{$job}{"used_mem"};
									$job_detail{$vo}{$queue}{$job}{"used_vmem"}=$ref_new{$vo}{$queue}{$job}{"used_vmem"};
									$job_detail{$vo}{$queue}{$job}{"start"}=$ref_new{$vo}{$queue}{$job}{"start"};
									$job_detail{$vo}{$queue}{$job}{"end"}=$ref_new{$vo}{$queue}{$job}{"end"};
								}
								else{
									unless($job_detail{$vo}{$queue}{$job}{"id"}){
										$job_detail{$vo}{$queue}{$job}=$ref_new{$vo}{$queue}{$job};
										$job_detail{$vo}{$queue}{$job}{"status"}="E";
									}
									elsif($job_detail{$vo}{$queue}{$job}{"status"} eq "R"){
										$job_detail{$vo}{$queue}{$job}{"status"}="E";
										$job_detail{$vo}{$queue}{$job}{"grid_id"}=$ref_new{$vo}{$queue}{$job}{"grid_id"} unless($job_detail{$vo}{$queue}{$job}{"grid_id"}=~/https/);
										$job_detail{$vo}{$queue}{$job}{"end"}=$ref_new{$vo}{$queue}{$job}{"end"};
										$job_detail{$vo}{$queue}{$job}{"exit_status"}=$ref_new{$vo}{$queue}{$job}{"exit_status"};
									}
									elsif($job_detail{$vo}{$queue}{$job}{"status"} eq "Q"){
										$job_detail{$vo}{$queue}{$job}{"status"}="E";
										$job_detail{$vo}{$queue}{$job}{"end"}=$ref_new{$vo}{$queue}{$job}{"end"};
										$job_detail{$vo}{$queue}{$job}{"exit_status"}=$ref_new{$vo}{$queue}{$job}{"exit_status"};
										$job_detail{$vo}{$queue}{$job}{"exec_host"}=$ref_new{$vo}{$queue}{$job}{"exec_host"};
										$job_detail{$vo}{$queue}{$job}{"used_cput"}=$ref_new{$vo}{$queue}{$job}{"used_cput"};
										$job_detail{$vo}{$queue}{$job}{"used_walltime"}=$ref_new{$vo}{$queue}{$job}{"used_walltime"};
										$job_detail{$vo}{$queue}{$job}{"used_mem"}=$ref_new{$vo}{$queue}{$job}{"used_mem"};
										$job_detail{$vo}{$queue}{$job}{"used_vmem"}=$ref_new{$vo}{$queue}{$job}{"used_vmem"};
										$job_detail{$vo}{$queue}{$job}{"start"}=$ref_new{$vo}{$queue}{$job}{"start"};
									}
								}
							}
						}
					}
				}
			}
			elsif(-e "$jm_dir/new/$job.store"){
				my %ref_new=%{retrieve("$jm_dir/new/$job.store")};
				foreach my $vo (keys(%ref_new)){
					foreach my $queue (keys(%{$ref_new{$vo}})){
						if($ref_new{$vo}{$queue}{$job}{"id"}){
							unless($job_detail{$vo}{$queue}{$job}{"status"} eq "E" and $job_detail{$vo}{$queue}{$job}{"end"}){
								if($job_ended{$job}{"log"}=~/start:(\d+)\|exec_host:(\d+.+)end:(\d+).+cputime:(\d+)\|walltime:(\d+)\|memory:(\d+)\|vmemory:(\d+)\|exit_status:(\d+)\|/){
									$proc{$z}="$jm_dir/new/$job.store";
									$z++;
									if(($act-$3)<$interval){
										$job_detail{$vo}{$queue}{$job}=$ref_new{$vo}{$queue}{$job};
										$job_detail{$vo}{$queue}{$job}{"status"}="E";
										$job_detail{$vo}{$queue}{$job}{"start"}=$1;
										$job_detail{$vo}{$queue}{$job}{"end"}=$3;
										if($8==64){
                                                                                        $job_detail{$vo}{$queue}{$job}{"exit_status"}=0;
                                                                                }
                                                                                else{
                                                                                        if(`grep $job:id_"root" $jm_dir/killed* 2>/dev/null`){
                                                                                                $job_detail{$vo}{$queue}{$job}{"exit_status"}="201";
                                                                                        }
                                                                                        elsif(`grep $job:id_ $jm_dir/killed* 2>/dev/null`){
                                                                                                $job_detail{$vo}{$queue}{$job}{"exit_status"}="200";
                                                                                        }
                                                                                        else{$job_detail{$vo}{$queue}{$job}{"exit_status"}=$8;}
                                                                                }
										$job_detail{$vo}{$queue}{$job}{"used_cput"}=$4;
										$job_detail{$vo}{$queue}{$job}{"used_walltime"}=$5;
										$job_detail{$vo}{$queue}{$job}{"used_mem"}=$7;
										$job_detail{$vo}{$queue}{$job}{"used_vmem"}=$8;
										my $exh="NO_HOST";
										my $host=$2;
										$host=~/(\d+)(.+)/;
										if($1>0){
											my @tmp_h=split(/\|/,$2);
											$tmp_h[1]=~/"(\S+)"/;
											$exh=$1;
										}
										$job_detail{$vo}{$queue}{$job}{"exec_host"}=$exh;
									}
								}
							}
						}
					}
				}
			}
		}
	}

##############################
##     bjobs -u all -l      ##
##############################

	my $globus_loc_buf=`grep GLOBUS_LOCATION= /etc/globus.conf 2> /dev/null`;
	my ($globus_loc)=($globus_loc_buf=~ /GLOBUS_LOCATION=(.+)/);
	my $command_buff_search=`grep BSUB $globus_loc/libexec/globus-sh-tools-vars.sh 2> /dev/null | sed "s/GLOBUS_SH_BSUB=//g" 2> /dev/null`;
	my ($command_search)=($command_buff_search=~ /"(.+)\/bsub"/);
	unless($command_search=~/\w+/){
		$command_search=`whereis bjobs 2>/dev/null`;
		if($command_search=~/bjobs:\s+(\S+)/){
			my $tmp_command=$1;
			my $ver=`file $1 2>/dev/null`;
			if($ver=~/executable/){
				$command=$tmp_command;
			}
			else{
				$command_search=`which bjobs 2>/dev/null`;
				if($command_search=~/^(\S+qstat)/){
					$command=$1;
				}
				else{$command="bjobs";}
			}
		}
		else{
			$command_search=`which bjobs 2>/dev/null`;
			if($command_search=~/^(\S+bjobs)/){
				$command=$1;
			}
			else{$command="bjobs";}
		}
	}
	else{$command="$command_search/bjobs";}
	my $ce_name=`hostname -s 2>/dev/null`;
	chop($ce_name);

	my %job_proc=();
	foreach my $a (keys(%job_detail)){
		foreach my $b (keys(%{$job_detail{$a}})){
			foreach my $c (keys(%{$job_detail{$a}{$b}})){
				$job_proc{$c}{"status"}=$job_detail{$a}{$b}{$c}{"status"};
				$job_proc{$c}{"vo"}=$a;
				$job_proc{$c}{"queue"}=$b;
			}
		}
	}

	my $valid_list_r="";
	my @job_list=`$command -u all -r 2>/dev/null | grep $ce_name 2>/dev/null`;
	push(@all_jobs,@job_list);
	my @tmp_jobs=();
	foreach my $bjobs_line (@job_list){
		if($bjobs_line=~/^(\d+) /){
			$id=$1;
			$bjobs{$id}=$id;
			unless($update){
				if($job_proc{$id}{"status"} or (-e "$jm_dir/new/$id.store")){$valid_list_r.=" $id";}
				else{push(@tmp_jobs,$id);}
			}
			else{
				if(-e "$jm_dir/new/$id.start"){
					unlink("$jm_dir/new/$id.start");
					if($job_proc{$id}{"status"} or (-e "$jm_dir/new/$id.store")){$valid_list_r.=" $id";}
					else{push(@tmp_jobs,$id);}
				}
			}
		}
	}
	$found="";
	@job_list=`$command -u all -p 2>/dev/null | grep $ce_name 2>/dev/null | grep -P '^\\d+' 2>/dev/null`;
	push(@all_jobs,@job_list);
	foreach $bjobs_line (@job_list){
		if($bjobs_line=~/^(\d+) /){
			$id=$1;
			$bjobs{$id}=$id;
			unless($job_proc{$id}{"status"}){
				if(-e "$jm_dir/new/$id.store"){
					$proc{$z}="$jm_dir/new/$id.store";
					$z++;
					my %ref=%{retrieve("$jm_dir/new/$id.store")};
					foreach $vo (keys(%ref)){
						foreach $queue (keys(%{$ref{$vo}})){
							if($ref{$vo}{$queue}{$id}{"id"}){
								$job_detail{$vo}{$queue}{$id}=$ref{$vo}{$queue}{$id};
								$job_detail{$vo}{$queue}{$id}{"r_proc"}=$act;
							}
						}
					}
				}
				elsif(-e "$jm_dir/new/$id"){
					$proc{$z}="$jm_dir/new/$id";
					$z++;
					open(NEW,"$jm_dir/new/$id");
					my $line=<NEW>;
					close(NEW);
					if($line=~/submitted:(\d+)\|user:"(\S+)"\|queue:"(\S+)"\|grid_id:(\S+)\|/){
						$vo=`groups $2 2>/dev/null | sed 's/$2 : //g' 2>/dev/null`;
						chop($vo);
						$queue=$3;
						$job_detail{$vo}{$queue}{$id}{"end"}=undef;
						$job_detail{$vo}{$queue}{$id}{"start"}=undef;
						$job_detail{$vo}{$queue}{$id}{"used_cput"}=undef;
						$job_detail{$vo}{$queue}{$id}{"used_walltime"}=undef;
						$job_detail{$vo}{$queue}{$id}{"used_mem"}=undef;
						$job_detail{$vo}{$queue}{$id}{"used_vmem"}=undef;
						$job_detail{$vo}{$queue}{$id}{"exit_status"}=undef;
						$job_detail{$vo}{$queue}{$id}{"exec_host"}=undef;
						$job_detail{$vo}{$queue}{$id}{"qtime"}=$1;
						$job_detail{$vo}{$queue}{$id}{"id"}=$id;
						$job_detail{$vo}{$queue}{$id}{"group"}=$vo;
						$job_detail{$vo}{$queue}{$id}{"queue"}=$queue;
						$job_detail{$vo}{$queue}{$id}{"grid_id"}=$4;
						$job_detail{$vo}{$queue}{$id}{"user"}=$2;
						if(-e "$jm_dir/subject/$2"){
							$subject=`cat $jm_dir/subject/$2 2> /dev/null`;
						}
						else{
							my $sbj=`grep $2 /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
							$sbj=~/"(.+)"\s+mapped/;
							$subject=$1;
						}
						$job_detail{$vo}{$queue}{$id}{"subject"}=$subject;
						$job_detail{$vo}{$queue}{$id}{"name"}=$id;
						$job_detail{$vo}{$queue}{$id}{"status"}="Q";
						$job_detail{$vo}{$queue}{$id}{"r_proc"}=$act;
					}
				}
				else{push(@tmp_jobs,$id);}
			}
			else{
                                my $vo=$job_proc{$id}{"vo"};
                                my $queue=$job_proc{$id}{"queue"};
                                $job_detail{$vo}{$queue}{$id}{"r_proc"}=$act;
                        }
		}
	}
	if($#tmp_jobs ne "-1"){
		$found=find(@tmp_jobs);
		$valid_list_r.=" $found";
	}
	if($valid_list_r=~/\d+/){
		@job_list=`$command -u all -l $valid_list_r 2>/dev/null`;
		$k=0;
		while($job_list[$k]){
			if($job_list[$k]=~/Job <(\d+)>/){
				$vo="";
				$queue="";
				$user="";
				$id=$1;
				if($job_list[$k]=~/User <(\S+)>.+Queue <(\S+)>/){
					$vo=`groups $1 2>/dev/null | sed 's/$1 : //g' 2>/dev/null`;
					chop($vo);
					$user=$1;
					$queue=$2;
				}
				else{
					my $tmp_joblist=$job_list[$k].$job_list[$k+1];
					$tmp_joblist=~s/\s{2,}//g;
					if($tmp_joblist=~/User <(\S+)>.+Queue <(\S+)>/){
						$vo=`groups $1 2>/dev/null | sed 's/$1 : //g' 2>/dev/null`;
                                                chop($vo);
                                                $user=$1;
						$queue=$2;
					}
				}
				if(-e "$jm_dir/new/$id.store"){
					$proc{$z}="$jm_dir/new/$id.store";
					$z++;
					my %ref=%{retrieve("$jm_dir/new/$id.store")};
					foreach my $vo_tmp (keys(%ref)){
						foreach my $queue_tmp (keys(%{$ref{$vo_tmp}})){
							if($ref{$vo_tmp}{$queue_tmp}{$id}{"id"}){
								$vo=$vo_tmp;
								$queue=$queue_tmp;
								$job_detail{$vo}{$queue}{$id}=$ref{$vo}{$queue}{$id};
								$user=$ref{$vo}{$queue}{$id}{"user"};
								if(-e "$jm_dir/subject/$user"){
									$subject=`cat $jm_dir/subject/$user 2> /dev/null`;
								}
								else{
									my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
									$sbj=~/"(.+)"\s+mapped/;
									$subject=$1;
								}
								$job_detail{$vo}{$queue}{$id}{"subject"}=$subject;
								$job_detail{$vo}{$queue}{$id}{"r_proc"}=$act;
								$job_detail{$vo}{$queue}{$id}{"status"}="R";
							}
						}
					}
				}
				elsif(-e "$jm_dir/new/$id" and $job_proc{$id}{"status"} ne "R"){
					open(NEW,"$jm_dir/new/$id");
					my $job=<NEW>;
					close(NEW);
					$proc{$z}="$jm_dir/new/$id";
                                        $z++;
					if($job=~/submitted:(\d+)\|user:"(\S+)"\|queue:"(\S+)"\|grid_id:(\S+)\|/){
						$vo=`groups $2 2>/dev/null | sed 's/$2 : //g' 2>/dev/null`;
						chop($vo);
						$queue=$3;
						$job_detail{$vo}{$queue}{$id}{"end"}=undef;
						$job_detail{$vo}{$queue}{$id}{"exit_status"}=undef;
						$job_detail{$vo}{$queue}{$id}{"qtime"}=$1;
						$job_detail{$vo}{$queue}{$id}{"id"}=$id;
						$job_detail{$vo}{$queue}{$id}{"group"}=$vo;
						$job_detail{$vo}{$queue}{$id}{"queue"}=$queue;
						$job_detail{$vo}{$queue}{$id}{"grid_id"}=$4;
						$job_detail{$vo}{$queue}{$id}{"user"}=$2;
						if(-e "$jm_dir/subject/$2"){
							$subject=`cat $jm_dir/subject/$2 2> /dev/null`;
						}
						else{
							my $sbj=`grep $2 /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
							$sbj=~/"(.+)"\s+mapped/;
							$subject=$1;
						}
						$job_detail{$vo}{$queue}{$id}{"subject"}=$subject;
						$job_detail{$vo}{$queue}{$id}{"name"}=$id;
						$job_detail{$vo}{$queue}{$id}{"r_proc"}=$act;
						$job_detail{$vo}{$queue}{$id}{"status"}="R";
					}
					else{
						unless($job_proc{$id}{"status"}){
							if($job_list[$k]=~/User <(\S+)>.+Queue <(\S+)>/){
								$vo=`groups $1 2>/dev/null | sed 's/$1 : //g' 2>/dev/null`;
								chop($vo);
								$queue=$2;
								$user=$1;
							}
							else{
								my $tmp_line=$job_list[$k];
								chop($tmp_line);
								$tmp_line.=$job_list[$k+1];
								$tmp_line=~/User <(\S+)>.Queue <(\S+)>/;
								$vo=`groups $1 2>/dev/null | sed 's/$1 : //g' 2>/dev/null`;
								chop($vo);
								$queue=$2;
								$user=$1;
							}
							$job_detail{$vo}{$queue}{$id}{"end"}=undef;
							$job_detail{$vo}{$queue}{$id}{"exit_status"}=undef;
							$job_detail{$vo}{$queue}{$id}{"id"}=$id;
							$job_detail{$vo}{$queue}{$id}{"group"}=$vo;
							$job_detail{$vo}{$queue}{$id}{"queue"}=$queue;
							$job_detail{$vo}{$queue}{$id}{"user"}=$user;
							if(-e "$jm_dir/subject/$user"){
								$subject=`cat $jm_dir/subject/$user 2> /dev/null`;
							}
							else{
								my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
								$sbj=~/"(.+)"\s+mapped/;
								$subject=$1;
							}
							$job_detail{$vo}{$queue}{$id}{"subject"}=$subject;
							my $tmp_line=$job_list[$k];
							until($job_list[$k]=~/Submitted from host/){
								$k++;
								chop($tmp_line);
								$tmp_line.=$job_list[$k];
							}
							$tmp_line=~s/\s+//g;
							if($tmp_line=~/"(\S+\/globus-cache-export[^\/]+\/)globus-cache-export/){
								$grid_id=`grep -h -E 'EDG_WL_JOBID|GLITE_WMS_JOBID' $1* 2>/dev/null | grep https 2>/dev/null | sed 's/\\s*export\\s*//' 2>/dev/null`;
								unless($grid_id){
									$grid_id=`grep JobWrapper $1* 2>/dev/null | grep https 2>/dev/null`;
									if($grid_id=~/JobWrapper\.(\S+)\.sh/){
										$grid_id=$1;
										$grid_id=~s/_3a/:/g;
										$grid_id=~s/_2f/\//g;
									}
								}
							}
							$tmp_line="";
							if($grid_id=~/[EDG_WL_JOBID|GLITE_WMS_JOBID]="(\S+)"/){
								$job_detail{$vo}{$queue}{$id}{"grid_id"}=$1;
							}
							elsif($grid_id=~/https/){
								$job_detail{$vo}{$queue}{$id}{"grid_id"}=$grid_id;
							}
							else{$job_detail{$vo}{$queue}{$id}{"grid_id"}="n/a";}
							$grid_id="";
							$job_list[$k]=~/^(.+): Submitted from host/;
							$job_detail{$vo}{$queue}{$id}{"qtime"}=`date -d "$1" +%s 2>/dev/null`;
							chop($job_detail{$vo}{$queue}{$id}{"qtime"});
							if($job_detail{$vo}{$queue}{$id}{"qtime"} > $act){
								$job_detail{$vo}{$queue}{$id}{"qtime"}=`date -d "$1 $year" +%s 2>/dev/null`;
								chop($job_detail{$vo}{$queue}{$id}{"qtime"});
							}
							$job_detail{$vo}{$queue}{$id}{"name"}=$id;
							$job_detail{$vo}{$queue}{$id}{"r_proc"}=$act;
							$job_detail{$vo}{$queue}{$id}{"status"}="R";
						}
						else{
							$vo=$job_proc{$id}{"vo"};
							$queue=$job_proc{$id}{"queue"};
							$job_detail{$vo}{$queue}{$id}{"r_proc"}=$act;
							$job_detail{$vo}{$queue}{$id}{"status"}="R";
						}
					}
				}
				else{
					if($job_proc{$id}{"status"}){
						$vo=$job_proc{$id}{"vo"};
						$queue=$job_proc{$id}{"queue"};
					}
				}
			}
			elsif($job_list[$k]=~/^(.+): Submitted from/ and (($job_proc{$id}{"status"} ne "R") or !defined($job_proc{$id}{"status"}))){
				$job_detail{$vo}{$queue}{$id}{"status"}="Q";
                                $job_detail{$vo}{$queue}{$id}{"id"}=$id;
                                $job_detail{$vo}{$queue}{$id}{"name"}=$id;
                                $job_detail{$vo}{$queue}{$id}{"group"}=$vo if($vo);
                                $job_detail{$vo}{$queue}{$id}{"user"}=$user if($user);
                                $job_detail{$vo}{$queue}{$id}{"queue"}=$queue if($queue);
                                $job_detail{$vo}{$queue}{$id}{"qtime"}=`date -d "$1" +%s 2>/dev/null`;
                                chop($job_detail{$vo}{$queue}{$id}{"qtime"});
				if($job_detail{$vo}{$queue}{$id}{"qtime"} > $act){
					$job_detail{$vo}{$queue}{$id}{"qtime"}=`date -d "$1 $year" +%s 2>/dev/null`;
					chop($job_detail{$vo}{$queue}{$id}{"qtime"});
				}
                                $job_detail{$vo}{$queue}{$id}{"grid_id"}="n/a" unless($job_detail{$vo}{$queue}{$id}{"grid_id"}=~/https/);
			}
			elsif($job_list[$k]=~/^(.+): Started on \d*\s*\S*\s*<(\S+)>/ and (($job_proc{$id}{"status"} ne "R") or !defined($job_proc{$id}{"status"}))){
				$job_detail{$vo}{$queue}{$id}{"status"}="R";
                                $job_detail{$vo}{$queue}{$id}{"start"}=`date -d "$1" +%s 2>/dev/null`;
                                chop($job_detail{$vo}{$queue}{$id}{"start"});
				if($job_detail{$vo}{$queue}{$id}{"start"} > $act){
					$job_detail{$vo}{$queue}{$id}{"start"}=`date -d "$1 $year" +%s 2>/dev/null`;
					chop($job_detail{$vo}{$queue}{$id}{"start"});
				}
				$job_detail{$vo}{$queue}{$id}{"exec_host"}=$2;
			}
			elsif($job_list[$k]=~/CPU time used is (\d+)/){
				$job_detail{$vo}{$queue}{$id}{"used_cput"}=$1;
				$job_detail{$vo}{$queue}{$id}{"used_walltime"}=$act-$job_detail{$vo}{$queue}{$id}{"start"};
			}
			elsif($job_list[$k]=~/MEM: (\d+).+SWAP: (\d+)/){
				$job_detail{$vo}{$queue}{$id}{"used_mem"}=$1*1024;
				$job_detail{$vo}{$queue}{$id}{"used_vmem"}=$2*1024;
			}
			$k++;
			if($job_list[$k]=~/SCHEDULING PARAMETERS:/){
				$job_detail{$vo}{$queue}{$id}{"r_proc"}=$act;
				if($job_detail{$vo}{$queue}{$id}{"status"} eq "R"){
					$job_detail{$vo}{$queue}{$id}{"used_cput"}=0 unless($job_detail{$vo}{$queue}{$id}{"used_cput"});
					$job_detail{$vo}{$queue}{$id}{"used_walltime"}=$act-$job_detail{$vo}{$queue}{$id}{"start"} if(!defined($job_detail{$vo}{$queue}{$id}{"used_walltime"}) or $job_detail{$vo}{$queue}{$id}{"used_walltime"}==0);
					$job_detail{$vo}{$queue}{$id}{"used_mem"}=0 unless($job_detail{$vo}{$queue}{$id}{"used_mem"});
					$job_detail{$vo}{$queue}{$id}{"used_vmem"}=0 unless($job_detail{$vo}{$queue}{$id}{"used_vmem"});
				}
				unless($job_detail{$vo}{$queue}{$id}{"subject"}){
					$user=$job_detail{$vo}{$queue}{$id}{"user"};
					if(-e "$jm_dir/subject/$user"){
						$subject=`cat $jm_dir/subject/$user 2> /dev/null`;
					}
					else{
						my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
						$sbj=~/"(.+)"\s+mapped/;
						$subject=$1;
					}
					$job_detail{$vo}{$queue}{$id}{"subject"}=$subject;
				}
			}
		}
	}

	foreach my $vo (keys(%job_detail)){
		foreach my $queue (keys(%{$job_detail{$vo}})){
			foreach my $job (keys(%{$job_detail{$vo}{$queue}})){
				if(($job_detail{$vo}{$queue}{$job}{"status"} =~ /Q/) and ($act!=$job_detail{$vo}{$queue}{$job}{"r_proc"}) and !($bjobs{$job})){
					my $log=`grep "jobid:$job" $lsf_acct 2>/dev/null`;
					unless($log){
						$log=`grep "jobid:$job" $lsf_acct_y 2>/dev/null`;
					}
					if($log=~/start:(\d+)\|exec_host:(\d+.+)end:(\d+).+cputime:(\d+)\|walltime:(\d+)\|memory:(\d+)\|vmemory:(\d+)\|exit_status:(\d+)/){
						$job_detail{$vo}{$queue}{$job}{"status"}="E";
						$job_detail{$vo}{$queue}{$job}{"start"}=$1;
						$job_detail{$vo}{$queue}{$job}{"end"}=$3;
						if($8==64){
							$job_detail{$vo}{$queue}{$job}{"exit_status"}=0;
						}
						else{$job_detail{$vo}{$queue}{$job}{"exit_status"}=$8;}
						$job_detail{$vo}{$queue}{$job}{"used_cput"}=$4;
						$job_detail{$vo}{$queue}{$job}{"used_walltime"}=$5;
						$job_detail{$vo}{$queue}{$job}{"used_mem"}=$6;
						$job_detail{$vo}{$queue}{$job}{"used_vmem"}=$7;
						my $exh="NO_HOST";
						my $host=$2;
						$host=~/(\d+)(.+)/;
						if($1>0){
							my @tmp_h=split(/\|/,$2);
							$tmp_h[1]=~/"(\S+)"/;
							$exh=$1;
						}
						$job_detail{$vo}{$queue}{$job}{"exec_host"}=$exh;
					}
					else{delete($job_detail{$vo}{$queue}{$job}) if($act-$job_detail{$vo}{$queue}{$job}{"r_proc"}>$interval);}
				}
				elsif(($job_detail{$vo}{$queue}{$job}{"status"} =~ /R/) and ($act!=$job_detail{$vo}{$queue}{$job}{"r_proc"}) and !(defined($update)) and !($bjobs{$job})){
                                        my $log=`grep "jobid:$job" $lsf_acct 2>/dev/null`;
                                        unless($log){
                                                $log=`grep "jobid:$job" $lsf_acct_y 2>/dev/null`;
                                        }
                                        if($log=~/start:(\d+)\|exec_host:(\d+.+)end:(\d+).+cputime:(\d+)\|walltime:(\d+)\|memory:(\d+)\|vmemory:(\d+)\|exit_status:(\d+)/){
                                                $job_detail{$vo}{$queue}{$job}{"status"}="E";
                                                $job_detail{$vo}{$queue}{$job}{"start"}=$1;
                                                $job_detail{$vo}{$queue}{$job}{"end"}=$3;
                                                if($8==64){
                                                        $job_detail{$vo}{$queue}{$job}{"exit_status"}=0;
                                                }
                                                else{$job_detail{$vo}{$queue}{$job}{"exit_status"}=$8;}
                                                $job_detail{$vo}{$queue}{$job}{"used_cput"}=$4;
                                                $job_detail{$vo}{$queue}{$job}{"used_walltime"}=$5;
                                                $job_detail{$vo}{$queue}{$job}{"used_mem"}=$6;
                                                $job_detail{$vo}{$queue}{$job}{"used_vmem"}=$7;
                                                my $exh="NO_HOST";
                                                my $host=$2;
                                                $host=~/(\d+)(.+)/;
                                                if($1>0){
                                                        my @tmp_h=split(/\|/,$2);
                                                        $tmp_h[1]=~/"(\S+)"/;
                                                        $exh=$1;
                                                }
                                                $job_detail{$vo}{$queue}{$job}{"exec_host"}=$exh;
                                        }
                                        else{delete($job_detail{$vo}{$queue}{$job}) if($act-$job_detail{$vo}{$queue}{$job}{"r_proc"}>$interval);}
                                }
				elsif(($job_detail{$vo}{$queue}{$job}{"status"} =~ /R/) and ($act!=$job_detail{$vo}{$queue}{$job}{"r_proc"}) and !($bjobs{$job})){
                                        if($act-$job_detail{$vo}{$queue}{$job}{"r_proc"}>604800){
                                                if(my $test=grep /^$job\s+.+[RUN|PEND]/,@all_jobs){
                                                        $job_detail{$vo}{$queue}{$job}{"r_proc"}=$act;
                                                }
                                                else{delete($job_detail{$vo}{$queue}{$job});}
                                        }
                                }
			}
		}
	}

	foreach $file (keys(%proc)){
		if($debug){
			my $f=$proc{$file};
			`cp "$f" "$f.backup_p" 2>/dev/null`;
		}
		my $f=$proc{$file};
		unlink($proc{$file});
	}
	opendir(DIR,"$jm_dir/new/");
	@list_new=readdir(DIR);
	closedir(DIR);
	@list_new=grep !/back/,@list_new;
	@list_new=grep /\d+/,@list_new;

	foreach my $njob (@list_new){
		$njob=~/(\d+)/;
		my $jobid=$1;
		if($job_proc{$jobid}{"status"} and $njob=~/store/){
			my %ref=%{retrieve("$jm_dir/new/$njob")};
			my $vo_tmp=$job_proc{$jobid}{"vo"};
			my $queue_tmp=$job_proc{$jobid}{"queue"};
			$job_detail{$vo_tmp}{$queue_tmp}{$jobid}{"grid_id"}=$ref{$vo_tmp}{$queue_tmp}{$jobid}{"grid_id"} unless($job_detail{$vo_tmp}{$queue_tmp}{$jobid}{"grid_id"}=~/https/);
			unlink("$jm_dir/new/$njob");
		}
		elsif($job_proc{$jobid}{"status"} and $njob=~/\d+$/){
			open(NEW,"$jm_dir/new/$njob");
			my $job=<NEW>;
			close(NEW);
			my $vo_tmp=$job_proc{$jobid}{"vo"};
			my $queue_tmp=$job_proc{$jobid}{"queue"};
			if($job=~/submitted:\d+\|user:"\S+"\|queue:"\S+"\|grid_id:(\S+)\|/){
				my $tmp_gr_id=$1;
				$job_detail{$vo_tmp}{$queue_tmp}{$jobid}{"grid_id"}=$tmp_gr_id unless($job_detail{$vo_tmp}{$queue_tmp}{$jobid}{"grid_id"}=~/https/);
				$tmp_gr_id="";
				unlink("$jm_dir/new/$njob");
			}
		}
		unlink("$jm_dir/new/$njob") if($job_proc{$jobid}{"status"} eq "E" or ($act-(stat("$jm_dir/new/$njob"))[9])>604800);
	}
	opendir(DIR,"$jm_dir/ended/");
	@list_end=readdir(DIR);
	closedir(DIR);
	@list_end=grep !/back/,@list_end;
	@list_end=grep /store/,@list_end;
	foreach my $njob (@list_end){
		$njob=~/(\d+)/;
		my $jobid=$1;
		unless($job_proc{$jobid}{"status"} eq "E"){
			my %ref=%{retrieve("$jm_dir/ended/$njob")};
			if($njob=~/store/){
				foreach my $vo (keys(%ref)){
					foreach my $queue (keys(%{$ref{$vo}})){
						if($ref{$vo}{$queue}{$jobid}{"id"}){
							$job_detail{$vo}{$queue}{$jobid}{"status"}="E";
							$job_detail{$vo}{$queue}{$jobid}{"id"}=$ref{$vo}{$queue}{$jobid}{"id"};
							$job_detail{$vo}{$queue}{$jobid}{"grid_id"}=$ref{$vo}{$queue}{$jobid}{"grid_id"} unless($job_detail{$vo}{$queue}{$jobid}{"grid_id"}=~/https/);
							$job_detail{$vo}{$queue}{$jobid}{"user"}=$ref{$vo}{$queue}{$jobid}{"user"};
							$job_detail{$vo}{$queue}{$jobid}{"group"}=$ref{$vo}{$queue}{$jobid}{"group"};
							$job_detail{$vo}{$queue}{$jobid}{"qtime"}=$ref{$vo}{$queue}{$jobid}{"qtime"};
							$job_detail{$vo}{$queue}{$jobid}{"queue"}=$ref{$vo}{$queue}{$jobid}{"queue"};
							$job_detail{$vo}{$queue}{$jobid}{"name"}=$ref{$vo}{$queue}{$jobid}{"name"};
							$job_detail{$vo}{$queue}{$jobid}{"subject"}=$ref{$vo}{$queue}{$jobid}{"subject"};
							$job_detail{$vo}{$queue}{$jobid}{"start"}=$ref{$vo}{$queue}{$jobid}{"start"};
							$job_detail{$vo}{$queue}{$jobid}{"exec_host"}=$ref{$vo}{$queue}{$jobid}{"exec_host"};
							$job_detail{$vo}{$queue}{$jobid}{"end"}=$ref{$vo}{$queue}{$jobid}{"end"};
							$job_detail{$vo}{$queue}{$jobid}{"exit_status"}=$ref{$vo}{$queue}{$jobid}{"exit_status"};
							$job_detail{$vo}{$queue}{$jobid}{"used_cput"}=$ref{$vo}{$queue}{$jobid}{"used_cput"};
							$job_detail{$vo}{$queue}{$jobid}{"used_walltime"}=$ref{$vo}{$queue}{$jobid}{"used_walltime"};
							$job_detail{$vo}{$queue}{$jobid}{"used_mem"}=$ref{$vo}{$queue}{$jobid}{"used_mem"};
							$job_detail{$vo}{$queue}{$jobid}{"used_vmem"}=$ref{$vo}{$queue}{$jobid}{"used_vmem"};
							unlink("$jm_dir/ended/$njob");
						}
					}
				}
			}
		}
		else{unlink("$jm_dir/ended/$njob") if(($act-(stat("$jm_dir/ended/$njob"))[9])>604800);}
	}
	store(\%job_detail,"$jm_dir/processed/.jmgridice_job_detail");


}

##############################
##         OUTPUT           ##
##############################
                                                                                                                                                                                                         
my $output="";
my %subject=();
my $sbj_count=0;
foreach my $vo (sort keys(%job_detail)){
        my $flagp=0;
        my $temp_buffer="";
        $temp_buffer=$temp_buffer."*$vo";
        foreach my $queue (sort keys(%{$job_detail{$vo}})){
                my $buffer="";
                my $buffer1="|$queue";
                my $fin=0;
                my $ok=0;
                my $fail=0;
                my $run=0;
                my $que=0;
                my $flag=0;
                foreach my $job (sort keys(%{$job_detail{$vo}{$queue}})){
                        if(defined($job_detail{$vo}{$queue}{$job}{"id"})){
                                $flag=1;
				$flagp=1;
				unless($subject{$job_detail{$vo}{$queue}{$job}{"subject"}}){
					$sbj_count++;
					$subject{$job_detail{$vo}{$queue}{$job}{"subject"}}=$sbj_count;
				}
			}
                        if($job_detail{$vo}{$queue}{$job}{"status"} eq "E" and $job_detail{$vo}{$queue}{$job}{"end"}){
                                $fin++;
                                if($job_detail{$vo}{$queue}{$job}{"exit_status"} eq "0"){
                                        $ok++;
                                }
                                else{$fail++;}
                                $buffer=$buffer."^".$job_detail{$vo}{$queue}{$job}{"status"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"id"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"grid_id"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"name"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"user"};
                                $buffer=$buffer."\$".$subject{$job_detail{$vo}{$queue}{$job}{"subject"}};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"exec_host"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"used_cput"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"used_walltime"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"exit_status"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"used_mem"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"used_vmem"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"qtime"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"start"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"end"};
                        }
			elsif($job_detail{$vo}{$queue}{$job}{"status"} eq "R" or $job_detail{$vo}{$queue}{$job}{"status"} eq "E"){
                                $run++;
                                $buffer=$buffer."^R";
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"id"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"grid_id"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"name"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"user"};
                                $buffer=$buffer."\$".$subject{$job_detail{$vo}{$queue}{$job}{"subject"}};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"exec_host"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"used_cput"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"used_walltime"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"used_mem"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"used_vmem"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"qtime"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"start"};
                        }
                        elsif($job_detail{$vo}{$queue}{$job}{"status"} eq "Q" or $job_detail{$vo}{$queue}{$job}{"status"} eq "W" or $job_detail{$vo}{$queue}{$job}{"status"} eq "H"){
                                $que++;
                                $buffer=$buffer."^Q";
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"id"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"grid_id"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"name"};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"user"};
                                $buffer=$buffer."\$".$subject{$job_detail{$vo}{$queue}{$job}{"subject"}};
                                $buffer=$buffer."\$".$job_detail{$vo}{$queue}{$job}{"qtime"};
                        }
                }
                if($flag==1){
                        $temp_buffer=$temp_buffer.$buffer1."^$fin\$$ok\$$fail\$$run\$$que".$buffer;
                        $flag=0;
                }
        }
        if($flagp==1){
                $output=$output.$temp_buffer;
                $flagp=0;
        }
}
my $buffer="";
foreach my $sub_c (sort { $a <=> $b } values(%subject)){
        foreach my $sub_key (keys(%subject)){
                if($sub_c eq $subject{$sub_key} and ($sub_par=~/no/i)){
                        $buffer=$buffer."$sub_key^";
                }
                elsif($sub_c eq $subject{$sub_key}){
                        $buffer=$buffer."n/a^";
                }
        }
}
chop($buffer);
$output="$lrms*".$buffer.$output."*";
print $output;
