#!/usr/bin/perl

#Copyright (c) 2004 Istituto Nazionale di Fisica Nucleare
#For license conditions see http://www.cnaf.infn.it/license.html
#(c) by Guido Cuscela <guido.cuscela@ba.infn.it> and
#Giacinto Donvito <giacinto.donvito@ba.infn.it>, INFN-BARI

use Getopt::Long;

my ($lrms,$log_path,$date,$help);
GetOptions( "lrms:s" => \$lrms,
            "lrms-path:s" => \$log_path,
	    "jm-dir:s" => \$jm_dir,
            "date=s" => \$date,
            "help" => \$help
          );

if(defined($help)) {
           print "usage: perl $0 --lrms=pbs --lrms-path=<installation-path-of-batch-system> --date=yyyymmdd --jm-dir=<jm-dir> --help\n";
         exit 0;
}
elsif (!defined($lrms)) {
        die print "LRMS not defined!\n";
}
elsif (!defined($jm_dir)) {
        die print "jm dir $jm_dir not defined!\n";
}
elsif(!defined($log_path)){
	$log_path="/var/spool/pbs/server_priv/accounting";
}
else{
  unless($lrms =~ /\bpbs\b/i){
    die print "LRMS '$lrms' not available!\n";
  }
}

`echo -n $$ > $jm_dir/acct.pid 2>/dev/null`;
$cwd="/opt/gridice/monitoring/bin";

my $perl_comm=`ll /usr/bin/perl 2>/dev/null`;
if($perl_comm){$perl_comm="/usr/bin/perl";}
else{$perl_comm="perl";}
my $pbs_log="$log_path/$date";
my $globus_loc_buf=`grep GLOBUS_LOCATION= /etc/globus.conf 2>/dev/null`;
my ($globus_loc)=($globus_loc_buf=~ /GLOBUS_LOCATION=(.+)/);
my $command_buff_search=`grep QSTAT $globus_loc/libexec/globus-sh-tools-vars.sh 2> /dev/null | sed "s/GLOBUS_SH_QSTAT=//g" 2> /dev/null`;
my ($command_search)=($command_buff_search=~ /"(.+)\/qstat"/);
unless($command_search=~/\w+/){
	$command_search=`whereis qstat 2>/dev/null`;
	if($command_search=~/qstat:\s+(\S+)/){
		my $tmp_command=$1;
		my $ver=`file $1 2>/dev/null`;
		if($ver=~/executable/){
			$command=$tmp_command;
		}
		else{
			$command_search=`which qstat 2>/dev/null`;
			if($command_search=~/^(\S+qstat)/){
				$command=$1;
			}
			else{$command="qstat";}
		}
	}
	else{
		$command_search=`which qstat 2>/dev/null`;
		if($command_search=~/^(\S+qstat)/){
			$command=$1;
		}
		else{$command="qstat";}
	}
}
else{$command=$command_search."/qstat";}
my $flag_grep=0;
$acc_lines=`echo -n \`cat $pbs_log  2>/dev/null | wc -l  2>/dev/null\` 2>/dev/null`;

while(1){
	my $date_tmp = `date +%Y%m%d 2> /dev/null`;
        chop($date_tmp);
        my $time=`date +%k 2>/dev/null`*60+`date +%M 2>/dev/null`;
        exit if(($date_tmp ne $date) and ($time>1) and ($time<1435));
        unless(-e $pbs_log){
                sleep 5;
		$flag_grep=1;
        }
        else{
                if($flag_grep==0){
                        $flag_grep=1;
                        @loglines=`grep ";Q;" $pbs_log | tail -n 5 2> /dev/null`;
                        foreach my $logline (@loglines){
                                $logline =~ /;Q;(\d+)\./;
                                my $id=$1;
                                @job_info=`$command $id -f 2> /dev/null`;
                                my $count1=10;
                                until($job_info[0]=~/$id/ or $count1==0){
					my $count2=10;
					until($job_info[0]=~/$id/ or $count2==0){
                                        	@job_info=`$command $id -f 2> /dev/null`;
                                        	$count2--;
					}
					$count1--;
                                }
                                if($job_info[0]=~/$id/){
					unless(-e "$jm_dir/new/$id.qstat"){
	                                        open(OUT,">$jm_dir/new/$id.qstat");
        	                                print OUT @job_info;
                	                        close(OUT);
					}
                                }
                        }
                }
                else{
			my $tail_c=$acc_lines+1;
		        my @lines_tail=`tail -n +$tail_c $pbs_log 2>/dev/null`;
			if($#lines_tail eq '-1'){
                                sleep 2;
                        }
                        else{
				$acc_lines=$acc_lines+$#lines_tail+1;
                                while($line=shift(@lines_tail)){
                                        if($line =~ /;Q;(\d+)\./){
                                                my $id=$1;
                                                @job_info=`$command $id -f 2> /dev/null`;
						my $count1=10;
			                        until($job_info[0]=~/$id/ or $count1==0){
                       			                my $count2=10;
                                       			until($job_info[0]=~/$id/ or $count2>0){
		                                                @job_info=`$command $id -f 2> /dev/null`;
                       			                        $count2--;
                                       			}
							$count1--;
		                                        sleep 1;
                       			        }
		                                if($job_info[0]=~/$id/){
                       			                unless(-e "$jm_dir/new/$id.qstat"){
                                               			open(OUT,">$jm_dir/new/$id.qstat");
		                                                print OUT @job_info;
                       			                        close(OUT);
                                       			}
                               			}
                                        }
                                }
				sleep 3;
                        }
                }
		my $comm = `ps -eafww 2> /dev/null | grep messlog_mon.pl 2> /dev/null | grep -v grep 2> /dev/null`;
		unless($comm){
        	        system("$perl_comm $cwd/messlog_mon.pl --lrms=$lrms --lrms-path=$log_path --jm-dir=$jm_dir &");
	        }
        }
}
