#!/usr/bin/perl

#Copyright (c) 2004 Istituto Nazionale di Fisica Nucleare
#For license conditions see http://www.cnaf.infn.it/license.html
#(c) by Guido Cuscela <guido.cuscela@ba.infn.it> and
#Giacinto Donvito <giacinto.donvito@ba.infn.it>, INFN-BARI

#use Data::Dumper;
use Getopt::Long;
use threads;

my ($lrms,$log_path,$help);
GetOptions( "lrms:s" => \$lrms,
            "lrms-path:s" => \$log_path,
	    "jm-dir:s" => \$jm_dir,
	    "d" => \$debug,
            "help" => \$help
          );

if(!$lrms or !$jm_dir or !$log_path){
	open(JMC,"/opt/gridice/monitoring/etc/JM.conf");
	@jm_list=<JMC>;
	close(JMC);
	while($jm_line=shift(@jm_list)){
		if($jm_line=~/^\s*LRMS_TYPE\s*=\s*(\S+)/ and !$lrms){
			$lrms=$1;
		}
		if($jm_line=~/^\s*JM_TMP_DIR\s*=\s*(\S+)/ and !$jm_dir){
			$jm_dir=$1;
		}
		if($jm_line=~/^\s*LRMS_SPOOL_DIR\s*=\s*(\S+)/ and !$log_path){
			$log_path=$1;
		}
	}
}

if(defined($help)) {
           print "usage: perl $0 --lrms=pbs/lsf --lrms-path=<installation-path-of-batch-system> --jm-dir=<spool dir of job monitoring> --help\n";
         exit 0;
}
elsif (!defined($lrms)) {
        exit print "LRMS not defined!\n";
}
elsif (!defined($jm_dir)) {
        exit print "jm-dir $jm_dir not defined!\n";
}
elsif(!defined($log_path)){
	$log_path="/var/spool/pbs/server_priv/accounting" if($lrms =~ /pbs/i);
	exit if($lrms =~ /lsf/i)
}
else{
  unless($lrms =~ /pbs|lsf/i){
    exit print "LRMS '$lrms' not available! (possible pbs or lsf)\n";
  }
}

sub rem_bk{
	my $act=`date +%s 2>/dev/null`;
	chop($act);
	opendir(DIR,"$jm_dir/new/");
	my @list_new=readdir(DIR);
	@list_new=grep /\d+/,@list_new;
	close(DIR);
	foreach my $file (@list_new){
		my $ts=(stat("$jm_dir/new/$file"))[9];
		my $diff=$act-$ts;
		unlink("$jm_dir/new/$file") if($diff>7200);
	}
	opendir(DIR,"$jm_dir/ended/");
	my @list_end=readdir(DIR);
	@list_end=grep /\d+/,@list_end;
	close(DIR);
	foreach my $file (@list_end){
		my $ts=(stat("$jm_dir/ended/$file"))[9];
		my $diff=$act-$ts;
		unlink("$jm_dir/ended/$file") if($diff>7200);
	}
}

sub retrieve{
        my $file=$_[0];
        my @values=("id","grid_id","user","group","qtime","queue","exit_status","start","end","status","used_cput","used_walltime","used_mem","used_vmem","exec_host","subject","name","r_proc");
        open(RD,"<$file");
        my @temp_detail=<RD>;
        close(RD);
        my $j=0;
        my %detail=();
        foreach my $job_det (@temp_detail){
                my @fields=split(/\$/,$job_det);
                my $i=0;
                while($i<17){
                        $detail{$fields[3]}{$fields[5]}{$fields[0]}{$values[$i]}=$fields[$i];
                        $i++;
                }
                if($fields[$i]){
                        $detail{$fields[3]}{$fields[5]}{$fields[0]}{"r_proc"}=$fields[$i];
                        chop($detail{$fields[3]}{$fields[5]}{$fields[0]}{"r_proc"});
                }
                else{chop($detail{$fields[3]}{$fields[5]}{$fields[0]}{"name"});}
        }
        return \%detail;
}

sub store{
        my %hash=%{$_[0]};
        my $file=$_[1];
        open(JOB,">$file");
        foreach my $k (keys(%hash)){
                foreach my $k1 (keys(%{$hash{$k}})){
                        foreach my $k2 (sort keys(%{$hash{$k}{$k1}})){
                              my $out=$hash{$k}{$k1}{$k2}{"id"};
                              $out=$out."\$".$hash{$k}{$k1}{$k2}{"grid_id"};
                              $out=$out."\$".$hash{$k}{$k1}{$k2}{"user"};
                              $out=$out."\$".$hash{$k}{$k1}{$k2}{"group"};
                              $out=$out."\$".$hash{$k}{$k1}{$k2}{"qtime"};
                              $out=$out."\$".$hash{$k}{$k1}{$k2}{"queue"};
                              $out=$out."\$".$hash{$k}{$k1}{$k2}{"exit_status"};
                              $out=$out."\$".$hash{$k}{$k1}{$k2}{"start"};
                              $out=$out."\$".$hash{$k}{$k1}{$k2}{"end"};
                              $out=$out."\$".$hash{$k}{$k1}{$k2}{"status"};
                              $out=$out."\$".$hash{$k}{$k1}{$k2}{"used_cput"};
                              $out=$out."\$".$hash{$k}{$k1}{$k2}{"used_walltime"};
                              $out=$out."\$".$hash{$k}{$k1}{$k2}{"used_mem"};
                              $out=$out."\$".$hash{$k}{$k1}{$k2}{"used_vmem"};
                              $out=$out."\$".$hash{$k}{$k1}{$k2}{"exec_host"};
                              $out=$out."\$".$hash{$k}{$k1}{$k2}{"subject"};
                              $out=$out."\$".$hash{$k}{$k1}{$k2}{"name"};
                              $out=$out."\$".$hash{$k}{$k1}{$k2}{"r_proc"} if($hash{$k}{$k1}{$k2}{"r_proc"});
                              $out=$out."\n";
                              print JOB $out;
                       }
               }
        }
        close(JOB);
}
`echo -n $$ > $jm_dir/mess.pid 2>/dev/null`;
$cwd="/opt/gridice/monitoring/bin";

####################################
####           PBS              ####
####################################


if($lrms =~ /pbs/i){
	my $day=`date +%e 2>/dev/null`;
        chop($day);
	my $perl_comm=`ll /usr/bin/perl 2>/dev/null`;
	if($perl_comm){$perl_comm="/usr/bin/perl";}
	else{$perl_comm="perl";}
        my @datas=("Job_Name\\s=\\s(.+)\\n","job_state\\s=\\s(\\w)","queue\\s=\\s(.+)\\n","ctime\\s=\\s\\w+\\s(.+)\\n","euser\\s=\\s(\\w+)","egroup\\s=\\s(\\w+)");
	my @val_s=("name","status","queue","qtime","user","group");
        my @datal=("Job_Name\\s=\\s(.+)\\n","resources_used.cput\\s=\\s+(.+)\\n","resources_used.mem\\s=\\s+(.+)kb","resources_used.vmem\\s=\\s+(.+)kb","resources_used.walltime\\s=\\s+(.+)\\n","job_state\\s=\\s(\\w)","queue\\s=\\s(.+)\\n","ctime\\s=\\s\\w+\\s(.+)\\n","exec_host\\s=\\s([\\w-]+)\\..+/","mtime\\s=\\s\\w+\\s(.+)\\n","euser\\s=\\s(\\w+)","egroup\\s=\\s(\\w+)");
	my @val_l=("name","used_cput","used_mem","used_vmem","used_walltime","status","queue","qtime","exec_host","start","user","group");
        my $globus_loc_buf=`grep GLOBUS_LOCATION= /etc/globus.conf 2> /dev/null`;
        my ($globus_loc)=($globus_loc_buf=~ /GLOBUS_LOCATION=(.+)/);
        my $command_buff_search=`grep QSTAT $globus_loc/libexec/globus-sh-tools-vars.sh 2> /dev/null | sed "s/GLOBUS_SH_QSTAT=//g" 2> /dev/null`;
        my ($command_search)=($command_buff_search=~ /"(.+)\/qstat"/);
	unless($command_search=~/\w+/){
		$command_search=`whereis qstat 2>/dev/null`;
		if($command_search=~/qstat:\s+(\S+)/){
			my $tmp_command=$1;
			my $ver=`file $1 2>/dev/null`;
			if($ver=~/executable/){
				$command=$tmp_command;
			}
			else{
				$command_search=`which qstat 2>/dev/null`;
				if($command_search=~/^(\S+qstat)/){
					$command=$1;
				}
				else{$command="qstat";}
			}
		}
		else{
			$command_search=`which qstat 2>/dev/null`;
			if($command_search=~/^(\S+qstat)/){
				$command=$1;
			}
			else{$command="qstat";}
		}
	}
	$mess_lines=`echo -n \`cat /var/log/messages 2>/dev/null | wc -l 2>/dev/null\` 2>/dev/null`;
	@tmp_tail=();
        while(1){

	        my $date = `date +%Y%m%d 2> /dev/null`;
		chop($date);
		chop($log_path) if($log_path =~ /\/$/);
                my $pbs_log=$log_path."/".$date;
		my $yesterday=`date -d '1 day ago' +%Y%m%d 2>/dev/null`;
		chop($yesterday);
		my $pbs_log_y=$log_path."/".$yesterday;
		my $tail_c=$mess_lines+1;
	        my @lines_tail=`tail -n +$tail_c /var/log/messages 2>/dev/null`;
		push(@lines_tail,@tmp_tail);
		if($#lines_tail eq '-1'){
                        sleep 3;
			my $temp_mess=`echo -n \`cat /var/log/messages | wc -l\` 2>/dev/null`;
			if($mess_lines > $temp_mess){
				my $tmp_tail_c=$mess_lines+1;
                        	$mess_lines=0;
				@tmp_tail=`tail -n +$tmp_tail_c /var/log/messages.1 2>/dev/null`;
                	}
			else{@tmp_tail=();}
                }
                else{
			if($#tmp_tail ne '-1'){
	                        $mess_lines=$mess_lines+$#lines_tail-$#tmp_tail;
        	        }
			else{$mess_lines=$mess_lines+$#lines_tail+1;}
			@tmp_tail=();
			while($line=shift(@lines_tail)) {
                                if($line=~/Submitted job.+to batch system.+with ID\s+(\d+)/){
					my %job_detail=();
                                        my $id=$1;
					if(-e "$jm_dir/new/$id.qstat"){
						open(IN,"<$jm_dir/new/$id.qstat");
						@job_info=<IN>;
						close(IN);
						unless($job_info[0]=~/$id/){
							@job_info=`$command $id -f 2> /dev/null`;
	                                                $count=4;
        	                                        until($job_info[0]=~/$id/ or $count==0){
                	                                        sleep 1;
                        	                                @job_info=`$command $id -f 2> /dev/null`;
                                	                        $count--;
                                        	        }
                                                	unless($job_info[0]=~/$id/){
								next;
                                                	}
						}
					}
					else{
                                        	@job_info=`$command $id -f 2> /dev/null`;
	                                        $count=9;
        	                                until($job_info[0]=~/$id/ or $count==0){
                	                                sleep 1;
                        	                        @job_info=`$command $id -f 2> /dev/null`;
                                	                $count--;
                                        	}
	                                        unless($job_info[0]=~/$id/){
							next;
						}
					}
					my $grid_id1="";
                                        my $grid_id="";
					my $stage_in="";
                                        if($job_info[3] =~ /job_state = [QREHTWS]/ ){
                                                my $i=0;
                                                my $y=0;
                                                if(my @grid=grep /EDG_WL_JOBID|GLITE_WMS_JOBID/,@job_info){
                                                        while($datas[$y] and $job_info[$i]){
                                                                if($job_info[$i]=~$datas[$y]){
                                                                        $hashs[$y]=$1;
                                                                        $i++;
                                                                        $y++;
                                                                }
								elsif($job_info[$i] =~ /(EDG_WL_JOBID|GLITE_WMS_JOBID)=(.+),/){
        	 			                                $grid_id=$2;
									$i++;
       	 							}
							        elsif($job_info[$i] =~ /(EDG_WL_JOBID|GLITE_WMS_JOBID)=(.+)/){
                							$grid_id1=$2;
									$i++;
							        }
							        elsif($job_info[$i] =~ /\s+(.*),/ and $grid_id1){
						                	$grid_id=$grid_id1.$1;
						                	$grid_id1="";
							        }
								else{$i++;}
                                                        }
							for($i=0;$i<=$#hashs;$i++){
								$job_detail{$hashs[5]}{$hashs[2]}{$id}{$val_s[$i]}=$hashs[$i];
							}
							$job_detail{$hashs[5]}{$hashs[2]}{$id}{"grid_id"}=$grid_id;
							$grid_id="";
							$grid_id1="";
                                                }
                                                else{
                                                        while($datas[$y] and $job_info[$i]){
                                                                if($job_info[$i]=~$datas[$y]){
                                                                        $hashs[$y]=$1;
                                                                        $i++;
                                                                        $y++;
                                                                }
								elsif($job_info[$i]=~/stagein =.+globus/){
									$stage_in=$job_info[$i];
									until($job_info[$i]=~/gpg$/){
										chop($stage_in);
										$i++;
										$stage_in.=$job_info[$i];
									}
									$stage_in=~s/\s+//g;
								}
                                                                else{$i++;}
                                                        }
							if($stage_in=~/:(.+globus-cache-export.+\/)globus-cache-export/){
								$tmp_dir=$1;
								$grid_id=`grep -h EDG_WL_JOBID $tmp_dir* 2>/dev/null | grep https 2>/dev/null | sed 's/export EDG_WL_JOBID="\\s*//' 2>/dev/null`;
								$grid_id=`grep -h GLITE_WMS_JOBID $tmp_dir* 2>/dev/null | grep https 2>/dev/null | sed 's/export GLITE_WMS_JOBID="\\s*//' 2>/dev/null` unless($grid_id=~/https/);
								chop($grid_id);
								chop($grid_id);
							}
							for($i=0;$i<=$#hashs;$i++){
								$job_detail{$hashs[5]}{$hashs[2]}{$id}{$val_s[$i]}=$hashs[$i];
							}
							if($grid_id=~/https/){
								$job_detail{$hashs[5]}{$hashs[2]}{$id}{"grid_id"}=$grid_id;
							}
							else{
								chop($tmp_dir);
								$grid_id=`grep -h JobWrapper $tmp_dir/export.txt 2>/dev/null`;
								if($grid_id =~ /JobWrapper.(\S+)\.sh/){
								        $grid_id = $1;
								        $grid_id =~ s/https_3a_2f_2f(\S+)_3a(\d+)_2f(\S+)/https:\/\/$1:$2\/$3/;
									$job_detail{$hashs[5]}{$hashs[2]}{$id}{"grid_id"}=$grid_id;
								}
								else{
		                                                        $job_detail{$hashs[5]}{$hashs[2]}{$id}{"grid_id"}="n/a";
								}
								$tmp_dir="";
							}
							$grid_id="";
                                                }
                                                my $date=$job_detail{$hashs[5]}{$hashs[2]}{$id}{"qtime"};
                                                $job_detail{$hashs[5]}{$hashs[2]}{$id}{"qtime"}=`date -d "$date" +%s 2> /dev/null`;
                                                chop($job_detail{$hashs[5]}{$hashs[2]}{$id}{"qtime"});
                                                $job_detail{$hashs[5]}{$hashs[2]}{$id}{"used_cput"}="00";
                                                $job_detail{$hashs[5]}{$hashs[2]}{$id}{"used_mem"}="00";
                                                $job_detail{$hashs[5]}{$hashs[2]}{$id}{"used_vmem"}="00";
                                                $job_detail{$hashs[5]}{$hashs[2]}{$id}{"used_walltime"}="00";
						if($job_detail{$hashs[5]}{$hashs[2]}{$id}{"status"} eq "R"){
							$job_detail{$hashs[5]}{$hashs[2]}{$id}{"start"}=$job_detail{$hashs[5]}{$hashs[2]}{$id}{"qtime"};
						}
						else{$job_detail{$hashs[5]}{$hashs[2]}{$id}{"start"}=undef;}
						$job_detail{$hashs[5]}{$hashs[2]}{$id}{"end"}=undef;
						$job_detail{$hashs[5]}{$hashs[2]}{$id}{"exec_host"}="NO HOST";
						$job_detail{$hashs[5]}{$hashs[2]}{$id}{"exit_status"}=undef;
						$job_detail{$hashs[5]}{$hashs[2]}{$id}{"id"}=$id;
                                                my $acc=$job_detail{$hashs[5]}{$hashs[2]}{$id}{"user"};
						if(-e "$jm_dir/subject/$acc"){
							$job_detail{$hashs[5]}{$hashs[2]}{$id}{"subject"}=`cat $jm_dir/subject/$acc 2> /dev/null | sed s/^[0-9]*' '// 2>/dev/null`;
                        			}
                        			else{
							my $sbj=`grep $acc /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
                                			$sbj=~/"(.+)"\s+mapped/;
                                			$job_detail{$hashs[5]}{$hashs[2]}{$id}{"subject"}=$1;
                        			}
                                                store(\%job_detail,"$jm_dir/new/$id");
                                                %job_detail=();
                                        }
                                        elsif($job_info[7] =~ /job_state = [QREHTWS]/){
                                                my $i=0;
                                                my $y=0;
                                                if(my @grid=grep /EDG_WL_JOBID|GLITE_WMS_JOBID/,@job_info){
                                                        while($datal[$y] and $job_info[$i]){
                                                                if($job_info[$i]=~$datal[$y]){
                                                                        $hashl[$y]=$1;
                                                                        $i++;
                                                                        $y++;
                                                                }
								elsif($job_info[$i] =~ /(EDG_WL_JOBID|GLITE_WMS_JOBID)=(.+),/){
                                                                        $grid_id=$2;
                                                                        $i++;
                                                                }
                                                                elsif($job_info[$i] =~ /(EDG_WL_JOBID|GLITE_WMS_JOBID)=(.+)/){
                                                                        $grid_id1=$2;
                                                                        $i++;
                                                                }
                                                                elsif($job_info[$i] =~ /\s+(.*),/ and $grid_id1){
                                                                        $grid_id=$grid_id1.$1;
                                                                        $grid_id1="";
                                                                }
                                                                else{$i++;}
                                                        }
							for($i=0;$i<=$#hashl;$i++){
								$job_detail{$hashl[11]}{$hashl[6]}{$id}{$val_l[$i]}=$hashl[$i];
							}
							$job_detail{$hashl[11]}{$hashl[6]}{$id}{"grid_id"}=$grid_id;
							$grid_id="";
                                                }
                                                else{
                                                        while($datal[$y] and $job_info[$i]){
                                                                if($job_info[$i]=~$datal[$y]){
                                                                        $hashl[$y]=$1;
                                                                        $i++;
                                                                        $y++;
                                                                }
								elsif($job_info[$i]=~/stagein =.+globus/){
                                                                        $stage_in=$job_info[$i];
                                                                        until($job_info[$i]=~/gpg$/){
                                                                                chop($stage_in);
                                                                                $i++;
                                                                                $stage_in.=$job_info[$i];
                                                                        }
                                                                        $stage_in=~s/\s+//g;
                                                                }
                                                                else{$i++;}
                                                        }
							if($stage_in=~/:(.+globus-cache-export.+\/)globus-cache-export/){
								$tmp_dir=$1;
                                                                $grid_id=`grep -h EDG_WL_JOBID $tmp_dir* 2>/dev/null | grep https 2>/dev/null | sed 's/export EDG_WL_JOBID="\\s*//' 2>/dev/null`;
								$grid_id=`grep -h GLITE_WMS_JOBID $tmp_dir* 2>/dev/null | grep https 2>/dev/null | sed 's/export GLITE_WMS_JOBID="\\s*//' 2>/dev/null` unless($grid_id=~/https/);
                                                                chop($grid_id);
                                                                chop($grid_id);
                                                        }
							for($i=0;$i<=$#hashl;$i++){
								$job_detail{$hashl[11]}{$hashl[6]}{$id}{$val_l[$i]}=$hashl[$i];
							}
							if($grid_id=~/https/){
                                                                $job_detail{$hashs[11]}{$hashs[6]}{$id}{"grid_id"}=$grid_id;
                                                        }
                                                        else{
								chop($tmp_dir);
                                                                $grid_id=`grep -h JobWrapper $tmp_dir/export.txt 2>/dev/null`;
                                                                if($grid_id =~ /JobWrapper.(\S+)\.sh/){
                                                                        $grid_id = $1;
                                                                        $grid_id =~ s/https_3a_2f_2f(\S+)_3a
(\d+)_2f(\S+)/https:\/\/$1:$2\/$3/;
                                                                }
                                                                else{
                                                                        $job_detail{$hashs[5]}{$hashs[2]}{$id}{"grid_id"}="n/a";
                                                                }
								$tmp_dir="";
                                                        }
							$grid_id="";
                                                }
                                                my $date=$job_detail{$hashl[11]}{$hashl[6]}{$id}{"qtime"};
                                                $job_detail{$hashl[11]}{$hashl[6]}{$id}{"qtime"}=`date -d "$date" +\%s 2> /dev/null`;
						chop($job_detail{$hashl[11]}{$hashl[6]}{$id}{"qtime"});
                                                $job_detail{$hashl[11]}{$hashl[6]}{$id}{"used_cput"} =~ /(\d+):(\d+):(\d+)/;
                                                $job_detail{$hashl[11]}{$hashl[6]}{$id}{"used_cput"} = ($3+$2*60+$1*3600);
                                                $job_detail{$hashl[11]}{$hashl[6]}{$id}{"used_walltime"} =~ /(\d+):(\d+):(\d+)/;
                                                $job_detail{$hashl[11]}{$hashl[6]}{$id}{"used_walltime"} = ($3+$2*60+$1*3600);
                                                $job_detail{$hashl[11]}{$hashl[6]}{$id}{"end"}=undef;
                                                $job_detail{$hashl[11]}{$hashl[6]}{$id}{"exit_status"}=undef;
						$job_detail{$hashl[11]}{$hashl[6]}{$id}{"id"}=$id;
                                                my $acc=$job_detail{$hashl[11]}{$hashl[6]}{$id}{"user"};
						if(-e "$jm_dir/subject/$acc"){
							$job_detail{$hashl[11]}{$hashl[6]}{$id}{"subject"}=`cat $jm_dir/subject/$acc 2> /dev/null | sed s/^[0-9]*' '// 2>/dev/null`;
                                                }
                                                else{
							my $sbj=`grep $acc /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
                                                        $sbj=~/"(.+)"\s+mapped/;
                                                        $job_detail{$hashl[11]}{$hashl[6]}{$id}{"subject"}=$1;
                                                }
                                                store(\%job_detail,"$jm_dir/new/$id");
                                                %job_detail=();
                                        }
                                }
                                elsif($line =~ /\(ID\s+(\d+)\.(.+)\)\s+has finished/){
					my %job_detail=();
                                        my $id=$1;
					my $dom=$2;
                                        $ln=`grep \\;E\\; $pbs_log 2> /dev/null | grep $id.$dom 2> /dev/null`;
					unless($ln){$ln=`grep \\;E\\; $pbs_log_y 2> /dev/null | grep $id.$dom 2> /dev/null`;}
                                        if($ln=~/;E;(\d+).+;user=(.+)\s+group=(.+)\s+jobname=(.+)\s+queue=(.+)\s+ctime=(\d+).+start=(\d+)\s+exec_host=([\w-]+)\..+end=(\d+)\s+Exit_status=(.+)\sresources_used.cput=(\S*\d+):(\S*\d+):(\S*\d+)\s+resources_used.mem=(.+)kb\s+resources_used.vmem=(.+)kb\s+resources_used.walltime=(\S*\d+):(\S*\d+):(\S*\d+)/){
						$job_detail{$3}{$5}{$id}{"status"}="E";
        	                                $job_detail{$3}{$5}{$id}{"id"}=$1;
                	                        $job_detail{$3}{$5}{$id}{"user"}=$2;
						my $vo=$3;
                                                my $q=$5;
						if(-e "$jm_dir/new/$id"){
							my $ref=retrieve("$jm_dir/new/$id");
							$job_detail{$vo}{$q}{$id}{"grid_id"}=$ref->{$vo}{$q}{$id}{"grid_id"};
							unlink("$jm_dir/new/$id");
						}
        	                                else{$job_detail{$3}{$5}{$id}{"grid_id"}="n/a";}
                	                        $job_detail{$3}{$5}{$id}{"group"}=$3;
                        	                $job_detail{$3}{$5}{$id}{"name"}=$4;
                                	        $job_detail{$3}{$5}{$id}{"queue"}=$5;
                                        	$job_detail{$3}{$5}{$id}{"qtime"}=$6;
	                                        $job_detail{$3}{$5}{$id}{"start"}=$7;
        	                                $job_detail{$3}{$5}{$id}{"end"}=$9;
                	                        $job_detail{$3}{$5}{$id}{"exec_host"}=$8;
                        	                $job_detail{$3}{$5}{$id}{"exit_status"}=$10;
                                	        $cput_h=$11;
                                        	$cput_m=$12;
	                                        $cput_s=$13;
        	                                $job_detail{$3}{$5}{$id}{"used_cput"}=$cput_s+$cput_m*60+$cput_h*3600;
                	                        $job_detail{$3}{$5}{$id}{"used_mem"}=$14;
                        	                $job_detail{$3}{$5}{$id}{"used_vmem"}=$15;
                                	        $wallt_h=$16;
                                        	$wallt_m=$17;
	                                        $wallt_s=$18;
        	                                $job_detail{$3}{$5}{$id}{"used_walltime"}=$wallt_s+$wallt_m*60+$wallt_h*3600;
						if($job_detail{$3}{$5}{$id}{"used_walltime"}<0){$job_detail{$3}{$5}{$id}{"used_walltime"}=$job_detail{$3}{$5}{$id}{"end"}-$job_detail{$3}{$5}{$id}{"start"};}
						my $acc=$job_detail{$3}{$5}{$id}{"user"};
                                        	if(-e "$jm_dir/subject/$acc"){
							$job_detail{$vo}{$q}{$id}{"subject"}=`cat $jm_dir/subject/$acc 2> /dev/null | sed s/^[0-9]*' '// 2>/dev/null`;
	                                        }
        	                                else{
							my $sbj=`grep $acc /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
                        	        		$sbj=~/"(.+)"\s+mapped/;
                                			$job_detail{$vo}{$q}{$id}{"subject"}=$1;
                                        	}
                                        	store(\%job_detail,"$jm_dir/ended/$id");
					}
					elsif($ln=~/;E;(\d+).+;user=(.+)\s+group=(.+)\s+jobname=(.+)\s+queue=(.+)\s+ctime=(\d+).+start=(\d+)\s+exec_host=([\w-]+)\..+end=(\d+)\s+Exit_status=(\d+)/){
                                                $job_detail{$3}{$5}{$id}{"status"}="E";
                                                $job_detail{$3}{$5}{$id}{"id"}=$1;
                                                $job_detail{$3}{$5}{$id}{"user"}=$2;
						my $vo=$3;
						my $q=$5;
                                                if(-e "$jm_dir/new/$id"){
                                                        my $ref=retrieve("$jm_dir/new/$id");
                                                        $job_detail{$vo}{$q}{$id}{"grid_id"}=$ref->{$vo}{$q}{$id}{"grid_id"};
                                                        unlink("$jm_dir/new/$id");
                                                }
                                                else{$job_detail{$3}{$5}{$id}{"grid_id"}="n/a";}
                                                $job_detail{$3}{$5}{$id}{"group"}=$3;
                                                $job_detail{$3}{$5}{$id}{"name"}=$4;
                                                $job_detail{$3}{$5}{$id}{"queue"}=$5;
                                                $job_detail{$3}{$5}{$id}{"qtime"}=$6;
                                                $job_detail{$3}{$5}{$id}{"start"}=$7;
                                                $job_detail{$3}{$5}{$id}{"end"}=$9;
                                                $job_detail{$3}{$5}{$id}{"exec_host"}=$8;
                                                $job_detail{$3}{$5}{$id}{"exit_status"}=$10;
                                                $job_detail{$3}{$5}{$id}{"used_cput"}="0";
                                                $job_detail{$3}{$5}{$id}{"used_mem"}="0";
                                                $job_detail{$3}{$5}{$id}{"used_vmem"}="0";
                                                $job_detail{$3}{$5}{$id}{"used_walltime"}="0";
                                                my $acc=$job_detail{$3}{$5}{$id}{"user"};
                                                if(-e "$jm_dir/subject/$acc"){
                                                        $job_detail{$vo}{$q}{$id}{"subject"}=`cat $jm_dir/subject/$acc 2> /dev/null | sed s/^[0-9]*' '// 2>/dev/null`;
                                                }
                                                else{
                                                        my $sbj=`grep $acc /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
                                                        $sbj=~/"(.+)"\s+mapped/;
                                                        $job_detail{$vo}{$q}{$id}{"subject"}=$1;
                                                }
                                        	store(\%job_detail,"$jm_dir/ended/$id");
                                        }
                                        %job_detail=();
				}
				elsif($line =~ /batch ID\s+(\d+)\.(.+)\)\s+REMOVED from batch system ok/){
					my %job_detail=();
                                        my $id=$1;
					my $dom=$2;
					my $test_job=`ls /var/spool/pbs/server_priv/jobs/$id.*.SC 2>/dev/null`;
					unless($test_job){
		                        	my @lines=`grep $id.$dom $pbs_log 2> /dev/null | grep ";D;" 2> /dev/null`;
						unless(defined($lines[0])){
							@lines=`grep $id.$dom $pbs_log_y 2> /dev/null | grep ";D;" 2> /dev/null`;
						}
						my @lines1=`grep $id.$dom $pbs_log 2> /dev/null | grep ";E;" 2> /dev/null`;
						unless(defined($lines1[0])){
							@lines1=`grep $id.$dom $pbs_log_y 2> /dev/null | grep ";E;" 2> /dev/null`;
						}
						push(@lines,@lines1);
                		                my $flag=0;
                        		        foreach $ln (@lines){
                                			if($ln=~/(.+);D;$id.+;requestor=(\w+)@/ and $flag==0){
								$data=`date -d "$1" +\%s 2> /dev/null`;
								chop($data);
	                                                       	$remover=$2;
		                                                $flag=1;
        		                                }
                		                        elsif($ln=~/;E;(\d+).+;user=(.+)\s+group=(.+)\s+jobname=(.+)\s+queue=(.+)\s+ctime=(\d+).+start=(\d+)\s+exec_host=([\w-]+)\..+end=(\d+)\s+Exit_status=(.+)\s+resources_used.cput=(\S*\d+):(\S*\d+):(\S*\d+)\s+resources_used.mem=(.+)kb\s+resources_used.vmem=(.+)kb\s+resources_used.walltime=(\S*\d+):(\S*\d+):(\S*\d+)/ and $flag==1){
								$job_detail{$3}{$5}{$id}{"status"}="E";
                                		                $job_detail{$3}{$5}{$id}{"id"}=$1;
                                        		        $job_detail{$3}{$5}{$id}{"user"}=$2;
                                                        	$job_detail{$3}{$5}{$id}{"grid_id"}="n/a";
	                                                       	$job_detail{$3}{$5}{$id}{"group"}=$3;
		                                                $job_detail{$3}{$5}{$id}{"name"}=$4;
        		                                        $job_detail{$3}{$5}{$id}{"queue"}=$5;
                		                                $job_detail{$3}{$5}{$id}{"qtime"}=$6;
                        		                        $job_detail{$3}{$5}{$id}{"start"}=$7;
                                		                $job_detail{$3}{$5}{$id}{"end"}=$9;
                                        		        $job_detail{$3}{$5}{$id}{"exec_host"}=$8;
                                                        	$job_detail{$3}{$5}{$id}{"exit_status"}=$10;
	                                                       	$cput_h=$11;
		                                                $cput_m=$12;
        		                                        $cput_s=$13;
                		                                $job_detail{$3}{$5}{$id}{"used_cput"}=$cput_s+$cput_m*60+$cput_h*3600;
                        		                        $job_detail{$3}{$5}{$id}{"used_mem"}=$14;
                                		                $job_detail{$3}{$5}{$id}{"used_vmem"}=$15;
                                        		        $wallt_h=$16;
                                                        	$wallt_m=$17;
	                                                       	$wallt_s=$18;
		                                                $job_detail{$3}{$5}{$id}{"used_walltime"}=$wallt_s+$wallt_m*60+$wallt_h*3600;
								if($job_detail{$3}{$5}{$id}{"used_walltime"}<0){$job_detail{$3}{$5}{$id}{"used_walltime"}=$job_detail{$3}{$5}{$id}{"end"}-$job_detail{$3}{$5}{$id}{"start"};}
								my $vo=$3;
                		                        	my $q=$5;
								my $acc=$2;
								if(-e "$jm_dir/new/$id"){
									my $ref=retrieve("$jm_dir/new/$id");
		                        	        	        $job_detail{$vo}{$q}{$id}{"grid_id"}=$ref->{$vo}{$q}{$id}{"grid_id"};
                		                                	unlink("$jm_dir/new/$id");
								}
								if(-e "$jm_dir/subject/$acc"){
									$job_detail{$vo}{$q}{$id}{"subject"}=`cat $jm_dir/subject/$acc 2> /dev/null | sed s/^[0-9]*' '// 2>/dev/null`;
                		                		}
                        		                	else{
									my $sbj=`grep $acc /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
                                        		               	$sbj=~/"(.+)"\s+mapped/;
                                                		        	$job_detail{$vo}{$q}{$id}{"subject"}=$1;
	                                        		}
		                                                store(\%job_detail,"$jm_dir/ended/$id");
	        	                                        %job_detail=();
								$flag=0;
							}
							elsif($ln=~/;E;(\d+).+;user=(.+)\s+group=(.+)\s+jobname=(.+)\s+queue=(.+)\s+ctime=(\d+).+start=(\d+)\s+exec_host=([\w-]+)\..+end=(\d+)\s+Exit_status=(\d+)/ and $flag==1){
        	                	                	$job_detail{$3}{$5}{$id}{"status"}="E";
                	                	                $job_detail{$3}{$5}{$id}{"id"}=$1;
                        	                	        $job_detail{$3}{$5}{$id}{"user"}=$2;
                               	                        	$job_detail{$3}{$5}{$id}{"grid_id"}="n/a";
	                                       	                $job_detail{$3}{$5}{$id}{"group"}=$3;
        	                                       	        $job_detail{$3}{$5}{$id}{"name"}=$4;
                	                                       	$job_detail{$3}{$5}{$id}{"queue"}=$5;
                        	                                $job_detail{$3}{$5}{$id}{"qtime"}=$6;
       	                        	                        $job_detail{$3}{$5}{$id}{"start"}=$7;
               	                        	                $job_detail{$3}{$5}{$id}{"end"}=$9;
                       	                        	        $job_detail{$3}{$5}{$id}{"exec_host"}=$8;
                               	                        	$job_detail{$3}{$5}{$id}{"exit_status"}=$10;
	                                       	                $job_detail{$3}{$5}{$id}{"used_cput"}="0";
        	                                       	        $job_detail{$3}{$5}{$id}{"used_mem"}="0";
                	                                       	$job_detail{$3}{$5}{$id}{"used_vmem"}="0";
                        	                                $job_detail{$3}{$5}{$id}{"used_walltime"}="0";
       	                        	                        my $vo=$3;
               	                        	                my $q=$5;
								my $acc=$2;
								if(-e "$jm_dir/new/$id"){
		                       	                                my $ref=retrieve("$jm_dir/new/$id");
	        	                               	                $job_detail{$vo}{$q}{$id}{"grid_id"}=$ref->{$vo}{$q}{$id}{"grid_id"};
                	        	                               	unlink("$jm_dir/new/$id");
								}
                                	                        if(-e "$jm_dir/subject/$acc"){
       	                                	                        $job_detail{$vo}{$q}{$id}{"subject"}=`cat $jm_dir/subject/$acc 2> /dev/null | sed s/^[0-9]*' '// 2>/dev/null`;
               	                                	        }
                       	                                	else{
	                               	                                my $sbj=`grep $acc /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
        	                               	                        $sbj=~/"(.+)"\s+mapped/;
                	                               	                $job_detail{$vo}{$q}{$id}{"subject"}=$1;
                        	                               	}
                                	                        store(\%job_detail,"$jm_dir/ended/$id");
       	                                	                %job_detail=();
                       	                        	        $flag=0;
	                               	                }
						}
						if($flag==1 and !(-e "$jm_dir/new/$id")){
							if(-e "$jm_dir/new/$id.qstat"){
								my $queue=`grep queue $jm_dir/new/$id.qstat 2>/dev/null`;
								$queue=~/queue\s*=\s*(\S+)/;
								$queue=$1;
								my $user=`grep euser $jm_dir/new/$id.qstat 2>/dev/null`;
	                                                        $user=~/euser\s*=\s*(\S+)/;
        	                                                $user=$1;
								my $vo=`grep egroup $jm_dir/new/$id.qstat 2>/dev/null`;
                        	                                $vo=~/egroup\s*=\s*(\S+)/;
                                	                        $vo=$1;
								my $name=`grep Job_Name $jm_dir/new/$id.qstat 2>/dev/null`;
								$name=~/Job_Name = (\S+)/;
								$job_detail{$vo}{$queue}{$id}{"name"}=$1;
								my $date=`grep ctime $jm_dir/new/$id.qstat 2>/dev/null`;
								$date=~/ctime = (.+)/;
	        	                                        $job_detail{$vo}{$queue}{$id}{"qtime"}=`date -d "$1" +\%s 2> /dev/null`;
								chop($job_detail{$vo}{$queue}{$id}{"qtime"});
								$job_detail{$vo}{$queue}{$id}{"id"}=$id;
								$job_detail{$vo}{$queue}{$id}{"group"}=$vo;
								$job_detail{$vo}{$queue}{$id}{"queue"}=$queue;
								$job_detail{$vo}{$queue}{$id}{"user"}=$user;
								$job_detail{$vo}{$queue}{$id}{"status"}="E";
								$job_detail{$vo}{$queue}{$id}{"used_cput"} = "00";
								$job_detail{$vo}{$queue}{$id}{"used_walltime"} = "00";
								$job_detail{$vo}{$queue}{$id}{"used_mem"}="00";
								$job_detail{$vo}{$queue}{$id}{"used_vmem"}="00";
								if($remover =~ /root/){
                                                	                $job_detail{$vo}{$queue}{$id}{"exit_status"}="-201";
                                                        	}
	                                                        else{$job_detail{$vo}{$queue}{$id}{"exit_status"}="-200";}
								$job_detail{$vo}{$queue}{$id}{"exec_host"} = "NO HOST";
								$job_detail{$vo}{$queue}{$id}{"start"} = "00";
								$job_detail{$vo}{$queue}{$id}{"end"} = $data;
								if(-e "$jm_dir/subject/$user"){
                                        	                        $job_detail{$vo}{$queue}{$id}{"subject"}=`cat $jm_dir/subject/$user 2> /dev/null | sed s/^[0-9]*' '// 2>/dev/null`;
                                                	        }
	                                                        else{
                                                        	        my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
        	                                                        $sbj=~/"(.+)"\s+mapped/;
                	                                                $job_detail{$vo}{$queue}{$id}{"subject"}=$1;
                        	                                }
								my $grid_id="n/a";
								my $grid_id1="";
								my @wl_id = `grep -P 'EDG_WL_JOBID|GLITE_WMS_JOBID' $jm_dir/new/$id.qstat -A1 2>/dev/null`;
								my $i=0;
								while($wl_id[$i]){
                                	                                if($wl_id[$i] =~ /(EDG_WL_JOBID|GLITE_WMS_JOBID)=(.+),/){
                                        	                                $grid_id=$2;
                                                	                        $i+=2;
                                                        	        }
                                                                	elsif($wl_id[$i] =~ /(EDG_WL_JOBID|GLITE_WMS_JOBID)=(.+)/){
	                                                                        $grid_id1=$2;
        	                                                                $i++;
                	                                                }
                        	                                        elsif($wl_id[$i] =~ /\s+(.*),/ and $grid_id1){
                                	                                        $grid_id=$grid_id1.$1;
                                        	                                $grid_id1="";
                                                	                }
                                                        	        else{$i++;}
	                                                        }
								$job_detail{$vo}{$queue}{$id}{"grid_id"}=$grid_id;
								$grid_id="";
								$grid_id1="";
							}
							else{
								$job_detail{"garbage"}{"garbage"}{$id}{"status"}="E";
								$job_detail{"garbage"}{"garbage"}{$id}{"start"} = "00";
								$job_detail{"garbage"}{"garbage"}{$id}{"exec_host"} = "NO HOST";
								$job_detail{"garbage"}{"garbage"}{$id}{"end"} = $data;
								if($remover =~ /root/){
									$job_detail{"garbage"}{"garbage"}{$id}{"exit_status"}="-201";
								}
								else{$job_detail{"garbage"}{"garbage"}{$id}{"exit_status"}="-200";}
								$job_detail{"garbage"}{"garbage"}{$id}{"used_cput"} = "00";
								$job_detail{"garbage"}{"garbage"}{$id}{"used_mem"}="00";
								$job_detail{"garbage"}{"garbage"}{$id}{"used_vmem"}="00";
								$job_detail{"garbage"}{"garbage"}{$id}{"used_walltime"} = "00";
								$job_detail{"garbage"}{"garbage"}{$id}{"id"}=$id;
								$job_detail{"garbage"}{"garbage"}{$id}{"name"}="-2";
								$job_detail{"garbage"}{"garbage"}{$id}{"queue"}="-2";
								$job_detail{"garbage"}{"garbage"}{$id}{"qtime"}="-2";
								$job_detail{"garbage"}{"garbage"}{$id}{"grid_id"}="n/a";
								$job_detail{"garbage"}{"garbage"}{$id}{"user"}="-2";
								$job_detail{"garbage"}{"garbage"}{$id}{"group"}="-2";
								$job_detail{"garbage"}{"garbage"}{$id}{"subject"}="n/a";
							}
                                        		store(\%job_detail,"$jm_dir/ended/$id");
						}
						elsif($flag==1){
							my %ref=%{retrieve("$jm_dir/new/$id")};
							foreach my $vo (keys(%ref)){
								foreach my $queue (keys(%{$ref{$vo}})){
									if($ref{$vo}{$queue}{$id}{"id"}){
										$job_detail{$vo}{$queue}{$id}=$ref{$vo}{$queue}{$id};
										if($remover=~/root/){
											$job_detail{$vo}{$queue}{$id}{"exit_status"}="-201";
										}
										else{$job_detail{$vo}{$queue}{$id}{"exit_status"}="-200";}
										$job_detail{$vo}{$queue}{$id}{"start"} = "00";
										$job_detail{$vo}{$queue}{$id}{"end"} = $data;
									}
								}
							}
							unlink("$jm_dir/new/$id");
	                                        	store(\%job_detail,"$jm_dir/ended/$id");
						}
						$flag=0;
	                                        %job_detail=();
					}
                        	}
				elsif($line =~ /GRAM.+: "(.+)" mapped to (\w+) /){
                                	my $user_info=$1;
	                                my $loc_acc=$2;
        	                        if(($job_mon{$loc_acc} ne $user_info) or !defined($job_mon{$loc_acc}) or !(-e "$jm_dir/subject/$loc_acc")){
                	                        open(OUT,">$jm_dir/subject/$loc_acc");
                        	                print OUT "$user_info";
                                	        close(OUT);
                                        	$job_mon{$loc_acc}=$user_info;
					}
                                }
			}
			sleep 3;
		}
		my $date_new = `date +%Y%m%d 2> /dev/null`;
	        chop($date_new);
        	my $flag = `ps -eafww 2> /dev/null | grep acclog_mon.pl 2> /dev/null | grep $date_new 2> /dev/null | grep -v grep 2> /dev/null`;
	        unless($flag){
        	        system("$perl_comm $cwd/acclog_mon.pl --lrms=$lrms --lrms-path=$log_path --date=$date_new --jm-dir=$jm_dir &");
	        }
        	else{
	                my $time=`date +%k 2>/dev/null`*60+`date +%M 2>/dev/null`;
        	        if($time > 1435){
                	        my $date_new=`date -d '1 day' +%Y%m%d 2> /dev/null`;
                        	chop($date_new);
	                        my $flag_tmp = `ps -eafww 2> /dev/null | grep acclog_mon.pl 2> /dev/null | grep $date_new 2> /dev/null | grep -v grep 2> /dev/null`;
        	                unless($flag_tmp){
                	                system("$perl_comm $cwd/acclog_mon.pl --lrms=$lrms --lrms-path=$log_path --date=$date_new --jm-dir=$jm_dir &");
	                        }
	                }
        	}
		$today=`date +%e 2>/dev/null`;
		chop($today);
		if($today ne $day){
			$day=$today;
			threads->new(\&rem_bk);
		}
	}
}



####################################
####           LSF              ####
####################################

elsif($lrms =~ /lsf/i){
	my $day=`date +%e 2>/dev/null`;
	chop($day);
	my $globus_loc_buf=`grep GLOBUS_LOCATION= /etc/globus.conf 2> /dev/null`;
	my ($globus_loc)=($globus_loc_buf=~ /GLOBUS_LOCATION=(.+)/);
	my $command_buff_search=`grep BSUB $globus_loc/libexec/globus-sh-tools-vars.sh 2> /dev/null | sed "s/GLOBUS_SH_BSUB=//g" 2> /dev/null`;
	my ($command_search)=($command_buff_search=~ /"(.+)\/bsub"/);
	unless($command_search=~/\w+/){
		$command_search=`whereis bjobs 2>/dev/null`;
		if($command_search=~/bjobs:\s+(\S+)/){
			my $tmp_command=$1;
			my $ver=`file $1 2>/dev/null`;
			if($ver=~/executable/){
				$command=$tmp_command;
			}
			else{
				$command_search=`which bjobs 2>/dev/null`;
				if($command_search=~/^(\S+qstat)/){
					$command=$1;
				}
				else{$command="bjobs";}
			}
		}
		else{
			$command_search=`which bjobs 2>/dev/null`;
			if($command_search=~/^(\S+bjobs)/){
				$command=$1;
			}
			else{$command="bjobs";}
		}
	}
	else{$command="$command_search/bjobs";}
	my $loops=50;
	my $lsf_log="$jm_dir/acct_window";
	my $flag = `ps -eafww 2>/dev/null | grep parse_lsf 2>/dev/null | grep lrms 2>/dev/null | grep -v grep 2>/dev/null | grep -v defunct 2>/dev/null | wc -l 2>/dev/null | sed 's/\\s*//' 2>/dev/null`;
        if($flag == 0){
        	system("$cwd/parse_lsf -lrms=$log_path -jm-dir=$jm_dir &");
        }
	chop($flag);
	opendir(DIR,"$jm_dir/new/");
	@list_new=readdir(DIR);
	closedir(DIR);
	@list_new=grep /^(\d+)$/,@list_new;
	foreach my $new_q (@list_new){
		my %job_detail=();
		my $test=`grep jobid:$new_q $lsf_log 2>/dev/null`;
		unless($test){
			open(INJ,"<$jm_dir/new/$new_q");
                	my $job_q=<INJ>;
			close(INJ);
			if($job_q=~/jobid:(\d+)\|submitted:(\d+)\|user:"(.+)"\|queue:"(.+)"\|grid_id:(.+)\|/){
				my $vo=`groups $3 2>/dev/null | sed 's/$3 : //g' 2>/dev/null`;
				chop($vo);
				my $queue=$4;
				my $user=$3;
				$job_detail{$vo}{$queue}{$new_q}{"id"}=$new_q;
				$job_detail{$vo}{$queue}{$new_q}{"grid_id"}=$5;
				$job_detail{$vo}{$queue}{$new_q}{"user"}=$3;
				$job_detail{$vo}{$queue}{$new_q}{"qtime"}=$2;
				$job_detail{$vo}{$queue}{$new_q}{"group"}=$vo;
				$job_detail{$vo}{$queue}{$new_q}{"queue"}=$queue;
				$job_detail{$vo}{$queue}{$new_q}{"name"}=$new_q;
				$job_detail{$vo}{$queue}{$new_q}{"end"}=undef;
				$job_detail{$vo}{$queue}{$new_q}{"exit_status"}=undef;
				$job_detail{$vo}{$queue}{$new_q}{"status"}="Q";
				$job_detail{$vo}{$queue}{$new_q}{"exec_host"}="NO_HOST";
				$job_detail{$vo}{$queue}{$new_q}{"start"}=undef;
				$job_detail{$vo}{$queue}{$new_q}{"used_cput"}=undef;
				$job_detail{$vo}{$queue}{$new_q}{"used_walltime"}=undef;
				$job_detail{$vo}{$queue}{$new_q}{"used_mem"}=undef;
				$job_detail{$vo}{$queue}{$new_q}{"used_vmem"}=undef;
				if(-e "$jm_dir/subject/$user"){
					$subject=`cat $jm_dir/subject/$user 2> /dev/null`;
				}
				else{
					my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
					$sbj=~/"(.+)"\s+mapped/;
					$subject=$1;
				}
				$job_detail{$vo}{$queue}{$id}{"subject"}=$subject;
				store(\%job_detail,"$jm_dir/new/$new_q.store");
				unlink("$jm_dir/new/$new_q");
			}
		}
		else{
			if($test=~/submitted:(\d+)\|start:(\d+)\|exec_host:(\d+.+)end:(\d+)\|user:"(.+)"\|queue:"(.+)"\|cputime:(\d+)\|walltime:(\d+)\|memory:(\d+)\|vmemory:(\d+)\|exit_status:(\d+)/){
				my $vo=`groups $5 2>/dev/null | sed 's/$5 : //g' 2>/dev/null`;
				chop($vo);
				my $queue=$6;
				my $user=$5;
				$job_detail{$vo}{$queue}{$new_q}{"id"}=$new_q;
				$job_detail{$vo}{$queue}{$new_q}{"user"}=$5;
				$job_detail{$vo}{$queue}{$new_q}{"qtime"}=$1;
				$job_detail{$vo}{$queue}{$new_q}{"group"}=$vo;
				$job_detail{$vo}{$queue}{$new_q}{"queue"}=$queue;
				$job_detail{$vo}{$queue}{$new_q}{"name"}=$new_q;
				$job_detail{$vo}{$queue}{$new_q}{"end"}=$4;
				$job_detail{$vo}{$queue}{$new_q}{"exit_status"}=$11;
				$job_detail{$vo}{$queue}{$new_q}{"status"}="E";
				$job_detail{$vo}{$queue}{$new_q}{"start"}=$2;
				$job_detail{$vo}{$queue}{$new_q}{"used_cput"}=$7;
				$job_detail{$vo}{$queue}{$new_q}{"used_walltime"}=$8;
				$job_detail{$vo}{$queue}{$new_q}{"used_mem"}=$9;
				$job_detail{$vo}{$queue}{$new_q}{"used_vmem"}=$10;
				my $exh="NO_HOST";
				my $host=$3;
				$host=~/(\d+)(.+)/;
				if($1>0){
					my @tmp_h=split(/\|/,$2);
					$tmp_h[1]=~/"(\S+)"/;
					$exh=$1;
				}
				$job_detail{$vo}{$queue}{$id}{"exec_host"}=$exh;
				if(-e "$jm_dir/subject/$user"){
                                        $subject=`cat $jm_dir/subject/$user 2> /dev/null`;
                                }
                                else{
                                        my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
                                        $sbj=~/"(.+)"\s+mapped/;
                                        $subject=$1;
                                }
                                $job_detail{$vo}{$queue}{$id}{"subject"}=$subject;
				open(INJ,"<$jm_dir/new/$new_q");
				my $job_q=<INJ>;
				close(INJ);
				if($job_q=~/grid_id:(.+)\|/){$job_detail{$vo}{$queue}{$new_q}{"grid_id"}=$1;}
				else{$job_detail{$vo}{$queue}{$new_q}{"grid_id"}="n/a";}
				store(\%job_detail,"$jm_dir/ended/$new_q.store");
				unlink("$jm_dir/new/$new_q");
			}
		}
	}
	
	$mess_lines=`echo -n \`cat /var/log/messages 2>/dev/null | wc -l 2>/dev/null\` 2>/dev/null`;
        while(1){
                my $date = `date +%Y%m%d 2> /dev/null`;
		my $act = `date +%s 2> /dev/null`;
                chop($date);
		chop($act);
		my $tail_c=$mess_lines+1;
                my @lines_tail=`tail -n +$tail_c /var/log/messages 2>/dev/null`;
		push(@lines_tail,@tmp_tail);
		if($#lines_tail eq '-1'){
                        sleep 1;
			my $temp_mess=`echo -n \`cat /var/log/messages 2>/dev/null | wc -l 2>/dev/null\``;
                        if($mess_lines > $temp_mess){
				my $tmp_tail_c=$mess_lines+1;
                                $mess_lines=0;
				@tmp_tail=`tail -n +$tmp_tail_c /var/log/messages.1 2>/dev/null`;
                        }
			else{@tmp_tail=();}
			$loops=50;
			$flag = `ps -eafww 2>/dev/null | grep parse_lsf 2>/dev/null | grep lrms 2>/dev/null | grep -v grep 2>/dev/null | grep -v defunct 2>/dev/null | wc -l 2>/dev/null | sed 's/\\s*//' 2>/dev/null`;
                	if($flag == 0){
                        	system("$cwd/parse_lsf -lrms=$log_path -jm-dir=$jm_dir &");
                	}
                }
                else{
			my $act=`date +%s 2>/dev/null`;
			chop($act);
			my $year=`date +%Y 2>/dev/null`;
			chop($year);
			$year--;
			if($#tmp_tail ne '-1'){
                                $mess_lines=$mess_lines+$#lines_tail-$#tmp_tail;
                        }
			else{$mess_lines=$mess_lines+$#lines_tail+1;}
			@tmp_tail=();
			while($line=shift(@lines_tail)){
				$loops--;
                                if($line=~/Submitted job.+to batch system.+with ID\s+(\d+)/){
                        		%job_detail=();
                                        my $id=$1;
                                        if(-e "$jm_dir/new/$id"){
						my $job_q="";
                                                open(INJ,"<$jm_dir/new/$id");
                                                $job_q=(<INJ>)[0];
                                                close(INJ);
						if($job_q=~/jobid:(\d+)\|submitted:(\d+)\|user:"(.+)"\|queue:"(.+)"\|grid_id:(.+)\|/){
							$vo=`groups $3 2>/dev/null | sed 's/$3 : //g' 2>/dev/null`;
							chop($vo);
							$queue=$4;
							$user=$3;
							$job_detail{$vo}{$queue}{$id}{"id"}=$id;
							$job_detail{$vo}{$queue}{$id}{"grid_id"}=$5;
							$job_detail{$vo}{$queue}{$id}{"user"}=$3;
							$job_detail{$vo}{$queue}{$id}{"qtime"}=$2;
							$job_detail{$vo}{$queue}{$id}{"group"}=$vo;
							$job_detail{$vo}{$queue}{$id}{"queue"}=$queue;
							$job_detail{$vo}{$queue}{$id}{"name"}=$id;
							$job_detail{$vo}{$queue}{$id}{"end"}=undef;
							$job_detail{$vo}{$queue}{$id}{"exit_status"}=undef;
							$job_detail{$vo}{$queue}{$id}{"status"}="Q";
							$job_detail{$vo}{$queue}{$id}{"exec_host"}="NO_HOST";
        	                                        $job_detail{$vo}{$queue}{$id}{"start"}=undef;
                	                                $job_detail{$vo}{$queue}{$id}{"used_cput"}=undef;
                        	                        $job_detail{$vo}{$queue}{$id}{"used_walltime"}=undef;
                                	                $job_detail{$vo}{$queue}{$id}{"used_mem"}=undef;
                                        	        $job_detail{$vo}{$queue}{$id}{"used_vmem"}=undef;
							if(-e "$jm_dir/subject/$user"){
	                                                	$subject=`cat $jm_dir/subject/$user 2> /dev/null`;
        	                                        }
                	                                else{
                        	                        	my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
                                	                        $sbj=~/"(.+)"\s+mapped/;
                                        	                $subject=$1;
                                                	}
	                                                $job_detail{$vo}{$queue}{$id}{"subject"}=$subject;
							store(\%job_detail,"$jm_dir/new/$id.store");
							unlink("$jm_dir/new/$id");
						}
						else{
							unlink("$jm_dir/new/$id");
							my @job_info=`$command -u all -l $id 2>/dev/null | sed 's/^\\s*//g' 2>/dev/null`;
        	                                        if($job_info[1]=~/$id/){
                	                                        my $i=0;
                        	                                my $lsf_bj="";
                                	                        until($job_info[$i]=~/SCHEDULING PARAMETERS/i){
                                        	                        chop($job_info[$i]);
                                                	                $lsf_bj.=$job_info[$i];
                                                        	        $i++;
                                                        	}
	                                                        if($lsf_bj=~/Status <(RUN|PEND)>/){
        	                                                        if($lsf_bj=~/Command .+"(\S+\/globus-cache-export[^\/]+\/)globus-cache-export/){
                	                                                        $grid_id=`grep -h -E 'EDG_WL_JOBID|GLITE_WMS_JOBID' $1* 2>/dev/null | grep https 2>/dev/null | sed 's/\\s*export\\s*//' 2>/dev/null`;
                        	                                                unless($grid_id){
                                	                                                $grid_id=`grep JobWrapper $1* 2>/dev/null | grep https 2>/dev/null`;
                                        	                                        if($grid_id=~/JobWrapper\.(\S+)\.sh/){
                                                	                                        $grid_id=$1;
												$grid_id=~s/_3a/:/g;
												$grid_id=~s/_2f/\//g;
											}
										}
									}
									$lsf_bj=~/User <(.+)>.+Queue <(.+)>/;
									$user=$1;
									$queue=$2;
									$vo=`groups $1 2>/dev/null | sed 's/$1 : //g' 2>/dev/null`;
									chop($vo);
									$job_detail{$vo}{$queue}{$id}{"id"}=$id;
									if($grid_id=~/[EDG_WL_JOBID|GLITE_WMS_JOBID]="(\S+)"/){
										$job_detail{$vo}{$queue}{$id}{"grid_id"}=$1;
									}
									elsif($grid_id=~/https/){
										$job_detail{$vo}{$queue}{$id}{"grid_id"}=$grid_id;
									}
									else{$job_detail{$vo}{$queue}{$id}{"grid_id"}="n/a";}
									$grid_id="";
									$job_detail{$vo}{$queue}{$id}{"user"}=$3;
									$job_detail{$vo}{$queue}{$id}{"group"}=$vo;
									$job_detail{$vo}{$queue}{$id}{"qtime"}=$2;
									$job_detail{$vo}{$queue}{$id}{"queue"}=$queue;
									$job_detail{$vo}{$queue}{$id}{"name"}=$id;
									$job_detail{$vo}{$queue}{$id}{"end"}=undef;
									$job_detail{$vo}{$queue}{$id}{"exit_status"}=undef;
									$job_detail{$vo}{$queue}{$id}{"status"}="Q";
									$job_detail{$vo}{$queue}{$id}{"exec_host"}="NO_HOST";
									$job_detail{$vo}{$queue}{$id}{"start"}=undef;
									$job_detail{$vo}{$queue}{$id}{"used_cput"}=undef;
									$job_detail{$vo}{$queue}{$id}{"used_walltime"}=undef;
									$job_detail{$vo}{$queue}{$id}{"used_mem"}=undef;
									$job_detail{$vo}{$queue}{$id}{"used_vmem"}=undef;
									if(-e "$jm_dir/subject/$user"){
										$subject=`cat $jm_dir/subject/$user 2> /dev/null`;
									}
									else{
										my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
										$sbj=~/"(.+)"\s+mapped/;
										$subject=$1;
									}
									$job_detail{$vo}{$queue}{$id}{"subject"}=$subject;
									store(\%job_detail,"$jm_dir/new/$id.store");
								}
							}
						}
					}
					else{
						my @job_info=`$command -u all -l $id 2>/dev/null | sed 's/^\\s*//g' 2>/dev/null`;
						if($job_info[1]=~/$id/){
							my $i=0;
                                        		my $lsf_bj="";
		                                        until($job_info[$i]=~/SCHEDULING PARAMETERS/i){
                		                                chop($job_info[$i]);
                                		                $lsf_bj.=$job_info[$i];
                                                		$i++;
                                        		}
							$lsf_bj=~s/\s{2,}//g;
							if($lsf_bj=~/Status <(RUN|PEND)>/){
								if($lsf_bj=~/Command .+"(\S+\/globus-cache-export[^\/]+\/)globus/){
									$grid_id=`grep -h -E 'EDG_WL_JOBID|GLITE_WMS_JOBID' $1* 2>/dev/null | grep https 2>/dev/null | sed 's/\\s*export\\s*//' 2>/dev/null`;
									unless($grid_id){
		                                                                $grid_id=`grep JobWrapper $1* 2>/dev/null | grep https 2>/dev/null`;
                		                                                if($grid_id=~/JobWrapper\.(\S+)\.sh/){
                                		                                        $grid_id=$1;
											$grid_id=~s/_3a/:/g;
											$grid_id=~s/_2f/\//g;
										}
									}
								}
								$lsf_bj=~/User <(\S+)>.+Queue <(\S+)>/;
								$queue=$2;
								$vo=`groups $1 2>/dev/null | sed 's/$1 : //g' 2>/dev/null`;
								$user=$1;
								chop($vo);
								$job_detail{$vo}{$queue}{$id}{"id"}=$id;
								if($grid_id=~/[EDG_WL_JOBID|GLITE_WMS_JOBID]="(\S+)"/){
									$job_detail{$vo}{$queue}{$id}{"grid_id"}=$1;
								}
								elsif($grid_id=~/https/){
									$job_detail{$vo}{$queue}{$id}{"grid_id"}=$grid_id;
								}
								else{$job_detail{$vo}{$queue}{$id}{"grid_id"}="n/a";}
								$grid_id="";
								$lsf_bj=~/(\w+ \w+ \S+): Submitted from host/;
								$job_detail{$vo}{$queue}{$id}{"qtime"}=`date -d "$1" +%s 2>/dev/null`;
								chop($job_detail{$vo}{$queue}{$id}{"qtime"});
								if($job_detail{$vo}{$queue}{$id}{"qtime"} > $act){
									$job_detail{$vo}{$queue}{$id}{"qtime"}=`date -d "$1 $year" +%s 2>/dev/null`;
									chop($job_detail{$vo}{$queue}{$id}{"qtime"});
								}
								$job_detail{$vo}{$queue}{$id}{"user"}=$user;
								$job_detail{$vo}{$queue}{$id}{"group"}=$vo;
								$job_detail{$vo}{$queue}{$id}{"queue"}=$queue;
								$job_detail{$vo}{$queue}{$id}{"name"}=$id;
								$job_detail{$vo}{$queue}{$id}{"end"}=undef;
								$job_detail{$vo}{$queue}{$id}{"exit_status"}=undef;
								$job_detail{$vo}{$queue}{$id}{"status"}="Q";
								$job_detail{$vo}{$queue}{$id}{"exec_host"}="NO_HOST";
								$job_detail{$vo}{$queue}{$id}{"start"}=undef;
								$job_detail{$vo}{$queue}{$id}{"used_cput"}=undef;
								$job_detail{$vo}{$queue}{$id}{"used_walltime"}=undef;
								$job_detail{$vo}{$queue}{$id}{"used_mem"}=undef;
								$job_detail{$vo}{$queue}{$id}{"used_vmem"}=undef;
								if(-e "$jm_dir/subject/$user"){
									$subject=`cat $jm_dir/subject/$user 2> /dev/null`;
								}
								else{
									my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
									$sbj=~/"(.+)"\s+mapped/;
									$subject=$1;
								}
								$job_detail{$vo}{$queue}{$id}{"subject"}=$subject;
								store(\%job_detail,"$jm_dir/new/$id.store");
							}
						}
					}
                                }
                                elsif($line =~ /\(ID\s+(\d+).+has finished/){
                        		%job_detail=();
                                        my $id=$1;
                                        my $ln=`grep "jobid:$id" $lsf_log 2> /dev/null`;
					$ln=`grep "jobid:$id" $jm_dir/acct_output 2> /dev/null` unless($ln);
                                        if($ln=~/submitted:(\d+)\|start:(\d+)\|exec_host:(\d+.+)end:(\d+)\|user:"(\w+)"\|queue:"(\w+)"\|cputime:(\d+)\|walltime:(\d+)\|memory:(\d+)\|vmemory:(\d+)\|exit_status:(\d+)/){
						$vo=`groups $5 2>/dev/null | sed 's/$5 : //g' 2>/dev/null`;
						chop($vo);
						$queue=$6;
						$user=$5;
                                                $job_detail{$vo}{$queue}{$id}{"status"}="E";
                                                $job_detail{$vo}{$queue}{$id}{"id"}=$id;
                                                $job_detail{$vo}{$queue}{$id}{"user"}=$5;
                                                if(-e "$jm_dir/new/$id.store"){
                                                        my $ref=retrieve("$jm_dir/new/$id.store");
                                                        $job_detail{$vo}{$queue}{$id}{"grid_id"}=$ref->{$vo}{$queue}{$id}{"grid_id"};
                                                        unlink("$jm_dir/new/$id.store");
                                                }
						elsif(-e "$jm_dir/new/$id"){
							$job_detail{$vo}{$queue}{$id}{"grid_id"}=`echo -n \`cat $jm_dir/new/$id 2>/dev/null | sed 's/.*grid_id://' 2>/dev/null\` 2>/dev/null`;
							chop($job_detail{$vo}{$queue}{$id}{"grid_id"});
							unlink("$jm_dir/new/$id");
						}
                                                else{$job_detail{$vo}{$queue}{$id}{"grid_id"}="n/a";}
                                                $job_detail{$vo}{$queue}{$id}{"group"}=$vo;
                                                $job_detail{$vo}{$queue}{$id}{"name"}=$id;
                                                $job_detail{$vo}{$queue}{$id}{"queue"}=$6;
                                                $job_detail{$vo}{$queue}{$id}{"qtime"}=$1;
                                                $job_detail{$vo}{$queue}{$id}{"start"}=$2;
                                                $job_detail{$vo}{$queue}{$id}{"end"}=$4;
						if($11==64){
                                                        $job_detail{$vo}{$queue}{$id}{"exit_status"}=0;
                                                }
                                                else{$job_detail{$vo}{$queue}{$id}{"exit_status"}=$11;}
                                                $job_detail{$vo}{$queue}{$id}{"used_cput"}=$7;
                                                $job_detail{$vo}{$queue}{$id}{"used_mem"}=$9;
                                                $job_detail{$vo}{$queue}{$id}{"used_vmem"}=$10;
                                                $job_detail{$vo}{$queue}{$id}{"used_walltime"}=$8;
						my $exh="NO_HOST";
						my $host=$3;
						$host=~/(\d+)(.+)/;
			                        if($1>0){
                        			        my @tmp_h=split(/\|/,$2);
			                                $tmp_h[1]=~/"(\S+)"/;
                        			        $exh=$1;
			                        }
                        			$job_detail{$vo}{$queue}{$id}{"exec_host"}=$exh;
                                                if(-e "$jm_dir/subject/$user"){
                                                        $job_detail{$vo}{$queue}{$id}{"subject"}=`cat $jm_dir/subject/$user 2> /dev/null`;
                                                }
                                                else{
                                                        my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
                                                        $sbj=~/"(.+)"\s+mapped/;
                                                        $job_detail{$vo}{$queue}{$id}{"subject"}=$1;
                                                }
						store(\%job_detail,"$jm_dir/ended/$id.store");
                                        }
					else{
						my $bacct_comm=$command;
						$bacct_comm=~s/bjobs$/bacct/g;
                                                my @bacct=`$bacct_comm -u all -l $id 2>/dev/null`;
                                                map {s/\n|^\s*//g} @bacct;
                                                my $bacct=join("",@bacct);
						if($bacct=~/User <(\S+)>.+Queue <(\w+)>.+\s+(\w+\s+\w+\s+\S+): Submitted from host.+\s+(\w+\s+\w+\s+\S+): Dispatched to \d*\s*\S*\s*<(\S+)>.+\s+(\w+\s+\w+\s+\S+): Completed.+CPU_T.+SWAP(\d+)\.\d*\s+\d*\s+(\d+)\s+\w+\s+\S+\s+(\w+)\s+(\w+)/){
                                                        $user=$1;
                                                        $queue=$2;
                                                        $vo=`groups $1 2>/dev/null | sed 's/$1 : //g' 2>/dev/null`;
                                                        chop($vo);
                                                        $job_detail{$vo}{$queue}{$id}{"id"}=$id;
                                                        $job_detail{$vo}{$queue}{$id}{"queue"}=$queue;
                                                        $job_detail{$vo}{$queue}{$id}{"group"}=$vo;
                                                        $job_detail{$vo}{$queue}{$id}{"user"}=$user;
                                                        $job_detail{$vo}{$queue}{$id}{"qtime"}=`date -d "$3" +%s 2>/dev/null`;
                                                        chop($job_detail{$vo}{$queue}{$id}{"qtime"});
							if($job_detail{$vo}{$queue}{$id}{"qtime"} > $act){
                                                                $job_detail{$vo}{$queue}{$id}{"qtime"}=`date -d "$3 $year" +%s 2>/dev/null`;
                                                                chop($job_detail{$vo}{$queue}{$id}{"qtime"});
                                                        }
                                                        $job_detail{$vo}{$queue}{$id}{"start"}=`date -d "$4" +%s 2>/dev/null`;
                                                        chop($job_detail{$vo}{$queue}{$id}{"start"});
							if($job_detail{$vo}{$queue}{$id}{"start"} > $act){
                                                                $job_detail{$vo}{$queue}{$id}{"start"}=`date -d "$4 $year" +%s 2>/dev/null`;
                                                                chop($job_detail{$vo}{$queue}{$id}{"start"});
                                                        }
                                                        $job_detail{$vo}{$queue}{$id}{"exec_host"}=$5;
							$job_detail{$vo}{$queue}{$id}{"end"}=`date -d "$6" +%s 2>/dev/null`;
						        chop($job_detail{$vo}{$queue}{$id}{"end"});
							if($job_detail{$vo}{$queue}{$id}{"end"} > $act){
                                                                $job_detail{$vo}{$queue}{$id}{"end"}=`date -d "$6 $year" +%s 2>/dev/null`;
                                                                chop($job_detail{$vo}{$queue}{$id}{"end"});
                                                        }
                                                        $job_detail{$vo}{$queue}{$id}{"used_cput"}=$7;
							if($job_detail{$vo}{$queue}{$id}{"start"}>0){
								$job_detail{$vo}{$queue}{$id}{"used_walltime"}=$job_detail{$vo}{$queue}{$id}{"end"}-$job_detail{$vo}{$queue}{$id}{"start"};
							}
							else{$job_detail{$vo}{$queue}{$id}{"used_walltime"}=0;}
							my $used_mem=$9;
							my $used_vmem=$10;
							$used_mem=~s/0K/0/;
							$used_mem=~s/(\d+)M/$1/;
							$used_vmem=~s/0K/0/;
							$used_vmem=~s/(\d+)M/$1/;
							$job_detail{$vo}{$queue}{$id}{"used_mem"}=$used_mem*1024;
                                                        $job_detail{$vo}{$queue}{$id}{"used_vmem"}=$used_vmem*1024;
                                                        if(-e "$jm_dir/new/$id.store"){
                                                                my %ref=%{retrieve("$jm_dir/new/$id.store")};
                                                                $job_detail{$vo}{$queue}{$id}{"grid_id"}=$ref{$vo}{$queue}{$id}{"grid_id"};
                                                                unlink("$jm_dir/new/$id.store");
                                                        }
							elsif(-e "$jm_dir/new/$id"){
								$job_detail{$vo}{$queue}{$id}{"grid_id"}=`echo -n \`cat $jm_dir/new/$id 2>/dev/null | sed 's/.*grid_id://' 2>/dev/null\` 2>/dev/null`;
								chop($job_detail{$vo}{$queue}{$id}{"grid_id"});
								unlink("$jm_dir/new/$id");
							}
                                                        else{$job_detail{$vo}{$queue}{$id}{"grid_id"}="n/a";}
							if($bacct=~/Completed <done>/){
								$job_detail{$vo}{$queue}{$id}{"exit_status"}=0;
							}
							elsif($bacct=~/Completed <(\S+)>/){
								$job_detail{$vo}{$queue}{$id}{"exit_status"}=$1;
							}
							else{$job_detail{$vo}{$queue}{$id}{"exit_status"}=271;}
							$job_detail{$vo}{$queue}{$id}{"status"}="E";
							if(-e "$jm_dir/subject/$user"){
								$job_detail{$vo}{$queue}{$id}{"subject"}=`cat $jm_dir/subject/$user 2> /dev/null`;
							}
							else{
								my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
								$sbj=~/"(.+)"\s+mapped/;
								$job_detail{$vo}{$queue}{$id}{"subject"}=$1;
							}
							$job_detail{$vo}{$queue}{$id}{"name"}=$id;
							store(\%job_detail,"$jm_dir/ended/$id.store");
						}
						elsif($bacct=~/User <(\S+)>.+Queue <(\w+)>.+\s+(\w+\s+\w+\s+\S+): Submitted from host.+\s+(\w+\s+\w+\s+\S+): Completed.+CPU_T.+SWAP(\d+)\.\d*\s+\d*\s+(\d+)\s+\w+\s+\S+\s+(\w+)\s+(\w+)/){
							$user=$1;
							$queue=$2;
							$vo=`groups $1 2>/dev/null | sed 's/$1 : //g' 2>/dev/null`;
							chop($vo);
							$job_detail{$vo}{$queue}{$id}{"id"}=$id;
							$job_detail{$vo}{$queue}{$id}{"queue"}=$queue;
							$job_detail{$vo}{$queue}{$id}{"group"}=$vo;
							$job_detail{$vo}{$queue}{$id}{"user"}=$user;
							$job_detail{$vo}{$queue}{$id}{"qtime"}=`date -d "$3" +%s 2>/dev/null`;
							chop($job_detail{$vo}{$queue}{$id}{"qtime"});
							if($job_detail{$vo}{$queue}{$id}{"qtime"} > $act){
                                                                $job_detail{$vo}{$queue}{$id}{"qtime"}=`date -d "$3 $year" +%s 2>/dev/null`;
                                                                chop($job_detail{$vo}{$queue}{$id}{"qtime"});
                                                        }
							$job_detail{$vo}{$queue}{$id}{"start"}=0;
							$job_detail{$vo}{$queue}{$id}{"exec_host"}="NO_HOST";
							$job_detail{$vo}{$queue}{$id}{"end"}=`date -d "$4" +%s 2>/dev/null`;
							chop($job_detail{$vo}{$queue}{$id}{"end"});
							if($job_detail{$vo}{$queue}{$id}{"end"} > $act){
                                                                $job_detail{$vo}{$queue}{$id}{"end"}=`date -d "$4 $year" +%s 2>/dev/null`;
                                                                chop($job_detail{$vo}{$queue}{$id}{"end"});
                                                        }
							$job_detail{$vo}{$queue}{$id}{"used_cput"}=$5;
							if($job_detail{$vo}{$queue}{$id}{"start"}>0){
								$job_detail{$vo}{$queue}{$id}{"used_walltime"}=$job_detail{$vo}{$queue}{$id}{"end"}-$job_detail{$vo}{$queue}{$id}{"start"};
							}
							else{$job_detail{$vo}{$queue}{$id}{"used_walltime"}=0;}
							my $used_mem=$7;
							my $used_vmem=$8;
							$used_mem=~s/0K/0/;
							$used_mem=~s/(\d+)M/$1/;
							$used_vmem=~s/0K/0/;
							$used_vmem=~s/(\d+)M/$1/;
							$job_detail{$vo}{$queue}{$id}{"used_mem"}=$used_mem*1024;
							$job_detail{$vo}{$queue}{$id}{"used_vmem"}=$used_vmem*1024;
							if(-e "$jm_dir/new/$id.store"){
								my %ref=%{retrieve("$jm_dir/new/$id.store")};
								$job_detail{$vo}{$queue}{$id}{"grid_id"}=$ref{$vo}{$queue}{$id}{"grid_id"};
								unlink("$jm_dir/new/$id.store");
							}
							else{$job_detail{$vo}{$queue}{$id}{"grid_id"}="n/a";}
							if($bacct=~/Completed <done>/){
								$job_detail{$vo}{$queue}{$id}{"exit_status"}=0;
							}
							elsif(`grep '$id:id_"root"' $jm_dir/killed* 2>/dev/null`){
								$job_detail{$vo}{$queue}{$id}{"exit_status"}=201;
							}
							else{$job_detail{$vo}{$queue}{$id}{"exit_status"}=200;}
							$job_detail{$vo}{$queue}{$id}{"status"}="E";
							if(-e "$jm_dir/subject/$user"){
								$job_detail{$vo}{$queue}{$id}{"subject"}=`cat $jm_dir/subject/$user 2> /dev/null`;
							}
							else{
								my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
								$sbj=~/"(.+)"\s+mapped/;
								$job_detail{$vo}{$queue}{$id}{"subject"}=$1;
							}
							$job_detail{$vo}{$queue}{$id}{"name"}=$id;
							store(\%job_detail,"$jm_dir/ended/$id.store");
						}

					}
                                }
                                elsif($line =~ /batch ID\s+(\d+).+REMOVED from batch system ok/){
                        		%job_detail=();
                                        my $id=$1;
                                        my $ln=`grep "jobid:$id" $lsf_log 2> /dev/null`;
                                        $ln=`grep "jobid:$id" $jm_dir/acct_output 2> /dev/null` unless($ln);
					if($ln=~/submitted:(\d+)\|start:(\d+)\|exec_host:(\d+.+)end:(\d+)\|user:"(\w+)"\|queue:"(\w+)"\|cputime:(\d+)\|walltime:(\d+)\|memory:(\d+)\|vmemory:(\d+)\|exit_status:(\d+)/){
                                                $vo=`groups $5 2>/dev/null | sed 's/$5 : //g' 2>/dev/null`;
						chop($vo);
                                                $queue=$6;
                                                $user=$5;
						my $exit_status=$11;
                                                $job_detail{$vo}{$queue}{$id}{"status"}="E";
                                                $job_detail{$vo}{$queue}{$id}{"id"}=$id;
                                                $job_detail{$vo}{$queue}{$id}{"user"}=$5;
                                                if(-e "$jm_dir/new/$id.store"){
                                                        my $ref=retrieve("$jm_dir/new/$id.store");
                                                        $job_detail{$vo}{$queue}{$id}{"grid_id"}=$ref->{$vo}{$queue}{$id}{"grid_id"};	
                                                        unlink("$jm_dir/new/$id.store");
                                                }
						elsif(-e "$jm_dir/new/$id"){
                                                        $job_detail{$vo}{$queue}{$id}{"grid_id"}=`echo -n \`cat $jm_dir/new/$id 2>/dev/null | sed 's/.*grid_id://' 2>/dev/null\` 2>/dev/null`;
							chop($job_detail{$vo}{$queue}{$id}{"grid_id"});
                                                        unlink("$jm_dir/new/$id");
                                                }
                                                else{$job_detail{$vo}{$queue}{$id}{"grid_id"}="n/a";}
                                                $job_detail{$vo}{$queue}{$id}{"group"}=$vo;
                                                $job_detail{$vo}{$queue}{$id}{"name"}=$id;
                                                $job_detail{$vo}{$queue}{$id}{"queue"}=$6;
                                                $job_detail{$vo}{$queue}{$id}{"qtime"}=$1;
                                                $job_detail{$vo}{$queue}{$id}{"start"}=$2;
                                                $job_detail{$vo}{$queue}{$id}{"end"}=$4;
                                                $job_detail{$vo}{$queue}{$id}{"used_cput"}=$7;
                                                $job_detail{$vo}{$queue}{$id}{"used_mem"}=$9;
                                                $job_detail{$vo}{$queue}{$id}{"used_vmem"}=$10;
						$job_detail{$vo}{$queue}{$id}{"used_walltime"}=$8;
                                                my $exh="NO_HOST";
						my $host=$3;
                                                $host=~/(\d+)(.+)/;
                                                if($1>0){
                                                        my @tmp_h=split(/\|/,$2);
                                                        $tmp_h[1]=~/"(\S+)"/;
                                                        $exh=$1;
                                                }
                                                $job_detail{$vo}{$queue}{$id}{"exec_host"}=$exh;
                                                if(-e "$jm_dir/subject/$user"){
                                                        $job_detail{$vo}{$queue}{$id}{"subject"}=`cat $jm_dir/subject/$user 2> /dev/null`;
                                                }
                                                else{
                                                        my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
                                                        $sbj=~/"(.+)"\s+mapped/;
                                                        $job_detail{$vo}{$queue}{$id}{"subject"}=$1;
                                                }
						my $remover=`grep id_$id $jm_dir/killed* 2>/dev/null`;
						if($remover=~/root/){
                                                	$job_detail{$vo}{$queue}{$id}{"exit_status"}=201;
						}
						elsif($remover=~/$user/){
							$job_detail{$vo}{$queue}{$id}{"exit_status"}=200;
						}
						else{$job_detail{$vo}{$queue}{$id}{"exit_status"}=$exit_status;}
                                                store(\%job_detail,"$jm_dir/ended/$id.store");
                                        }
					else{
						my $bacct_comm=$command;
						$bacct_comm=~s/bjobs$/bacct/g;
                                                my @bacct=`$bacct_comm -u all -l $id 2>/dev/null`;
                                                map {s/\n|^\s*//g} @bacct;
                                                my $bacct=join("",@bacct);
						if($bacct=~/User <(\S+)>.+Queue <(\w+)>.+\s+(\w+\s+\w+\s+\S+): Submitted from host.+\s+(\w+\s+\w+\s+\S+): Dispatched to \d*\s*\S*\s*<(\S+)>.+\s+(\w+\s+\w+\s+\S+): Completed.+CPU_T.+SWAP(\d+)\.\d*\s+\d*\s+(\d+)\s+\w+\s+\S+\s+(\w+)\s+(\w+)/){
                                                        $user=$1;
                                                        $queue=$2;
                                                        $vo=`groups $1 2>/dev/null | sed 's/$1 : //g' 2>/dev/null`;
                                                        chop($vo);
                                                        $job_detail{$vo}{$queue}{$id}{"id"}=$id;
                                                        $job_detail{$vo}{$queue}{$id}{"queue"}=$queue;
                                                        $job_detail{$vo}{$queue}{$id}{"group"}=$vo;
                                                        $job_detail{$vo}{$queue}{$id}{"user"}=$user;
                                                        $job_detail{$vo}{$queue}{$id}{"qtime"}=`date -d "$3" +%s 2>/dev/null`;
                                                        chop($job_detail{$vo}{$queue}{$id}{"qtime"});
							if($job_detail{$vo}{$queue}{$id}{"qtime"} > $act){
                                                                $job_detail{$vo}{$queue}{$id}{"qtime"}=`date -d "$3 $year" +%s 2>/dev/null`;
                                                                chop($job_detail{$vo}{$queue}{$id}{"qtime"});
                                                        }
                                                        $job_detail{$vo}{$queue}{$id}{"start"}=`date -d "$4" +%s 2>/dev/null`;
                                                        chop($job_detail{$vo}{$queue}{$id}{"start"});
							if($job_detail{$vo}{$queue}{$id}{"start"} > $act){
                                                                $job_detail{$vo}{$queue}{$id}{"start"}=`date -d "$4 $year" +%s 2>/dev/null`;
                                                                chop($job_detail{$vo}{$queue}{$id}{"start"});
                                                        }
                                                        $job_detail{$vo}{$queue}{$id}{"exec_host"}=$5;
							$job_detail{$vo}{$queue}{$id}{"end"}=`date -d "$6" +%s 2>/dev/null`;
                                                        chop($job_detail{$vo}{$queue}{$id}{"end"});
							if($job_detail{$vo}{$queue}{$id}{"end"} > $act){
                                                                $job_detail{$vo}{$queue}{$id}{"end"}=`date -d "$6 $year" +%s 2>/dev/null`;
                                                                chop($job_detail{$vo}{$queue}{$id}{"end"});
                                                        }
                                                        $job_detail{$vo}{$queue}{$id}{"used_cput"}=$7;
							if($job_detail{$vo}{$queue}{$id}{"start"}>0){
                                                                $job_detail{$vo}{$queue}{$id}{"used_walltime"}=$job_detail{$vo}{$queue}{$id}{"end"}-$job_detail{$vo}{$queue}{$id}{"start"};
                                                        }
							else{$job_detail{$vo}{$queue}{$id}{"used_walltime"}=0;}
							my $used_mem=$9;
                                                        my $used_vmem=$10;
                                                        $used_mem=~s/0K/0/;
                                                        $used_mem=~s/(\d+)M/$1/;
                                                        $used_vmem=~s/0K/0/;
                                                        $used_vmem=~s/(\d+)M/$1/;
                                                        $job_detail{$vo}{$queue}{$id}{"used_mem"}=$used_mem*1024;
                                                        $job_detail{$vo}{$queue}{$id}{"used_vmem"}=$used_vmem*1024;
                                                        if(-e "$jm_dir/new/$id.store"){
                                                                my %ref=%{retrieve("$jm_dir/new/$id.store")};
                                                                $job_detail{$vo}{$queue}{$id}{"grid_id"}=$ref{$vo}{$queue}{$id}{"grid_id"};
                                                                unlink("$jm_dir/new/$id.store");
                                                        }
                                                        else{$job_detail{$vo}{$queue}{$id}{"grid_id"}="n/a";}
                                                        if($bacct=~/Completed <done>/){
                                                                $job_detail{$vo}{$queue}{$id}{"exit_status"}=0;
                                                        }
                                                        elsif(`grep '$id:id_"root"' $jm_dir/killed* 2>/dev/null`){
                                                                $job_detail{$vo}{$queue}{$id}{"exit_status"}=201;
                                                        }
                                                        else{$job_detail{$vo}{$queue}{$id}{"exit_status"}=200;}
                                                        $job_detail{$vo}{$queue}{$id}{"status"}="E";
							if(-e "$jm_dir/subject/$user"){
                                                                $job_detail{$vo}{$queue}{$id}{"subject"}=`cat $jm_dir/subject/$user 2> /dev/null`;
                                                        }
                                                        else{
                                                                my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
                                                                $sbj=~/"(.+)"\s+mapped/;
                                                                $job_detail{$vo}{$queue}{$id}{"subject"}=$1;
                                                        }
                                                        $job_detail{$vo}{$queue}{$id}{"name"}=$id;
                                                        store(\%job_detail,"$jm_dir/ended/$id.store");
                                                }
						elsif($bacct=~/User <(\S+)>.+Queue <(\w+)>.+\s+(\w+\s+\w+\s+\S+): Submitted from host.+\s+(\w+\s+\w+\s+\S+): Completed.+CPU_T.+SWAP(\d+)\.\d*\s+\d*\s+(\d+)\s+\w+\s+\S+\s+(\w+)\s+(\w+)/){
                                                        $user=$1;
                                                        $queue=$2;
                                                        $vo=`groups $1 2>/dev/null | sed 's/$1 : //g' 2>/dev/null`;
                                                        chop($vo);
                                                        $job_detail{$vo}{$queue}{$id}{"id"}=$id;
                                                        $job_detail{$vo}{$queue}{$id}{"queue"}=$queue;
                                                        $job_detail{$vo}{$queue}{$id}{"group"}=$vo;
                                                        $job_detail{$vo}{$queue}{$id}{"user"}=$user;
                                                        $job_detail{$vo}{$queue}{$id}{"qtime"}=`date -d "$3" +%s 2>/dev/null`;
                                                        chop($job_detail{$vo}{$queue}{$id}{"qtime"});
							if($job_detail{$vo}{$queue}{$id}{"qtime"} > $act){
                                                                $job_detail{$vo}{$queue}{$id}{"qtime"}=`date -d "$3 $year" +%s 2>/dev/null`;
                                                                chop($job_detail{$vo}{$queue}{$id}{"qtime"});
                                                        }
                                                        $job_detail{$vo}{$queue}{$id}{"start"}=0;
                                                        $job_detail{$vo}{$queue}{$id}{"exec_host"}="NO_HOST";
                                                        $job_detail{$vo}{$queue}{$id}{"end"}=`date -d "$4" +%s 2>/dev/null`;
                                                        chop($job_detail{$vo}{$queue}{$id}{"end"});
							if($job_detail{$vo}{$queue}{$id}{"end"} > $act){
								$job_detail{$vo}{$queue}{$id}{"end"}=`date -d "$4 $year" +%s 2>/dev/null`;
								chop($job_detail{$vo}{$queue}{$id}{"end"});
							}
							$job_detail{$vo}{$queue}{$id}{"used_cput"}=$5;
                                                        if($job_detail{$vo}{$queue}{$id}{"start"}>0){
                                                                $job_detail{$vo}{$queue}{$id}{"used_walltime"}=$job_detail{$vo}{$queue}{$id}{"end"}-$job_detail{$vo}{$queue}{$id}{"start"};
                                                        }
                                                        else{$job_detail{$vo}{$queue}{$id}{"used_walltime"}=0;}
                                                        my $used_mem=$7;
                                                        my $used_vmem=$8;
                                                        $used_mem=~s/0K/0/;
                                                        $used_mem=~s/(\d+)M/$1/;
                                                        $used_vmem=~s/0K/0/;
                                                        $used_vmem=~s/(\d+)M/$1/;
                                                        $job_detail{$vo}{$queue}{$id}{"used_mem"}=$used_mem*1024;
                                                        $job_detail{$vo}{$queue}{$id}{"used_vmem"}=$used_vmem*1024;
                                                        if(-e "$jm_dir/new/$id.store"){
                                                                my %ref=%{retrieve("$jm_dir/new/$id.store")};
                                                                $job_detail{$vo}{$queue}{$id}{"grid_id"}=$ref{$vo}{$queue}{$id}{"grid_id"};
                                                                unlink("$jm_dir/new/$id.store");
                                                        }
                                                        else{$job_detail{$vo}{$queue}{$id}{"grid_id"}="n/a";}
                                                        if($bacct=~/Completed <done>/){
                                                                $job_detail{$vo}{$queue}{$id}{"exit_status"}=0;
                                                        }
							elsif(`grep '$id:id_"root"' $jm_dir/killed* 2>/dev/null`){
                                                                $job_detail{$vo}{$queue}{$id}{"exit_status"}=201;
                                                        }
                                                        else{$job_detail{$vo}{$queue}{$id}{"exit_status"}=200;}
                                                        $job_detail{$vo}{$queue}{$id}{"status"}="E";
                                                        if(-e "$jm_dir/subject/$user"){
                                                                $job_detail{$vo}{$queue}{$id}{"subject"}=`cat $jm_dir/subject/$user 2> /dev/null`;
                                                        }
                                                        else{
                                                                my $sbj=`grep $user /var/log/globus-gatekeeper.log 2> /dev/null |grep mapped 2> /dev/null | grep "Notice:" 2> /dev/null |tail -n 1 2> /dev/null | sed "s/Notice: 5: //g" 2> /dev/null`;
                                                                $sbj=~/"(.+)"\s+mapped/;
                                                                $job_detail{$vo}{$queue}{$id}{"subject"}=$1;
                                                        }
                                                        $job_detail{$vo}{$queue}{$id}{"name"}=$id;
                                                        store(\%job_detail,"$jm_dir/ended/$id.store");
                                                }
                                        }
                                }
				elsif($line =~ /GRAM.+: "(.+)" mapped to (\w+) /){
                                	my $user_info=$1;
	                                my $loc_acc=$2;
        	                        if(($job_mon{$loc_acc} ne $user_info) or !defined($job_mon{$loc_acc}) or !(-e "$jm_dir/subject/$loc_acc")){
                	                        open(OUT,">$jm_dir/subject/$loc_acc");
                        	                print OUT "$user_info";
                                	        close(OUT);
                                        	$job_mon{$loc_acc}=$user_info;
					}
                                }
                	}
			if($loops==0){
				$loops=50;
				my $flag = `ps -eafww 2>/dev/null | grep parse_lsf 2>/dev/null | grep lrms 2>/dev/null | grep -v grep 2>/dev/null | grep -v defunct 2>/dev/null | wc -l 2>/dev/null | sed 's/\\s*//' 2>/dev/null`;
                		if($flag == 0){
                        		system("$cwd/parse_lsf -lrms=$log_path -jm-dir=$jm_dir &");
                		}
			}
			$today=`date +%e 2>/dev/null`;
			chop($today);
			if($today ne $day){
				$day=$today;
				threads->new(\&rem_bk);
			}
			sleep 3;
                }
        }
}
